//------------------------------------------------------------------------------
// (c) 05-2001 Gottfried Chen
//------------------------------------------------------------------------------

#ifndef APPLICATION_COMMAND_H
#define APPLICATION_COMMAND_H

#include <System/Object.h>
#include <string>

namespace Ge
{

// A Command is a code segment, that can be executed. Commands can be
// read from .set files or be bound to keys, triggered by timers,...
// How to add a new command:
// - Derive your command class from Command
// - Write a constructor that interprets the parameters
// - Override execute()
// - Add it to the Command::create(), if it is read from .set-files
class Command : public Object
{
public:
    // "Factory" function used by CommandLine class
    static
    Command* create(const char* command, const char* parameters = 0);

    // If you want an output string, then provide a <consoleOutput> pointer
    virtual
    void execute(std::string* consoleOutput = 0) = 0;

    const std::string& getString() const;

protected:
    Command(const char* command);

private:
    std::string mString;
};

} // namespace Ge

#endif // APPLICATION_COMMAND_H