#include "Precompiled.h"
//------------------------------------------------------------------------------
// (c) 05-2001 Gottfried Chen
//------------------------------------------------------------------------------

#include "CommandLine.h"
#include "Command.h"

#include <System/StringTools.h>

namespace Ge
{

//------------------------------------------------------------------------------
CommandLine::CommandLine(const char* line) :
//------------------------------------------------------------------------------
mComment(false),
mCommand(0)
{
    GE_ASSERT(line != 0);
    if (line != 0)
    {
        std::string buffer(line);
        if (buffer.size() != 0)
        {    
            // Check for comment or blank line
            if (buffer.find("//") != std::string::npos ||
                StringTools::firstNonWhiteSpace(buffer) == -1)
                mComment = true;

            // Split into command and parameters
            if (!mComment)
            {
                std::string command;
                std::string parameters;
                StringTools::split(buffer, command, parameters);
                mCommand = Command::create(command.c_str(), parameters.c_str());
            }
        }
    }
}

//------------------------------------------------------------------------------
bool CommandLine::isComment() const
//------------------------------------------------------------------------------
{
    return mComment;
}

//------------------------------------------------------------------------------
bool CommandLine::isValid() const
//------------------------------------------------------------------------------
{
    return mCommand != 0;
}

//------------------------------------------------------------------------------
void CommandLine::execute(std::string* consoleOutput) const
//------------------------------------------------------------------------------
{
    if (mCommand != 0)
        mCommand->execute(consoleOutput);
    else if (consoleOutput != 0)
        *consoleOutput = "Command is invalid";
}

//------------------------------------------------------------------------------
CommandLine::~CommandLine()
//------------------------------------------------------------------------------
{
    if (mCommand != 0)
    {
        GE_DELETE mCommand;
    }
}

} // namespace Ge