//------------------------------------------------------------------------------
// (c) 02-2002 Gottfried Chen
//------------------------------------------------------------------------------

#ifndef CONTEXT_H
#define CONTEXT_H

#include <string>

namespace Ge
{

class Context
{
public:
    Context(const char* name);

    virtual void init() = 0;

    // Update this Context's inner state
    virtual void update() = 0;

    // Render the Context.
    // the Renderer().beginScene() has already been called.
    // the Renderer().endScene() will be called by the Framework.
    virtual void render() = 0;

    // Do stuff before being paused (i.e. switched to another Context).
    virtual void pause();

    // Do stuff after you are resumed (i.e. become the active Context again).
    virtual void resume();

    virtual void exit() = 0;
    
    const std::string& getName() const;

protected:
    void setName(const char* name);

private:
    std::string mName;
//    InputContext mInput;
};

inline
//------------------------------------------------------------------------------
const std::string& Context::getName() const
//------------------------------------------------------------------------------
{
    return mName;
}

} // namespace Ge

#endif // CONTEXT_H
