#include "Precompiled.h"
//------------------------------------------------------------------------------
// (c) 05-2001 Gottfried Chen
//------------------------------------------------------------------------------

#include "Framework.h"
#include "Metronome.h"
#include "ScaledMetronome.h"
#include "Settings.h"
#include "Context.h"

#include <Engine/DisplayStatistics.h>
#include <Engine/ModelPool.h>

#include <DirectXRenderer/DirectXRenderer.h>
#include <DirectXRenderer/DirectXTexturePool.h>

#include <Input/Mouse.h>
#include <Input/Keyboard.h>

#include <System/Log.h>

namespace Ge
{

//------------------------------------------------------------------------------
Framework::Framework(const char* settingsFileName) :
//------------------------------------------------------------------------------
mWindow(0),
mActiveContext(0)
{
    if (settingsFileName != 0)
        mSettingsName = settingsFileName;
}

//------------------------------------------------------------------------------
Framework::~Framework()
//------------------------------------------------------------------------------
{
}

//------------------------------------------------------------------------------
void Framework::init()
//------------------------------------------------------------------------------
{
    // Load settings
    if (mSettingsName != "")
        Settings::instance(mSettingsName.c_str());

    // Start game time
    theMetronome();

    // Start log
    Log::global();

    // Open window and renderer
    mWindow = createWindow();
    GE_ASSERT(mWindow != 0);

    Renderer::instancePtr() = GE_NEW DirectXRenderer(mWindow);
    
    TexturePool::instancePtr() =
        GE_NEW DirectXTexturePool(*(static_cast<DirectXRenderer*>(&theRenderer())));
    
    ModelPool::instancePtr() =
        GE_NEW ModelPool(theTexturePool(), static_cast<DirectXRenderer*>(&theRenderer()));
}

//------------------------------------------------------------------------------
void Framework::update()
//------------------------------------------------------------------------------
{
    // Update all system devices
    theMetronome().update();
    theScaledMetronome().update();
    theDisplayStatistics().tick(Timer::system().get());
    theMouse().update();
    theKeyboard().update();

    if (mActiveContext != 0)
    {
        mActiveContext->update();
    }
}

//------------------------------------------------------------------------------
void Framework::render()
//------------------------------------------------------------------------------
{
    theRenderer().beginScene();

    if (getActiveContext() != 0)
    {
        getActiveContext()->render();
    }

    theRenderer().print(theDisplayStatistics());
    theRenderer().print(5, 5, getActiveContext()->getName().c_str());
    theRenderer().endScene();
}

//------------------------------------------------------------------------------
void Framework::exit()
//------------------------------------------------------------------------------
{
}

//------------------------------------------------------------------------------
Handle Framework::getWindow()
//------------------------------------------------------------------------------
{
    GE_ASSERT(mWindow != 0);
    return mWindow;
}

//------------------------------------------------------------------------------
Context* Framework::getActiveContext()
//------------------------------------------------------------------------------
{
    return mActiveContext;
}

//------------------------------------------------------------------------------
const Context* Framework::getActiveContext() const
//------------------------------------------------------------------------------
{
    return mActiveContext;
}

//------------------------------------------------------------------------------
void Framework::setActiveContext(Context& c)
//------------------------------------------------------------------------------
{
    mActiveContext = &c;
}

} // namespace Ge
