#ifndef METRONOME_H
#define METRONOME_H
#pragma once

//------------------------------------------------------------------------------
// (c) 03-2001 Gottfried Chen
//------------------------------------------------------------------------------

#include <System/Types.h>
#include <System/Timer.h>

namespace Ge
{

// This class represents a metronome. Every frame it must be updated. It keeps
// track of the number of time update was called (e.g. can be used as a frame count).
// Further it stores the absolute time at each update and the delta time since
// the previous update.
class Metronome
{
public:
    static
    Metronome& instance();
   
    // Returns absolute time for this frame in milliseconds.
    Timer::Unit getAbsolute();

    // Returns time since last frame in milliseconds
    Timer::Unit getDelta();

    // Call this once every frame.
    void update();

    // Returns number of times update was called. Update count will
    // wrap if it gets to big.
    UInt getUpdateCount() const;

private:
    Metronome();

    Timer mTime;
    Timer::Unit mTimeAtUpdate;
    Timer::Unit mDelta;

    UInt mUpdateCount;
};

inline
//------------------------------------------------------------------------------
Metronome& theMetronome()
//------------------------------------------------------------------------------
{
    return Metronome::instance();
}

}

#endif // METRONOME_H
