#ifndef SCALED_METRONOME_H
#define SCALED_METRONOME_H
#pragma once

//------------------------------------------------------------------------------
// (c) 03-2001 Gottfried Chen
//------------------------------------------------------------------------------

#include <System/Types.h>
#include <System/Timer.h>

namespace Ge
{

// This class represents is similar to the Metronome, but can be scaled by
// a factor. The ScaledMetronome is started in sync with the Metronome (i.e.
// they'll report the same time if the ratio is kept at 1).
class ScaledMetronome
{
public:
    static
    ScaledMetronome& instance();
   
    // Returns absolute time for this frame in milliseconds.
    Timer::Unit getAbsolute();

    // Returns time since last frame.
    Timer::Unit getDelta();

    // Call this once every frame AFTER the Metronomes update().
    void update();

    // The default ratio is 1.
    void setRatio(Float);

    Float getRatio();
    
private:
    ScaledMetronome();

    Timer::Unit mTimeAtUpdate;
    Timer::Unit mDelta;
    Float mRatio;
};

inline
//------------------------------------------------------------------------------
ScaledMetronome& theScaledMetronome()
//------------------------------------------------------------------------------
{
    return ScaledMetronome::instance();
}

}

#endif // SCALED_METRONOME_H
