#include "Precompiled.h"
//------------------------------------------------------------------------------
// (c) 05-2001 Gottfried Chen
//------------------------------------------------------------------------------

#include "Settings.h"
#include "CommandLine.h"

#include <System/Log.h>

#include <sstream>
#include <fstream>

namespace Ge
{

//------------------------------------------------------------------------------
Settings::Variable::Variable(const std::string& id, const std::string& value) :
//------------------------------------------------------------------------------
mId(id),
mValue(value)
{
}

//------------------------------------------------------------------------------
const std::string& Settings::Variable::getId() const
//------------------------------------------------------------------------------
{
    return mId;
}

//------------------------------------------------------------------------------
Float Settings::Variable::asFloat() const
//------------------------------------------------------------------------------
{
    // ### Later: do type checking
    return Float(atof(mValue.c_str()));
}

//------------------------------------------------------------------------------
const std::string& Settings::Variable::asString() const
//------------------------------------------------------------------------------
{
    return mValue;
}

//------------------------------------------------------------------------------
bool Settings::Variable::asBool() const
//------------------------------------------------------------------------------
{
    GE_ASSERT(mValue[0] == '0' || mValue[0] == '1');
    return mValue[0] == '0' ? false : true;
}

//------------------------------------------------------------------------------
void Settings::Variable::set(const Float& value)
//------------------------------------------------------------------------------
{
    std::stringstream temp;
    temp<<value<<std::ends;
    mValue = temp.str();
}

//------------------------------------------------------------------------------
void Settings::Variable::set(bool value)
//------------------------------------------------------------------------------
{
    mValue = value ? "1" : "0";
}

//------------------------------------------------------------------------------
void Settings::Variable::set(const std::string& value)
//------------------------------------------------------------------------------
{
    mValue = value;
}


//------------------------------------------------------------------------------
Settings& Settings::instance(const char* fileName)
//------------------------------------------------------------------------------
{
    static Settings settings(fileName);
    return settings;
}

//------------------------------------------------------------------------------
Settings::Variable Settings::getVariable(const char* id)
//------------------------------------------------------------------------------
{
    if (mVariables.find(id) != mVariables.end())
        return Variable(std::string(id), mVariables[id]);
    else
    {
        GE_ASSERT_MSG(false, "ID invalid");
        return Variable("INVALID", "INVALID");
    }
}

//------------------------------------------------------------------------------
void Settings::setVariable(const Variable& variable)
//------------------------------------------------------------------------------
{
    mVariables[variable.getId()] = variable.asString();
    mChanged = true;
}


//------------------------------------------------------------------------------
Settings::Settings(const char* fileName)
//------------------------------------------------------------------------------
{
    GE_ASSERT_MSG(fileName != 0, "No file name provided");
    GE_ASSERT_MSG(FilePath(fileName).getExtension() == ".set", "Wrong extension");

    bool ok(true);
    if (fileName != 0 && FilePath(fileName).getExtension() == ".set")
    {
        mFile = fileName;
        ok = load(fileName);
    }
    else
        ok = false;

    if (!ok)
    {
        // ### Insert default variables like resolution, fullscreen, language ?
    }
}


//------------------------------------------------------------------------------
bool Settings::load(const char* fileName)
//------------------------------------------------------------------------------
{
    GE_LOG_INFO("Loading settings from \""<<fileName<<"\""<<std::endl);
    bool ret(true);
    std::ifstream file(fileName, std::ios::in);
    // Could file be opened?
    if (file)
    {
        std::string buffer;
        while (!file.eof())
        {
            // Read a line
            std::getline(file, buffer);
            CommandLine cmd(buffer.c_str());
            if (!cmd.isComment())
                cmd.execute();
        }
    }
    else
    {
        GE_ASSERT_MSG(false, "Failed to load settings file");
        ret = false;
    }

    // This is done here, because loading uses setVariable(), so mChanged is
    // set to true.
    mChanged = false;
    return ret;
}

//------------------------------------------------------------------------------
Settings::~Settings()
//------------------------------------------------------------------------------
{
    if (mChanged)
        save();
}

//------------------------------------------------------------------------------
void Settings::save()
//------------------------------------------------------------------------------
{
    GE_ASSERT(false);
}

} // namespace Ge
