//------------------------------------------------------------------------------
// (c) 05-2001 Gottfried Chen
//------------------------------------------------------------------------------

#ifndef APPLICATION_SETTINGS_H
#define APPLICATION_SETTINGS_H

#include <System/Base.h>
#include <System/FilePath.h>
#include <System/StringTools.h>
#include <string>
#include <map>

namespace Ge
{

#define theSettings Settings::instance()

class Settings
{
public:
    class Variable
    {
    public:
        Variable(const std::string& id, const std::string& value);

        const std::string& getId() const;

        // I would prefer the accessors as template members, but
        // it didn't work with MSVC (also tried settings_cast<>()
        // approach, but I guess it's best like it is now).
        Float asFloat() const;
        const std::string& asString() const;
        bool asBool() const;

        void set(const Float&);
        void set(bool);
        void set(const std::string&);

    private:
        std::string mId;
        std::string mValue;
    };

    // The destructor writes the settings back to the file if the settings
    // were changed
    ~Settings();

    // The first call to instance calls the constructor with <fileName>
    static
    Settings& instance(const char* fileName = 0);


    Variable getVariable(const char* id);
    void setVariable(const Variable&);

private:
    Settings(const char* fileName);
    bool load(const char* fileName);
    void save();

    typedef std::map<std::string, std::string, StringTools::Pred> Map;
    Map mVariables;
    std::string mFile;
    bool mChanged;
};

} // namespace Ge

#endif // APPLICATION_SETTINGS_H