#include "Precompiled.h"
//------------------------------------------------------------------------------
// (c) 05-2001 Gottfried Chen
//------------------------------------------------------------------------------

#include "StringMap.h"
#include "StringId.h"

#include <System/FilePath.h>
#include <System/Log.h>


namespace Ge
{

//------------------------------------------------------------------------------
StringMap::StringMap()
//------------------------------------------------------------------------------
{
}

//------------------------------------------------------------------------------
StringMap& StringMap::instance()
//------------------------------------------------------------------------------
{
    static StringMap theMap;
    return theMap;
}

//------------------------------------------------------------------------------
bool StringMap::load(const char* fileName)
//------------------------------------------------------------------------------
{
    GE_LOG_INFO("Loading string map from \""<<fileName<<"\""<<std::endl);
    bool ret(true);
    // Is file name ok?
	if (fileName != 0 && FilePath(fileName).getExtension() == ".str")
	{
        std::ifstream file(fileName, std::ios::in);
        // Could file be opened?
        if (file)
            ret = parse(file);
        else
        {
            GE_ASSERT_MSG(false, "File could not be accessed");
            ret = false;
        }
	}
    else
    {
        GE_ASSERT_MSG(false, "File name corrupt");
        ret = false;
    }

    return ret;
}

//------------------------------------------------------------------------------
bool StringMap::parse(std::istream& file)
//------------------------------------------------------------------------------
{
    bool ret(true);
	std::string identifier;
	std::string text;
	std::string buffer;
	UInt spacePos(0);
    bool ok(true);
    while (!file.eof())
    {
        // Read a line
        std::getline(file, buffer);

	    // Separate into identifier and text
	    spacePos = buffer.find_first_of(' ');
	    if (spacePos != std::string::npos && spacePos != 0)
	    {
		    identifier = buffer.substr(0, spacePos);
		    text = buffer.substr(spacePos + 1);
		    // Text is longer than one line -> search for '~'
		    while (text[text.size()-1] != '~' && !file.eof())
		    {
                std::getline(file, buffer);
			    text += buffer;
		    }
		    
		    if (!file.eof())
		    {
                // ### Don't do search in release mode (costs too much),
                // maybe later.
			    GE_ASSERT_MSG(mMap.find(identifier) == mMap.end(),
                    (std::string("StringId ") + identifier
                    + " already exists").c_str());
			    mMap[identifier] = text.substr(0, text.size()-1);
		    }
		    else
            {
			    GE_ASSERT_MSG(false, "~ missing");
                ret = false;
            }
	    }
	    else
	    {
		    // No space separator found -> it must be either
            // a blank line or a comment
	        ok = buffer.size() == 0 ||
			     buffer.find_first_of("//") == buffer.find_first_not_of(' ');
            GE_ASSERT_MSG(ok, "File corrupt");
            if (!ok)
                ret = false;
	    }
    }

    return ret;
}

//------------------------------------------------------------------------------
const std::string& StringMap::get(const StringId& identifier) const
//------------------------------------------------------------------------------
{
    static std::string unknown("Unknown identifier");
    if (mMap.find(identifier.getId()) != mMap.end())
	    return mMap[identifier.getId()];
    else
    {
        GE_ASSERT_MSG(false, "Unknown identifier");
        return unknown;
    }
}

//------------------------------------------------------------------------------
UInt StringMap::getSize() const
//------------------------------------------------------------------------------
{
	return mMap.size();
}


} // namespace Ge