/*****************************************************************
|
|      XAudio Player Wrapper Class    
|
|
|      (c) 1996-1998 MpegTV, LLC
|      Author: Gilles Boccon-Gibod (gilles@mpegtv.com)
|
 ****************************************************************/

/*----------------------------------------------------------------------
|       includes
+---------------------------------------------------------------------*/
#ifndef STRICT
#define STRICT
#endif

#include <stdio.h>
#include <tchar.h>
#include "XaudioPlayer.h"
#include <control_win32.h>

/*----------------------------------------------------------------------
|       XaudioMessageHandler
+---------------------------------------------------------------------*/
LRESULT CALLBACK
XaudioMessageHandler(HWND window, 
                     UINT win32_message, 
                     WPARAM win32_wparam, 
                     LPARAM win32_lparam)
{
    XA_Message xaudio_message;

    XaudioPlayer *player = 
        (XaudioPlayer *)GetWindowLong(window, GWL_USERDATA);
    if (player == NULL || win32_message < XA_MSG_BASE) {
        return DefWindowProc(window, win32_message, win32_wparam, win32_lparam);
    }

    if (control_win32_params_to_message(player->control,
                                        &xaudio_message,
                                        win32_message,
                                        win32_wparam,
                                        win32_lparam) == XA_SUCCESS) {
        player->ProcessMessage(&xaudio_message);
        return 0;
    } else {
        return DefWindowProc(window, win32_message, win32_wparam, win32_lparam);
    }
}

/*----------------------------------------------------------------------
|       XaudioPlayer contructor
+---------------------------------------------------------------------*/
XaudioPlayer::XaudioPlayer(HINSTANCE instance) : m_Instance(instance)
{
    /* check the API version */
    unsigned int version;
    version = xaudio_get_version(XA_VERSION_ID_ASYNC_API);
    if ((((version >> 16)&0xFF) != XA_REQUIRED_VERSION_MAJOR) ||
        (((version >>  8)&0xFF) != XA_REQUIRED_VERSION_MINOR)) {
        MessageBox(NULL, 
                   TEXT("Xaudio DLL API version mismatch"), 
                   TEXT("Xaudio Error"), 
                   MB_OK            | 
                   MB_ICONERROR     | 
                   MB_SETFOREGROUND |
                   MB_TOPMOST);
#ifndef UNDER_CE
        throw XaudioException(XA_ERROR_VERSION_MISMATCH);
#else
        return;
#endif
    }

    /* make unique window class name */
    m_ClassName = new TCHAR[14];
    if (m_ClassName == NULL) {
#ifndef UNDER_CE
        throw XaudioException(XA_ERROR_OUT_OF_MEMORY);
#else
        return;
#endif
    }
    _stprintf(m_ClassName, TEXT("XaWrW%08x"), GetCurrentThreadId());

    /* register a window class */
    WNDCLASS               wclass;
    wclass.style         = 0;
    wclass.lpfnWndProc   = XaudioMessageHandler;
    wclass.cbClsExtra    = 0;
    wclass.cbWndExtra    = 0;
    wclass.hInstance     = instance;
    wclass.hIcon         = NULL;
    wclass.hCursor       = NULL;
    wclass.hbrBackground = NULL;
    wclass.lpszMenuName  = NULL;
    wclass.lpszClassName = m_ClassName;

    /* register the class and ignore any error because we might */
    /* be registering the class more than once                  */
    RegisterClass(&wclass);

    /* create a hidden window to receive notification messages */
    m_HiddenWindow = CreateWindow(
        wclass.lpszClassName,       // pointer to registered class name 
        TEXT("Xaudio Hidden Window"),// pointer to window name 
        0,                          // window style 
        0,                          // horizontal position of window 
        0,                          // vertical position of window 
        0,                          // window width 
        0,                          // window height 
        NULL,                       // handle to parent or owner window 
        NULL,                       // handle to menu or child-window identifier 
        instance,                   // handle to application instance 
        NULL);

    if (m_HiddenWindow) {
        /* set the window's user data to this object */
        SetWindowLong(m_HiddenWindow, GWL_USERDATA, (LONG)this);
    }

    /* instanciate the XA_Player object */
    this->Run((void *)&m_HiddenWindow);
}

/*----------------------------------------------------------------------
|       XaudioPlayer destructor
+---------------------------------------------------------------------*/
XaudioPlayer::~XaudioPlayer()
{
    /* delete the hidden window */
    DestroyWindow(m_HiddenWindow);
    UnregisterClass(m_ClassName, m_Instance);
    delete[] m_ClassName;
}

