//------------------------------------------------------------------------------
// (c) 04-2001 Gottfried Chen
//------------------------------------------------------------------------------

#ifndef DIRECTX_RENDERER_DIRECTX_CACHE
#define DIRECTX_RENDERER_DIRECTX_CACHE

#include <Engine/RenderCache.h>

struct IDirect3DVertexBuffer8;
struct IDirect3DIndexBuffer8;



namespace Ge
{

class DirectXRenderer;

class DirectXCache : public RenderCache
{
public:
    DirectXCache(const Mesh&, DirectXRenderer&);
    ~DirectXCache();

    virtual
    bool update();

    virtual
    void render() const;

    const Mesh& getMesh() const;
    UInt getVertexFormat() const;
    IDirect3DVertexBuffer8& getVertexBuffer() const;
    IDirect3DIndexBuffer8& getIndexBuffer() const;
  

private:
    bool reallocateVertexBuffer();
    bool reallocateIndexBuffer();
    bool createVertexBuffer();
    bool createIndexBuffer();

    const Mesh& mMesh;
    IDirect3DVertexBuffer8* mVertexBuffer;
    IDirect3DIndexBuffer8* mIndexBuffer;
    DirectXRenderer& mRenderer;
    UInt mVertexFormat;

    // The number of vertices, indices at the last time that update() was
    // called is stored here. At the next update() the corresponding buffer
    // is only reallocated, if the number has changed. Otherwise it is reused.
    UInt mLastVertexCount;
    UInt mLastIndexCount;
};

} // namespace Ge

#endif // DIRECTX_RENDERER_DIRECTX_CACHE

