//------------------------------------------------------------------------------
// (c) 04-2001 Gottfried Chen
//------------------------------------------------------------------------------

#ifndef DIRECTX_RENDERER_DIRECTX_HELPERS_H
#define DIRECTX_RENDERER_DIRECTX_HELPERS_H

// Since this includes <d3d8.h>, it's best, to include this header last, to
// avoid clashes with Microsoft macros
#include <d3d8.h>

#include <Engine/Mesh.h>

#include <Math/Vector.h>
#include <Math/Matrix.h>

#define DX_CHECK(x) Dx::check((x), (#x), __FILE__, __LINE__)
#define DX_CHECK_MSG(x, y) Dx::check((x), (y), __FILE__, __LINE__)
#define DX_RELEASE(x) if (x) { (x)->Release(); (x) = 0; }

namespace Ge
{

class Color;
class Material;

class Dx
{
public:
    static inline
    bool check(HRESULT hResult, const char* msg, const char* file, UInt line)
    {
#ifndef NDEBUG
        geAssert(!FAILED(hResult), msg, file, line);
#endif
        return FAILED(hResult) ? false : true;
    }

    static UInt getSizeFromVertexFormat(DWORD format);

    // Conversion functions
    static void matrixToD3dMatrix(const Matrix4x4& matrix, D3DMATRIX& d3dmatrix);
    static void vectorToD3dVector(const Vector3& in, D3DVECTOR& out);

    static void colorToD3dColor(const Color& in, D3DCOLORVALUE& out);
    static void colorToD3dColor(const Color& in, D3DCOLOR& out);
    static void d3dColorToColor(const D3DCOLORVALUE& in, Color& out);
    static void d3dColorToColor(const D3DCOLOR& in, Color& out);

    static void materialToD3dMaterial(const Material& in, D3DMATERIAL8& out);
    static void d3dMaterialToMaterial(const D3DMATERIAL8& in, Material& out);
};

} // namespace Ge


#include <DirectXRenderer/DirectXHelpers.inl>
#endif // DIRECTX_RENDERER_DIRECTX_HELPERS_H