//------------------------------------------------------------------------------
// (c) 04-2001 Gottfried Chen
//------------------------------------------------------------------------------

#include <DirectXRenderer/DirectXHelpers.h>
#include <Math/Matrix.h>
#include <Math/Vector.h>
#include <Engine/Color.h>
#include <Engine/Material.h>


namespace Ge
{

// Given a Direct3d flexible vertex format the size of the matching
// Vertex struct is calculated
inline
//------------------------------------------------------------------------------
UInt Dx::getSizeFromVertexFormat(DWORD format)
//------------------------------------------------------------------------------
{
    UInt ret(0);
    
    if ((format&D3DFVF_XYZ) != 0)
    {
        ret += 3*sizeof(float);
        // Clear subflag, so at the end checking unknown formats is possible
        format &= ~D3DFVF_XYZ;
    }

    if ((format&D3DFVF_NORMAL) != 0)
    {
        ret += 3*sizeof(float);
        format &= ~D3DFVF_NORMAL;
    }

    if ((format&D3DFVF_DIFFUSE) != 0)
    {
        ret += sizeof(DWORD);
        format &= ~D3DFVF_DIFFUSE;
    }

    if ((format&D3DFVF_SPECULAR) != 0)
    {
        ret += sizeof(DWORD);
        format &= ~D3DFVF_SPECULAR;
    }

    if ((format&D3DFVF_TEX2) != 0)
    {
        ret += 4*sizeof(float);
//        format &= ~D3DFVF_TEX2;
        format &= ~(D3DFVF_TEX2|D3DFVF_TEXCOORDSIZE2(0)|D3DFVF_TEXCOORDSIZE2(1));
    }
    else if ((format&D3DFVF_TEX1) != 0)
    {
        ret += 2*sizeof(float);
        format &= ~D3DFVF_TEX1;
    }

    GE_ASSERT_MSG(format == 0, "Unknown vertex format");
    return ret;
}

inline
//------------------------------------------------------------------------------
void Dx::matrixToD3dMatrix(const Matrix4x4& matrix, D3DMATRIX& d3dmatrix)
//------------------------------------------------------------------------------
{
    d3dmatrix._11 = matrix(0, 0);
    d3dmatrix._12 = matrix(1, 0);
    d3dmatrix._13 = matrix(2, 0);
    d3dmatrix._14 = matrix(3, 0);

    d3dmatrix._21 = matrix(0, 1);
    d3dmatrix._22 = matrix(1, 1);
    d3dmatrix._23 = matrix(2, 1);
    d3dmatrix._24 = matrix(3, 1);

    d3dmatrix._31 = matrix(0, 2);
    d3dmatrix._32 = matrix(1, 2);
    d3dmatrix._33 = matrix(2, 2);
    d3dmatrix._34 = matrix(3, 2);

    d3dmatrix._41 = matrix(0, 3);
    d3dmatrix._42 = matrix(1, 3);
    d3dmatrix._43 = matrix(2, 3);
    d3dmatrix._44 = matrix(3, 3);
}



inline
//------------------------------------------------------------------------------
void Dx::vectorToD3dVector(const Vector3& in, D3DVECTOR& out)
//------------------------------------------------------------------------------
{
    out.x = in[0];
    out.y = in[1];
    out.z = in[2];
}


inline
//------------------------------------------------------------------------------
void Dx::colorToD3dColor(const Color& in, D3DCOLORVALUE& out)
//------------------------------------------------------------------------------
{
    in.getFloat(out.r, out.g, out.b, out.a);
}


inline
//------------------------------------------------------------------------------
void Dx::colorToD3dColor(const Color& in, D3DCOLOR& out)
//------------------------------------------------------------------------------
{
    out = in.get(Color::ARGB);
}


inline
//------------------------------------------------------------------------------
void Dx::d3dColorToColor(const D3DCOLORVALUE& in, Color& out)
//------------------------------------------------------------------------------
{
    out.setFloat(in.r, in.g, in.b, in.a);
}


inline
//------------------------------------------------------------------------------
void Dx::d3dColorToColor(const D3DCOLOR& in, Color& out)
//------------------------------------------------------------------------------
{
    out.set(Color::ARGB, in);
}


inline
//------------------------------------------------------------------------------
void Dx::materialToD3dMaterial(const Material& in, D3DMATERIAL8& out)
//------------------------------------------------------------------------------
{
    memset(&out, 0, sizeof(out));
    Dx::colorToD3dColor(in.getColor(), out.Ambient);
    Dx::colorToD3dColor(in.getColor(), out.Diffuse);
    Dx::colorToD3dColor(in.getColor(), out.Specular);
    Dx::colorToD3dColor(Color::BLACK, out.Emissive);
    
    out.Power = 0;//in.mSpecularSharpness;
}


inline
//------------------------------------------------------------------------------
void Dx::d3dMaterialToMaterial(const D3DMATERIAL8& in, Material& out)
//------------------------------------------------------------------------------
{
//    Dx::d3dColorToColor(in.Ambient, out.mAmbient);
    Color c;
    Dx::d3dColorToColor(in.Diffuse, c);
    out.set(c);
//    Dx::d3dColorToColor(in.Specular, out.mSpecular);
//    Dx::d3dColorToColor(in.Emissive, out.mEmissive);
    
//    out.mSpecularSharpness = in.Power;
}



} // namespace Ge