//------------------------------------------------------------------------------
// (c) 04-2001 Gottfried Chen
//------------------------------------------------------------------------------

#ifndef DIRECTX_RENDERER_DIRECTX_RENDERER_H
#define DIRECTX_RENDERER_DIRECTX_RENDERER_H

#include <Engine/Renderer.h>
#include <Engine/ShadowVolume.h>
#include <Engine/Material.h>

#include <System/Types.h>

#include <Math/Matrix.h>

struct IDirect3D8;
struct IDirect3DDevice8;
struct IDirect3DVertexBuffer8;
struct ID3DXFont;

namespace Ge
{

class DirectXCache;
class DirectXTexture;

class DirectXRenderer : public Renderer
{
public:
    //------------------------------------------------------------------------------
    // Construction
    //------------------------------------------------------------------------------
    DirectXRenderer(Handle window,
                    const Color& background = Color(0.2f,0.2f,0.2f,1));
    ~DirectXRenderer();
    
    //------------------------------------------------------------------------------
    // Rendering
    //------------------------------------------------------------------------------
    virtual void beginScene();
    virtual void endScene();

    virtual void render(const Mesh&);
    virtual void render(const Lines&);
    void render(const DirectXCache&);
    void render(const ShadowVolume&);
    void render(const Emitter&);

    //------------------------------------------------------------------------------
    // Lights
    //------------------------------------------------------------------------------
    virtual void add(const Light&);
    virtual UInt getPossibleNumberOfLights() const;
    virtual void setAmbientLight(const Color&);
    
    // Does not disable lighting, but switches all currently set lights on or off.
    // <true> to enable lights, <false> to disable them.
    void switchLights(bool);

    //------------------------------------------------------------------------------
    // Tools
    //------------------------------------------------------------------------------
 
    IDirect3DDevice8& getDirect3dDevice() const;

    //------------------------------------------------------------------------------
    // Debugging
    //------------------------------------------------------------------------------
    virtual void print(UInt x, UInt y, const char* text);
    virtual void print(const DisplayStatistics&);

    //------------------------------------------------------------------------------
    // Render states
    //------------------------------------------------------------------------------

    virtual void set(const StateBlock&);
    virtual void set(const Material&);
    void set(RenderState::Blend);
    void set(RenderState::Fill);
    void set(RenderState::Shade);
    void set(RenderState::Specular);
    void set(RenderState::Depth);
    void set(RenderState::DepthCompare);
    void set(RenderState::Stencil s);
    void set(RenderState::ColorWrite);
    void set(RenderState::Culling);
    void set(RenderState::Lighting);

    //------------------------------------------------------------------------------
    // Shader playground
    //------------------------------------------------------------------------------
    void initShader();
    

    //------------------------------------------------------------------------------
    // Transformations
    //------------------------------------------------------------------------------
    virtual void setWorldMatrix(const Matrix4x4&);

    virtual void setWorldMatrix(const Matrix3x4&);
    
private:
    // Constructor helpers
    bool createDirect3d();
    bool createDirect3dDevice();
    bool createPrintFont();

    void setViewMatrix(const Matrix4x4&);
    void setProjectionMatrix(const Matrix4x4&);

//    void assembleShader(VertexShader& shader) const;

    IDirect3D8* mDirect3d;
    IDirect3DDevice8* mDirect3dDevice;
    ID3DXFont* mPrintFont;

    Handle mWindow;
    UInt mLightsInScene;

    Matrix4x4 mWorld;
    Matrix4x4 mView;

    StateBlock mStates;

    const Material* mMaterial;
};

} // namespace Ge

#endif // DIRECTX_RENDERER_DIRECTX_RENDERER_H