#include "Precompiled.h"
//------------------------------------------------------------------------------
// (c) Jul-2002 Gottfried Chen
//------------------------------------------------------------------------------

#include "DirectXRenderer.h"

namespace Ge
{

//------------------------------------------------------------------------------
void DirectXRenderer::set(RenderState::Blend b)
//------------------------------------------------------------------------------
{
    switch (b)
    {
    case RenderState::BLEND_OFF:
        getDirect3dDevice().SetRenderState(D3DRS_ALPHABLENDENABLE, FALSE);
        break;

    case RenderState::BLEND_MULTIPLY:
        getDirect3dDevice().SetRenderState(D3DRS_ALPHABLENDENABLE, TRUE);
        getDirect3dDevice().SetRenderState(D3DRS_SRCBLEND, D3DBLEND_DESTCOLOR);
        getDirect3dDevice().SetRenderState(D3DRS_DESTBLEND, D3DBLEND_ZERO);
        break;

    case RenderState::BLEND_ADD:
        getDirect3dDevice().SetRenderState(D3DRS_ALPHABLENDENABLE, TRUE);
        getDirect3dDevice().SetRenderState(D3DRS_SRCBLEND, D3DBLEND_ONE);
        getDirect3dDevice().SetRenderState(D3DRS_DESTBLEND, D3DBLEND_ONE);
        break;

    case RenderState::BLEND_ADD_SQUARED:
        getDirect3dDevice().SetRenderState(D3DRS_ALPHABLENDENABLE, TRUE);
        getDirect3dDevice().SetRenderState(D3DRS_SRCBLEND, D3DBLEND_SRCCOLOR);
        getDirect3dDevice().SetRenderState(D3DRS_DESTBLEND, D3DBLEND_ONE);
        break;

    case RenderState::BLEND_SOURCE_ALPHA:
        getDirect3dDevice().SetRenderState(D3DRS_ALPHABLENDENABLE, TRUE);
        getDirect3dDevice().SetRenderState(D3DRS_SRCBLEND, D3DBLEND_SRCALPHA);
        getDirect3dDevice().SetRenderState(D3DRS_DESTBLEND, D3DBLEND_INVSRCALPHA);
        break;

    case RenderState::BLEND_SOURCE_ALPHA_ADD:
        getDirect3dDevice().SetRenderState(D3DRS_ALPHABLENDENABLE, TRUE);
        getDirect3dDevice().SetRenderState(D3DRS_SRCBLEND, D3DBLEND_SRCALPHA);
        getDirect3dDevice().SetRenderState(D3DRS_DESTBLEND, D3DBLEND_ONE);
        break;

    default:
        GE_ASSERT_MSG(false, "Unknown blend mode.");
    }

    mStates.set(b);
}

//------------------------------------------------------------------------------
void DirectXRenderer::set(RenderState::Fill f)
//------------------------------------------------------------------------------
{
    switch(f)
    {
    case RenderState::FILL_SOLID:
        getDirect3dDevice().SetRenderState(D3DRS_FILLMODE, D3DFILL_SOLID);
        break;

    case RenderState::FILL_WIRE:
        getDirect3dDevice().SetRenderState(D3DRS_FILLMODE, D3DFILL_WIREFRAME);
        break;

    case RenderState::FILL_POINT:
        getDirect3dDevice().SetRenderState(D3DRS_FILLMODE, D3DFILL_POINT);
        break;

    default:
        GE_ASSERT_MSG(false, "Unknown fill mode.");
    }

    mStates.set(f);
}

//------------------------------------------------------------------------------
void DirectXRenderer::set(RenderState::Shade s)
//------------------------------------------------------------------------------
{
    switch(s)
    {
    case RenderState::SHADE_GOURAUD:
        getDirect3dDevice().SetRenderState(D3DRS_SHADEMODE, D3DSHADE_GOURAUD);
        break;

    case RenderState::SHADE_FLAT:
        getDirect3dDevice().SetRenderState(D3DRS_SHADEMODE, D3DSHADE_FLAT);
        break;

    default:
        GE_ASSERT_MSG(false, "Unknown shade mode.");
    }

    mStates.set(s);
}

//------------------------------------------------------------------------------
void DirectXRenderer::set(RenderState::Specular s)
//------------------------------------------------------------------------------
{
    switch(s)
    {
    case RenderState::SPECULAR_ON:
        getDirect3dDevice().SetRenderState(D3DRS_SPECULARENABLE, TRUE);
        break;

    case RenderState::SPECULAR_OFF:
        getDirect3dDevice().SetRenderState(D3DRS_SPECULARENABLE, FALSE);
        break;

    default:
        GE_ASSERT_MSG(false, "Unknown specular mode.");
    }

    mStates.set(s);
}

//------------------------------------------------------------------------------
void DirectXRenderer::set(RenderState::Depth d)
//------------------------------------------------------------------------------
{
    switch(d)
    {
    case RenderState::DEPTH_ON:
        getDirect3dDevice().SetRenderState(D3DRS_ZENABLE, D3DZB_TRUE);
        getDirect3dDevice().SetRenderState(D3DRS_ZWRITEENABLE, TRUE);
        break;

    case RenderState::DEPTH_OFF:
        getDirect3dDevice().SetRenderState(D3DRS_ZENABLE, D3DZB_FALSE);
        getDirect3dDevice().SetRenderState(D3DRS_ZWRITEENABLE, FALSE);
        break;

    case RenderState::DEPTH_READ_ONLY:
        getDirect3dDevice().SetRenderState(D3DRS_ZENABLE, D3DZB_TRUE);
        getDirect3dDevice().SetRenderState(D3DRS_ZWRITEENABLE, FALSE);
        break;

    default:
        GE_ASSERT_MSG(false, "Unknown depth mode.");
    }

    mStates.set(d);
}


//------------------------------------------------------------------------------
void DirectXRenderer::set(RenderState::DepthCompare d)
//------------------------------------------------------------------------------
{
    switch(d)
    {
    case RenderState::DEPTH_COMPARE_EQUAL:
        getDirect3dDevice().SetRenderState(D3DRS_ZFUNC, D3DCMP_EQUAL);
        break;

    case RenderState::DEPTH_COMPARE_LESS_OR_EQUAL:
        getDirect3dDevice().SetRenderState(D3DRS_ZFUNC, D3DCMP_LESSEQUAL);
        break;

    case RenderState::DEPTH_COMPARE_LESS:
        getDirect3dDevice().SetRenderState(D3DRS_ZFUNC, D3DCMP_LESS);
        break;

    default:
        GE_ASSERT_MSG(false, "Unknown depth compare mode.");
    }

    mStates.set(d);
}

//------------------------------------------------------------------------------
void DirectXRenderer::set(RenderState::Stencil s)
//------------------------------------------------------------------------------
{
    switch(s)
    {
    case RenderState::STENCIL_ON:
        getDirect3dDevice().SetRenderState(D3DRS_STENCILENABLE, TRUE);
        break;

    case RenderState::STENCIL_OFF:
        getDirect3dDevice().SetRenderState(D3DRS_STENCILENABLE, FALSE);
        break;

    default:
        GE_ASSERT_MSG(false, "Unknown stencil mode.");
    }

    mStates.set(s);
}

//------------------------------------------------------------------------------
void DirectXRenderer::set(RenderState::ColorWrite c)
//------------------------------------------------------------------------------
{
    switch(c)
    {
    case RenderState::COLOR_WRITE_ON:
        getDirect3dDevice().SetRenderState(D3DRS_COLORWRITEENABLE, 0xf);
        break;

    case RenderState::COLOR_WRITE_OFF:
        getDirect3dDevice().SetRenderState(D3DRS_COLORWRITEENABLE, 0x0);
        break;

    default:
        GE_ASSERT_MSG(false, "Unknown color write mode.");
    }

    mStates.set(c);
}


//------------------------------------------------------------------------------
void DirectXRenderer::set(RenderState::Culling c)
//------------------------------------------------------------------------------
{
    switch(c)
    {
    case RenderState::CULLING_CLOCKWISE:
        getDirect3dDevice().SetRenderState(D3DRS_CULLMODE, D3DCULL_CW);
        break;

    case RenderState::CULLING_COUNTERCLOCKWISE:
        getDirect3dDevice().SetRenderState(D3DRS_CULLMODE, D3DCULL_CCW);
        break;

    case RenderState::CULLING_OFF:
        getDirect3dDevice().SetRenderState(D3DRS_CULLMODE, D3DCULL_NONE);
        break;

    default:
        GE_ASSERT_MSG(false, "Unknown culling mode.");
    }

    mStates.set(c);
}

//------------------------------------------------------------------------------
void DirectXRenderer::set(RenderState::Lighting l)
//------------------------------------------------------------------------------
{
    switch(l)
    {
    case RenderState::LIGHTING_ON:
        getDirect3dDevice().SetRenderState(D3DRS_LIGHTING, TRUE);
        break;

    case RenderState::LIGHTING_OFF:
        getDirect3dDevice().SetRenderState(D3DRS_LIGHTING, FALSE);
        break;

    default:
        GE_ASSERT_MSG(false, "Unknown lighting mode.");
    }

    mStates.set(l);
}

} // namespace Ge
