#include "Precompiled.h"
//------------------------------------------------------------------------------
// (c) 06-2001 Gottfried Chen
//------------------------------------------------------------------------------

#include <DirectXRenderer/DirectXTexture.h>
#include <DirectXRenderer/DirectXRenderer.h>

#include <Image/Image.h>

#include <System/FilePath.h>
#include <System/StringTools.h>

#include <DirectXRenderer/DirectXHelpers.h>
#include <d3dx8tex.h>

namespace Ge
{

//------------------------------------------------------------------------------
DirectXTexture::DirectXTexture(const char* file,
                               const DirectXRenderer& renderer,
                               Texture::Filtering filtering) :
//------------------------------------------------------------------------------
Texture(file, filtering),
mRenderer(renderer)
{
    // Load image
    Image img(file);
    
    // Flip all but .jpg images, since they are bottom up
    std::string extension(FilePath(file).getExtension());
    StringTools::toLower(extension);
    if (extension != ".jpg" || extension != ".jpeg")
        img.flipX();

    createFromImage(img);
}

//------------------------------------------------------------------------------
DirectXTexture::DirectXTexture(const Image& image, const char* name,
                               const DirectXRenderer& renderer,
                               Texture::Filtering filtering) :
//------------------------------------------------------------------------------
Texture(name, filtering),
mRenderer(renderer)
{
    Image im(image);
    createFromImage(im);
}

//------------------------------------------------------------------------------
DirectXTexture::DirectXTexture(IDirect3DTexture8& texture, const char* name,
                               const DirectXRenderer& renderer,
                               Texture::Filtering filtering) :
//------------------------------------------------------------------------------
Texture(name, filtering),
mRenderer(renderer),
mTexture(&texture)
{
}

//------------------------------------------------------------------------------
DirectXTexture::~DirectXTexture()
//------------------------------------------------------------------------------
{
    if (mTexture != 0)
    {
        mTexture->Release();
    }

    mTexture = 0;
}

//------------------------------------------------------------------------------
void DirectXTexture::createFromImage(Image& image)
//------------------------------------------------------------------------------
{
    if (image.getBitDepth() == 24)
    {
        image.convertTo(Image::BGRA_8888);
        DX_CHECK(D3DXCreateTexture(
            &mRenderer.getDirect3dDevice(),
            image.getWidth(), image.getHeight(),
            1, 0,
            D3DFMT_X8R8G8B8,
            D3DPOOL_MANAGED,
            &mTexture));

        setFormat(BGRX_8888);
    }
    else if (image.getBitDepth() == 32)
    {
        image.convertTo(Image::BGRA_8888);
        DX_CHECK(D3DXCreateTexture(
            &mRenderer.getDirect3dDevice(),
            image.getWidth(), image.getHeight(),
            1, 0,
            D3DFMT_A8R8G8B8,
            D3DPOOL_MANAGED, &mTexture));

        setFormat(BGRA_8888);
    }
    else
    {
        // ToDo: Support 16 bit later, Support MIP maps later
        GE_ASSERT(false);
    }


    // Copy the pixel data into the texture
    D3DLOCKED_RECT rect;
    DX_CHECK(mTexture->LockRect(0, &rect, 0, 0));
    UInt8* data = static_cast<UInt8*>(rect.pBits);
    image.copyTo(data, rect.Pitch);
    DX_CHECK(mTexture->UnlockRect(0));
}

//------------------------------------------------------------------------------
IDirect3DTexture8* DirectXTexture::getDirect3dTexture() const
//------------------------------------------------------------------------------
{
    return mTexture;
}

//------------------------------------------------------------------------------
Image* DirectXTexture::convertToImage() const
//------------------------------------------------------------------------------
{
    GE_ASSERT(mTexture != 0);
    if (!mTexture)
    {
        return 0;
    }

    Image* image = 0;
    D3DSURFACE_DESC desc;
    mTexture->GetLevelDesc(0, &desc);
    image = GE_NEW Image(desc.Width, desc.Height, Image::BGRA_8888);    

    // Copy the pixel data into the texture
    D3DLOCKED_RECT rect;
    DX_CHECK(mTexture->LockRect(0, &rect, 0, 0));
    UInt8* data = static_cast<UInt8*>(rect.pBits);
    image->copyFrom(data, rect.Pitch);
    DX_CHECK(mTexture->UnlockRect(0));

    return image;
}

} // namespace Ge