#include "Precompiled.h"

#include <DirectXRenderer/DirectXTexturePool.h>
#include <DirectXRenderer/DirectXTexture.h>

#include <Image/Image.h>

namespace Ge
{
 
//------------------------------------------------------------------------------
DirectXTexturePool::DirectXTexturePool(const DirectXRenderer& renderer,
                                       Texture::Filtering filter) :
//------------------------------------------------------------------------------
TexturePool(filter),
mRenderer(renderer)
{
}

//------------------------------------------------------------------------------
Texture* DirectXTexturePool::get(const char* file)
//------------------------------------------------------------------------------
{
    Texture* texture = mTextures[std::string(file)];
    if (texture == 0)
    {
        texture = GE_NEW DirectXTexture(file, mRenderer, getDefaultFiltering());
        mTextures[std::string(file)] = texture;
    }
    
    return texture;
}

//------------------------------------------------------------------------------
Texture* DirectXTexturePool::get(zip::izipfile& zipfile, const char* file)
//------------------------------------------------------------------------------
{
    Texture* texture = mTextures[std::string(file)];
    if (texture == 0)
    {
        Image image(zipfile, file);
        texture = GE_NEW DirectXTexture(image, file, mRenderer, getDefaultFiltering());
        mTextures[std::string(file)] = texture;
    }

    return texture;
}

//------------------------------------------------------------------------------
Texture* DirectXTexturePool::get(const Image& im, const char* name)
//------------------------------------------------------------------------------
{
    Texture* texture = mTextures[std::string(name)];
    if (texture == 0)
    {
        texture = GE_NEW DirectXTexture(im, name, mRenderer, getDefaultFiltering());
        mTextures[std::string(name)] = texture;
    }
    
    return texture;
}

} // namespace Ge