#include "Precompiled.h"

#include "ShaderPlayground.h"

// Just some constants for later use
float c[4] = { 0.0f, 0.5f, 1.0f, 2.0f };

// This is the shader handle
DWORD shader(0);

Ge::Matrix4x4 projection;
Ge::Matrix4x4 world;
Ge::Matrix4x4 view;

// Declaration for my vertex shader
DWORD shaderDeclaration[] =
{
    // Use stream 0
    D3DVSD_STREAM(0),

    // v0 contains the position
    D3DVSD_REG(D3DVSDE_POSITION, D3DVSDT_FLOAT3 ),

    // v3 contains the normal
    D3DVSD_REG(D3DVSDE_NORMAL, D3DVSDT_FLOAT3 ),

    // v5 contains the diffuse vertex color
    //D3DVSD_REG(D3DVSDE_DIFFUSE, D3DVSDT_D3DCOLOR ),

    // v7 contains the texture coordinates
    D3DVSD_REG(D3DVSDE_TEXCOORD0, D3DVSDT_FLOAT2 ),

    // Sets a constant vector at register c0
    D3DVSD_CONST(0,1),*(DWORD*)&c[0],*(DWORD*)&c[1],*(DWORD*)&c[2],*(DWORD*)&c[3],

    // End the shader declaration
    D3DVSD_END()
};


// Simple vertex shader
// Constants
//      reg c0   = { 0.0f, 0.5f, 1.0f, 2.0f }
//      reg c1-4 = world matrix
//      reg c5-8 = proj*view*world matrix
//      reg c9   = camera position
// Stream 0
//     reg v0   = position
//     reg v3   = normal
//     reg v7   = texture coords
const char shaderCode[] =
"vs.1.0                         // Shader version 1.0                 \n"\
"                                                                     \n"\
"m4x4   oPos,   v0,     c5      // multiply position with pvm matrix  \n"\
"                                                                     \n"\
"mov    r1,     c9              // move light position to r1          \n"\
"dp3    r1.w,   r1,     r1      // normalize it                       \n"\
"rsq    r1.w,   r1.w                                                  \n"\
"mul    r1,     r1,     r1.w                                          \n"\
"                                                                     \n"\
"m3x3   r3,     v3,     c1                                            \n"\
"dp3    r3.w,   r3,     r3      // normalize it                       \n"\
"rsq    r3.w,   r3.w                                                  \n"\
"mul    r3,     r3,     r3.w                                          \n"\
"                                                                     \n"\
"dp3    r3.w,   r3,     r1                                            \n"\
"max    r3.w,   r3.w,   c0.x    // clamp texture coordinate at 0      \n"\
"                                                                     \n"\
"mov    oT0.x,  r3.w            // move texture coordinates           \n"\
"mov    oT0.y,  c0.y            // move texture coordinates           \n";





//------------------------------------------------------------------------------
// Archive
//------------------------------------------------------------------------------

// Constants
//      reg c0   = { 0.0f, 0.5f, 1.0f, 2.0f }
//      reg c1-4 = world matrix
//      reg c5-8 = proj*view*world matrix
//      reg c9   = camera position = light position
// Stream 0
//     reg v0   = position
//     reg v3   = normal
//     reg v7   = texture coords
const char quadLikeShaderCode[] =
"vs.1.0                         // Shader version 1.0               \n"\
"                                                                   \n"\
"m4x4   oPos,   v0,     c5      // multiply position with pvm matrix\n"\
"m3x3   r1,     v3,     c1      // transform normal to world space  \n"\
"dp3    r2,     r1,     c9      // dot product with light direction \n"\
"max    r2,     r2,     c0.x    // clamp r2 at 0                    \n"\
"mov    oT0.x,  r2              // move texture coordinates         \n"\
"mov    oT0.y,  c0.y            // move texture coordinates         \n";


// Constants
//      reg c5-8 = proj*view*world matrix
// Stream 0
//     reg v0   = position
//     reg v3   = normal
//     reg v7   = texture coords
const char textureNoLightShaderCode[] =
"vs.1.0                         // Shader version 1.0               \n"\
"                                                                   \n"\
"m4x4   oPos,   v0,     c5      // multiply position with pvm matrix\n"\
"mov    oT0.xy, v7              // move texture coordinates         \n";


// Constants
//      reg c0   = { 0.0f, 0.5f, 1.0f, 2.0f }
//      reg c1-4 = world matrix
//      reg c5-8 = proj*view*world matrix
//      reg c9   = camera position
// Stream 0
//     reg v0   = position
//     reg v3   = normal
//     reg v7   = texture coords
const char textureAndLightShaderCode[] =
"vs.1.0                         // Shader version 1.0                 \n"\
"                                                                     \n"\
"m4x4   oPos,   v0,     c5      // multiply position with pvm matrix  \n"\
"                                                                     \n"\
"mov    r1,     c9              // move light position to r1          \n"\
"dp3    r1.w,   r1,     r1      // normalize it to get a light vector \n"\
"rsq    r1.w,   r1.w                                                  \n"\
"mul    r1,     r1,     r1.w                                          \n"\
"                                                                     \n"\
"m3x3   r3,     v3,     c1      // move normal to r3                  \n"\
"dp3    r3.w,   r3,     r3      // normalize it                       \n"\
"rsq    r3.w,   r3.w                                                  \n"\
"mul    r3,     r3,     r3.w                                          \n"\
"                                                                     \n"\
"mov    r2,     c0.z            // base color is white                \n"\
"dp3    r3.w,   r3,     r1      // dot normal with light              \n"\
"mul    oD0,    r2,     r3.w    // set destination color              \n"\
"                                                                     \n"\
"mov    oT0.xy, v7              // move texture coordinates           \n";


// Constants
//      reg c0   = { 0.0f, 0.5f, 1.0f, 2.0f }
//      reg c1-4 = world matrix
//      reg c5-8 = proj*view*world matrix
//      reg c9   = camera position
// Stream 0
//     reg v0   = position
//     reg v3   = normal
//     reg v7   = texture coords
const char toonShaderCode[] =
"vs.1.0                         // Shader version 1.0                 \n"\
"                                                                     \n"\
"m4x4   oPos,   v0,     c5      // multiply position with pvm matrix  \n"\
"                                                                     \n"\
"mov    r1,     c9              // move light position to r1          \n"\
"dp3    r1.w,   r1,     r1      // normalize it                       \n"\
"rsq    r1.w,   r1.w                                                  \n"\
"mul    r1,     r1,     r1.w                                          \n"\
"                                                                     \n"\
"m3x3   r3,     v3,     c1                                            \n"\
"dp3    r3.w,   r3,     r3      // normalize it                       \n"\
"rsq    r3.w,   r3.w                                                  \n"\
"mul    r3,     r3,     r3.w                                          \n"\
"                                                                     \n"\
"dp3    r3.w,   r3,     r1                                            \n"\
"max    r3.w,   r3.w,   c0.x    // clamp texture coordinate at 0      \n"\
"                                                                     \n"\
"mov    oT0.x,  r3.w            // move texture coordinates           \n"\
"mov    oT0.y,  c0.y            // move texture coordinates           \n";

