//------------------------------------------------------------------------------
// (c) 04-2001 Gottfried Chen
//------------------------------------------------------------------------------

#ifndef DIRECTX_RENDERER_X_IMPORTER_H
#define DIRECTX_RENDERER_X_IMPORTER_H

#include <System/Types.h>

struct ID3DXMesh; 
struct _D3DMATERIAL8;
struct IDirect3DTexture8;
struct ID3DXBuffer;

namespace Ge
{

class Mesh;
class DirectXRenderer;
class Cluster;
class Material;

// Imports Microsoft .x file models
// This class cannot be used directly. Use the ModelPool instead.
class XImporter
{
public:
    ~XImporter();

    // Transforms to a model
    // <importMaterialAndTexture> -> see ObjImporter.h
    Cluster* import(bool importMaterialAndTexture,
                    bool recalculateNormals,
                    bool flipUvs) const;

    UInt getMaterialCount() const;

private:
    friend class ModelPool;

    XImporter(const char* file, const DirectXRenderer& renderer);

    // Construct a subtree corresponding to the DirectX mesh
    Cluster* constructCluster(bool importMaterialAndTexture,
                              Mesh* meshes[],
                              Material* materials[]) const;
    // Constructs an array of Mesh pointers, where each pointer points to
    // a Mesh corresponding to a material. The returned pointer must be
    // deleted with GE_DELETE_A after use.
    Mesh** constructMeshes(bool recalculateNormals, bool flipUvs) const;
    // Constructs an array of Material pointers, where each pointer points to
    // a Material for a sub mesh. The returned pointer must be
    // deleted with GE_DELETE_A after use.
    Material** constructMaterials() const;

    const _D3DMATERIAL8* getD3dMaterial(UInt index) const;

    ID3DXMesh* mMesh;
    UInt mMaterialCount;
    _D3DMATERIAL8* mMaterials;
    IDirect3DTexture8** mTextures;
    ID3DXBuffer* mMaterialBuffer;
    const DirectXRenderer& mRenderer;
    mutable bool mWasImported;
};

} // namespace Ge

#endif // DIRECTX_RENDERER_X_IMPORTER_H