#include "Precompiled.h"
//------------------------------------------------------------------------------
// (c) 04-2001 Gottfried Chen
//------------------------------------------------------------------------------

#include <Engine/AudioVisual.h>
#include <Engine/Camera.h>
#include <Engine/Sorter.h>
#include <Engine/KeyframeController.h>
#include <Engine/HierarchyAnimation.h>

namespace Ge
{

//------------------------------------------------------------------------------
AudioVisual::AudioVisual(const char* name) :
//------------------------------------------------------------------------------
mParent(0),
mName(name)
{
}

//------------------------------------------------------------------------------
void AudioVisual::setParent(AudioVisual* parent)
//------------------------------------------------------------------------------
{
    mParent = parent;
}

//------------------------------------------------------------------------------
const AudioVisual* AudioVisual::getParent() const
//------------------------------------------------------------------------------
{
    return mParent;
}

//------------------------------------------------------------------------------
void AudioVisual::accumulate(Sorter& sorter, const Camera& camera,
                             bool continueCulling)
//------------------------------------------------------------------------------
{
    if (cull(camera, continueCulling) != Intersect::DISJOINT)
        sorter.accumulate(*this);
}

//------------------------------------------------------------------------------
Intersect::Type AudioVisual::cull(const Camera& camera,
                                  bool continueCulling) const
//------------------------------------------------------------------------------
{
    return continueCulling ? camera.getWorldFrustum().intersects(getBound()) :
                             Intersect::CONTAINS;
}

//------------------------------------------------------------------------------
void AudioVisual::updateTransform()
//------------------------------------------------------------------------------
{
    mLocalTransform.asWorldTransform(mWorldTransform);
    if (getParent() != 0)
        mWorldTransform.multiply(mParent->worldTransform(), mWorldTransform);
}

//------------------------------------------------------------------------------
LocalTransform& AudioVisual::localTransform()
//------------------------------------------------------------------------------
{
    return mLocalTransform;
}

//------------------------------------------------------------------------------
const LocalTransform& AudioVisual::localTransform() const
//------------------------------------------------------------------------------
{
    return mLocalTransform;
}

//------------------------------------------------------------------------------
const WorldTransform& AudioVisual::worldTransform() const
//------------------------------------------------------------------------------
{
    return mWorldTransform;
}

//------------------------------------------------------------------------------
const Material& AudioVisual::getMaterial() const
//------------------------------------------------------------------------------
{
    return mMaterial;
}

//------------------------------------------------------------------------------
void AudioVisual::set(const Material& m)
//------------------------------------------------------------------------------
{
    mMaterial = m;
}

//------------------------------------------------------------------------------
StateBlock AudioVisual::getStateBlock() const
//------------------------------------------------------------------------------
{
    return mStates;
}

//------------------------------------------------------------------------------
void AudioVisual::setStateBlock(const StateBlock& s)
//------------------------------------------------------------------------------
{
    mStates = s;
}

//------------------------------------------------------------------------------
void AudioVisual::updateBoundUp()
//------------------------------------------------------------------------------
{
    if (getParent() != 0)
        mParent->updateBoundUp();
}

//------------------------------------------------------------------------------
const Sphere& AudioVisual::getBound() const
//------------------------------------------------------------------------------
{
    return mBound;
}

//------------------------------------------------------------------------------
void AudioVisual::setBound(const Sphere& sphere)
//------------------------------------------------------------------------------
{
    mBound = sphere;
}

//------------------------------------------------------------------------------
void AudioVisual::updateGeometry()
//------------------------------------------------------------------------------
{
    updateTransform();
    updateBoundDown();
    AudioVisual::updateBoundUp();
}


//------------------------------------------------------------------------------
void AudioVisual::copyDataFrom(const AudioVisual& av)
//------------------------------------------------------------------------------
{
    localTransform() = av.localTransform();
    setName(av.getName().c_str());
    mStates = av.mStates;
    mMaterial = av.mMaterial;
    // Stuff like world transform/render states, parent and bound are
    // not copied, since they have to be recalculated anyway.
}

//------------------------------------------------------------------------------
void AudioVisual::attach(KeyframeController& kfc)
//------------------------------------------------------------------------------
{
//    mControllers.push_back(Pointer<TimeController>(&kfc));
//    kfc.attachTo(*this);
}

//------------------------------------------------------------------------------
bool AudioVisual::tick(const Timer::Unit& time)
//------------------------------------------------------------------------------
{
    for (UInt i = 0; i<mControllers.size(); i++)
    {
        if (!mControllers[i]->tick(time) && mControllers[i]->isAutoDelete())
            mControllers.erase(mControllers.begin()+i);
    }

    return mControllers.size() > 0;
}

//------------------------------------------------------------------------------
Node* AudioVisual::asNode()
//------------------------------------------------------------------------------
{
    return 0;
}

//------------------------------------------------------------------------------
const Node* AudioVisual::asNode() const
//------------------------------------------------------------------------------
{
    return 0;
}

//------------------------------------------------------------------------------
const std::string& AudioVisual::getName() const
//------------------------------------------------------------------------------
{
    return mName;
}

//------------------------------------------------------------------------------
void AudioVisual::setName(const char* name)
//------------------------------------------------------------------------------
{
    mName = name;
}

//------------------------------------------------------------------------------
void AudioVisual::attach(HierarchyAnimation& animation)
//------------------------------------------------------------------------------
{
//    animation.attachMatchingControllers(*this);
}

//------------------------------------------------------------------------------
void AudioVisual::printSubTree(std::ostream& os, UInt indentLevel) const
//------------------------------------------------------------------------------
{
    if (indentLevel > 0)
    {
        for (UInt i = 0; i<indentLevel-2; i += 2)
            os<<"| ";

        os<<"|-";
    }

    os<<getName()<<std::endl;
}

} // namespace Ge