#include "Precompiled.h"
//------------------------------------------------------------------------------
// (c) 05-2001 Gottfried Chen
//------------------------------------------------------------------------------

#include <Engine/DefaultSorter.h>
#include <Engine/AudioVisual.h>
#include <Engine/Renderer.h>
#include <Engine/Camera.h>

#include "StateBlock.h"

#include <algorithm>

namespace Ge
{
    
//------------------------------------------------------------------------------
void DefaultSorter::accumulate(AudioVisual& object)
//------------------------------------------------------------------------------
{
    RenderState::Blend blend = RenderState::Blend(
        object.getStateBlock().get(RenderState::BLEND));
    
    if (blend != RenderState::BLEND_OFF)
    {
        // Current alpha sort algorithm: project bounding sphere center
        // onto camera look vector and sort by projection length.
        // The objects are then drawn from back to front.
        // This doesn't take into account e.g. transparent objects with
        // big bounding spheres that have a center in front a small transparent
        // object but still occlude it and should thus be drawn first.
        Vector3 cameraPosition;
        getCurrentCamera().getLocator().getGlobal().getColumn(MatrixBase::TRANSLATION,
                                                              cameraPosition);
        Vector3 cameraToObject(object.getBound().getCenter() - cameraPosition);

        Vector3 cameraAt;
        getCurrentCamera().getLocator().getGlobal().getColumn(MatrixBase::AT, cameraAt);
        Vector3 projection(
            cameraToObject.projectOn(cameraAt));

        mAlpha.push_back(AlphaObject(projection.lengthSquare(), &object));
    }
    else
        mOpaque.push_back(&object);
}

//------------------------------------------------------------------------------
void DefaultSorter::render(Renderer& renderer)
//------------------------------------------------------------------------------
{
    for (UInt i = 0; i<mOpaque.size(); ++i)
    {
        renderer.set(mOpaque[i]->getStateBlock());
        renderer.set(mOpaque[i]->getMaterial());
        mOpaque[i]->render(renderer);
    }

    if (mAlpha.size() > 0)
    {
        std::sort(mAlpha.begin(), mAlpha.end());
        // Draw from back to front (j is Int and not UInt because of j>=0!)
        for (Int j = mAlpha.size()-1; j>=0; --j)
        {
            renderer.set(mAlpha[j].mObject->getStateBlock());
            renderer.set(mAlpha[j].mObject->getMaterial());
            mAlpha[j].mObject->render(renderer);
        }
    }

    // Clear lists for next frame
    mOpaque.clear();
    mAlpha.clear();
}

} // namespace Ge