#include "Precompiled.h"
//------------------------------------------------------------------------------
// (c) 04-2001 Gottfried Chen
//------------------------------------------------------------------------------

#include <Engine/Node.h>


namespace Ge
{

//------------------------------------------------------------------------------
Node::Node(const char* name) :
//------------------------------------------------------------------------------
AudioVisual(name)
{
}

//------------------------------------------------------------------------------
void Node::addChild(AudioVisual& audioVisual)
//------------------------------------------------------------------------------
{
    audioVisual.setParent(this);
    mChildren.push_back(Pointer<AudioVisual>(&audioVisual));
}

//------------------------------------------------------------------------------
void Node::updateTransform()
//------------------------------------------------------------------------------
{
    // Update own world matrix then update all childrens world matrix
    AudioVisual::updateTransform();
    for (ChildVector::const_iterator i = mChildren.begin(); i!=mChildren.end(); i++)
        (*i)->updateTransform();
}

//------------------------------------------------------------------------------
void Node::render(Renderer&)
//------------------------------------------------------------------------------
{
    GE_ASSERT_MSG(false, "Tried to render a Node");
}

//------------------------------------------------------------------------------
void Node::accumulate(Sorter& sorter, const Camera& camera, bool continueCulling)
//------------------------------------------------------------------------------
{
    Intersect::Type intersect(cull(camera, continueCulling));
    if (intersect == Intersect::CONTAINS)
        accumulateChildren(sorter, camera, false);
    else if (intersect == Intersect::INTERSECTS)
        accumulateChildren(sorter, camera, true);
}

//------------------------------------------------------------------------------
void Node::accumulateChildren(Sorter& sorter, const Camera& camera,
                              bool continueCulling)
//------------------------------------------------------------------------------
{
    for (ChildVector::const_iterator i = mChildren.begin();
         i!=mChildren.end(); i++)
        (*i)->accumulate(sorter, camera, continueCulling);
}

//------------------------------------------------------------------------------
void Node::updateBoundUp()
//------------------------------------------------------------------------------
{
    mergeChildrenBounds();
    AudioVisual::updateBoundUp();
}

//------------------------------------------------------------------------------
void Node::updateBoundDown()
//------------------------------------------------------------------------------
{
    for (UInt i = 0; i<mChildren.size(); i++)
        mChildren[i]->updateBoundDown();

    mergeChildrenBounds();
}

//------------------------------------------------------------------------------
void Node::mergeChildrenBounds()
//------------------------------------------------------------------------------
{
    if (mChildren.size() > 0)
    {
        Sphere bound(mChildren[0]->getBound());
        for (UInt i = 1; i<mChildren.size(); i++)
            bound.merge(mChildren[i]->getBound());

        setBound(bound);
    }
}

//------------------------------------------------------------------------------
void Node::copyDataFrom(const Node& node)
//------------------------------------------------------------------------------
{
    AudioVisual::copyDataFrom(node);
    AudioVisual* childClone = 0;
    for (UInt i = 0; i<node.getChildCount(); i++)
    {
        childClone = node.child(i).clone();
        GE_ASSERT(childClone != 0);
        addChild(*childClone);
    }
}

//------------------------------------------------------------------------------
AudioVisual* Node::clone() const
//------------------------------------------------------------------------------
{
    Node* ret = GE_NEW Node;
    ret->copyDataFrom(*this);
    return ret;
}

//------------------------------------------------------------------------------
UInt Node::getChildCount() const
//------------------------------------------------------------------------------
{
    return mChildren.size();
}

//------------------------------------------------------------------------------
const AudioVisual& Node::child(UInt index) const
//------------------------------------------------------------------------------
{
    GE_ASSERT(index < getChildCount());
    return *(mChildren[index]);
}

//------------------------------------------------------------------------------
AudioVisual& Node::child(UInt index)
//------------------------------------------------------------------------------
{
    GE_ASSERT(index < getChildCount());
    return *(mChildren[index]);
}

//------------------------------------------------------------------------------
Node* Node::asNode()
//------------------------------------------------------------------------------
{
    return this;
}

//------------------------------------------------------------------------------
const Node* Node::asNode() const
//------------------------------------------------------------------------------
{
    return this;
}

//------------------------------------------------------------------------------
void Node::attach(HierarchyAnimation& animation)
//------------------------------------------------------------------------------
{
    AudioVisual::attach(animation);
    for (UInt i = 0; i<mChildren.size(); i++)
        mChildren[i]->attach(animation);
}

//------------------------------------------------------------------------------
void Node::printSubTree(std::ostream& os, UInt indentLevel) const
//------------------------------------------------------------------------------
{
    AudioVisual::printSubTree(os, indentLevel);
    for (UInt i = 0; i<mChildren.size(); i++)
        mChildren[i]->printSubTree(os, indentLevel+2);
}

} // namespace Ge