//------------------------------------------------------------------------------
// (c) 04-2001 Gottfried Chen
//------------------------------------------------------------------------------

#ifndef ENGINE_NODE_H
#define ENGINE_NODE_H

#include <Engine/AudioVisual.h>
#include <System/Pointer.h>
#include <vector>

namespace Ge
{

class Renderer;
class Camera;
class Sorter;

// Node is the base class for all nodes in the scene tree that are used for
// grouping objects.
class Node : public AudioVisual
{
public:
    Node(const char* name = "NoName");

    void addChild(AudioVisual& audioVisual);
    UInt getChildCount() const;
    const AudioVisual& child(UInt) const;
    AudioVisual& child(UInt);

    virtual
    void accumulate(Sorter&, const Camera&, bool continueCulling);

    virtual
    void attach(HierarchyAnimation&);

    virtual
    void render(Renderer&);

    virtual
    AudioVisual* clone() const;

    virtual
    Node* asNode();
    virtual
    const Node* asNode() const;

    virtual
    void printSubTree(std::ostream&, UInt indentLevel = 0) const;

protected:
    // Copy constructor replacement, so overwriting the AudioVisual's non
    // virtual member is intentional here, since AudioVisual::copyDataFrom()
    // is called in this function.
    void copyDataFrom(const Node&);

    // Traverses the subtree starting at *this downwards and updates all
    // world transforms recursively
    virtual
    void updateTransform();

    virtual
    void updateBoundUp();
    virtual
    void updateBoundDown();

    void accumulateChildren(Sorter& sorter, const Camera& camera,
                            bool continueCulling);

private:
    // Don't want a copy constructor, use clone instead
    Node(const Node&);

    void mergeChildrenBounds();

    // Use smart pointer for children, since children can be shared
    // (e.g. models that are placed multiple times in the world)
    typedef std::vector<Pointer<AudioVisual> > ChildVector;

    ChildVector mChildren;
};

} // namespace Ge

#endif // ENGINE_NODE_H