//------------------------------------------------------------------------------
// (c) 05-2001 Gottfried Chen
//------------------------------------------------------------------------------

#ifndef ENGINE_SORTER_H
#define ENGINE_SORTER_H

#include <System/Object.h>

namespace Ge
{

class AudioVisual;
class Renderer;
class Camera;

// Before the World is rendered an accumulation pass is done. The World is
// traversed recursively. Every AudioVisual, that is not culled, enters itself
// into the sorter using Sorter::accumulate().
// Start the recursion by calling World::getRoot()->accumulate(sorter)
class Sorter : public Object
{
public:
    Sorter();

    // Sets/gets the camera for this frame (the camera is stored in a pointer,
    // so you must assure that the camera lives long enough).
    void setCurrentCamera(const Camera& camera);
    const Camera& getCurrentCamera() const;

    virtual
    void accumulate(AudioVisual&) = 0;

    virtual
    void render(Renderer&) = 0;

private:
    const Camera* mCamera;
};

} // namespace Ge

#endif // ENGINE_SORTER_H