//------------------------------------------------------------------------------
// (c) 05-2001 Gottfried Chen
//------------------------------------------------------------------------------

#include <Engine/StateBuffer.h>

namespace Ge
{

//------------------------------------------------------------------------------
StateBuffer::StateBuffer() :
//------------------------------------------------------------------------------
mCursor(0)
{
}

//------------------------------------------------------------------------------
void StateBuffer::merge(const StateBuffer& buffer)
//------------------------------------------------------------------------------
{
    buffer.begin();
    const RenderState* state;
    while ((state = buffer.next()) != 0)
    {
        // Add state to *this, if type not already present
        if (get(state->getType()) == 0)
            set(*state);
    }
}

//------------------------------------------------------------------------------
void StateBuffer::set(const RenderState& state)
//------------------------------------------------------------------------------
{
    mMap[state.getType()] = &state;
}

//------------------------------------------------------------------------------
void StateBuffer::remove(const RenderState& state)
//------------------------------------------------------------------------------
{
    mMap.erase(state.getType());
}

//------------------------------------------------------------------------------
bool StateBuffer::contains(const RenderState& state) const
//------------------------------------------------------------------------------
{
    bool ret(true);
    Map::const_iterator it(mMap.find(state.getType()));
    if (it != mMap.end())
        ret = it->second->equals(state);
    else
        ret = false;

    return ret;
}

//------------------------------------------------------------------------------
const RenderState* StateBuffer::get(RenderState::Type type) const
//------------------------------------------------------------------------------
{
    return const_cast<StateBuffer*>(this)->get(type);
}

//------------------------------------------------------------------------------
RenderState* StateBuffer::get(RenderState::Type type)
//------------------------------------------------------------------------------
{
    RenderState* state = 0;
    Map::const_iterator it(mMap.find(type));
    if (it != mMap.end())
        state = it->second;

    return state;
}


//------------------------------------------------------------------------------
void StateBuffer::begin() const
//------------------------------------------------------------------------------
{
    mCursor = 0;
}

//------------------------------------------------------------------------------
const RenderState* StateBuffer::next() const
//------------------------------------------------------------------------------
{
    const RenderState* state = 0;
    while (mCursor < RenderState::TYPE_COUNT && state == 0)
    {
        state = get(RenderState::Type(mCursor));
        mCursor++;
    }

    return state;
}

} // namespace Ge