//------------------------------------------------------------------------------
// (c) 05-2001 Gottfried Chen
//------------------------------------------------------------------------------

#ifndef ENGINE_STATE_BUFFER_H
#define ENGINE_STATE_BUFFER_H

#include <System/Pointer.h>
#include <Engine/RenderState.h>
#include <map>

namespace Ge
{

// This class is used for storing render states for an AudioVisual. Every
// AudioVisual has a local and a world state buffer. The local state buffer
// contains the local set render states. The world state buffer contains the
// accumulated states starting from the root.
// RenderStates in the StateBuffer must live on the heap even though they
// are small in size (e.g. AlphaState). This is needed so that they can be
// stored in a Pointer and thus be used polymorphically.
class StateBuffer
{
public:
    StateBuffer();

    //------------------------------------------------------------------------------
    // Update recursion
    //------------------------------------------------------------------------------
    // If a render state of a certain type is present in <buffer> and <*this>
    // then the local render state has priority
    void merge(const StateBuffer& buffer);

    //------------------------------------------------------------------------------
    // Access
    //------------------------------------------------------------------------------
    void set(const RenderState&);
    void remove(const RenderState&);
    bool contains(const RenderState&) const;
    const RenderState* get(RenderState::Type) const;
    RenderState* get(RenderState::Type);

    //------------------------------------------------------------------------------
    // Traversing
    //------------------------------------------------------------------------------
    void begin() const;
    const RenderState* next() const;

private:
    typedef std::map<RenderState::Type, Pointer<RenderState> > Map;
    Map mMap;
    mutable UInt mCursor;
};

} // namespace Ge

#endif // ENGINE_STATE_BUFFER_H
