//------------------------------------------------------------------------------
// (c) 06-2001 Gottfried Chen
//------------------------------------------------------------------------------

#include <Engine/TextureState.h>
#include <Engine/Renderer.h>

namespace Ge
{

//------------------------------------------------------------------------------
TextureState::TextureState(const Texture& texture) :
//------------------------------------------------------------------------------
RenderState(TEXTURE_SET),
mTexture(&texture),
mCursor(true),
mIsNull(false)
{
}

//------------------------------------------------------------------------------
TextureState::TextureState() :
//------------------------------------------------------------------------------
RenderState(TEXTURE_SET),
mTexture(0),
mCursor(true),
mIsNull(true)
{
}

//------------------------------------------------------------------------------
bool TextureState::equals(const RenderState& state) const
//------------------------------------------------------------------------------
{
    bool ret(false);
    if (getType() == state.getType())
    {
        if (!isNull())
            ret = (mTexture->getName() ==
                   static_cast<const TextureState&>(state).mTexture->getName());
        else
            ret = static_cast<const TextureState&>(state).isNull();
    }

    return ret;
}

//------------------------------------------------------------------------------
bool TextureState::isNull() const
//------------------------------------------------------------------------------
{
    return mIsNull;
}

//------------------------------------------------------------------------------
void TextureState::set(const Renderer& renderer) const
//------------------------------------------------------------------------------
{
    renderer.set(*this);
}

//------------------------------------------------------------------------------
void TextureState::begin() const
//------------------------------------------------------------------------------
{
    GE_ASSERT(!isNull());
    mCursor = true;
}

//------------------------------------------------------------------------------
const Texture* TextureState::next() const
//------------------------------------------------------------------------------
{
    GE_ASSERT(!isNull());
    if (mCursor)
    {
        mCursor = false;
        return mTexture;
    }
    else
        return 0;
}

} // namespace Ge