#include "Precompiled.h"
//------------------------------------------------------------------------------
// (c) 04-2001 Gottfried Chen
//------------------------------------------------------------------------------

#include <Engine/WorldTransform.h>

namespace Ge
{

//------------------------------------------------------------------------------
WorldTransform::WorldTransform() :
//------------------------------------------------------------------------------
mScale(1),
mTranslation(Vector3::ORIGIN())
{
    mMatrix = Matrix3x3::IDENTITY();
}

//------------------------------------------------------------------------------
const Vector3& WorldTransform::getTranslation() const
//------------------------------------------------------------------------------
{
    return mTranslation;
}

//------------------------------------------------------------------------------
const Matrix3x3& WorldTransform::getMatrix3x3() const
//------------------------------------------------------------------------------
{
    return mMatrix;
}

//------------------------------------------------------------------------------
void WorldTransform::multiply(const WorldTransform& second,
                              const WorldTransform& first)
//------------------------------------------------------------------------------
{
    mScale = second.mScale*first.mScale;
    mTranslation = second.getMatrix3x3()*first.getTranslation()
                   + second.getTranslation();
    mMatrix = second.getMatrix3x3()*first.getMatrix3x3();
}

//------------------------------------------------------------------------------
const Float& WorldTransform::getScale() const
//------------------------------------------------------------------------------
{
    return mScale;
}

//------------------------------------------------------------------------------
void WorldTransform::getMatrix4x4(Matrix4x4& matrix) const
//------------------------------------------------------------------------------
{
    // Set scale and rotation
    for (int i = 0; i<3; i++)
        for (int j = 0; j<3; j++)
            matrix(i,j) = mMatrix(i,j);

    // Add translation
    matrix(0,3) = getTranslation()[0];
    matrix(1,3) = getTranslation()[1];
    matrix(2,3) = getTranslation()[2];

    // Fill up last row
    matrix(3,0) = matrix(3,1) = matrix(3,2) = 0;
    matrix(3,3) = 1;
}

} // namespace Ge