#include "Precompiled.h"
//------------------------------------------------------------------------------
// (c) Jun-2002 Gottfried Chen
//------------------------------------------------------------------------------

#include "Box.h"

#include "Mesh.h"

namespace Ge
{

//------------------------------------------------------------------------------
Mesh* Box::create(Float x, Float y, Float z)
//------------------------------------------------------------------------------
{
    Vector3* vertices = GE_NEW Vector3[24];
    Vector3* normals = GE_NEW Vector3[24];
    UInt16* indices = GE_NEW UInt16[36];

    x /= 2.0f;
    y /= 2.0f;
    z /= 2.0f;

    // Front
    vertices[0] = Vector3(-x,y,-z);
    vertices[1] = Vector3(x,y,-z);
    vertices[2] = Vector3(x,-y,-z);
    vertices[3] = Vector3(-x,-y,-z);

    normals[0] = normals[1] = normals[2] = normals[3] = Vector3(0,0, -1);
    
    indices[0] = 0;
    indices[1] = 1;
    indices[2] = 2;
    indices[3] = 0;
    indices[4] = 2;
    indices[5] = 3;

    // Back
    vertices[4] = Vector3(-x,y,z);
    vertices[5] = Vector3(x,y,z);
    vertices[6] = Vector3(x,-y,z);
    vertices[7] = Vector3(-x,-y,z);

    normals[4] = normals[5] = normals[6] = normals[7] = Vector3(0,0,1);
    
    indices[6] = 4;
    indices[7] = 6;
    indices[8] = 5;
    indices[9] = 4;
    indices[10] = 7;
    indices[11] = 6;

    // Top
    vertices[8] = Vector3(-x,y,z);
    vertices[9] = Vector3(x,y,z);
    vertices[10] = Vector3(x,y,-z);
    vertices[11] = Vector3(-x,y,-z);

    normals[8] = normals[9] = normals[10] = normals[11] = Vector3(0,1,0);

    indices[12] = 8;
    indices[13] = 9;
    indices[14] = 10;
    indices[15] = 8;
    indices[16] = 10;
    indices[17] = 11;

    // Bottom
    vertices[12] = Vector3(-x,-y,z);
    vertices[13] = Vector3(x,-y,z);
    vertices[14] = Vector3(x,-y,-z);
    vertices[15] = Vector3(-x,-y,-z);

    normals[12] = normals[13] = normals[14] = normals[15] = Vector3(0,-1,0);

    indices[18] = 12;
    indices[19] = 14;
    indices[20] = 13;
    indices[21] = 12;
    indices[22] = 15;
    indices[23] = 14;

    // Left
    vertices[16] = Vector3(-x,y,z);
    vertices[17] = Vector3(-x,y,-z);
    vertices[18] = Vector3(-x,-y,-z);
    vertices[19] = Vector3(-x,-y,z);

    normals[16] = normals[17] = normals[18] = normals[19] = Vector3(-1,0,0);

    indices[24] = 16;
    indices[25] = 17;
    indices[26] = 18;
    indices[27] = 16;
    indices[28] = 18;
    indices[29] = 19;

    // Right
    vertices[20] = Vector3(x,y,z);
    vertices[21] = Vector3(x,y,-z);
    vertices[22] = Vector3(x,-y,-z);
    vertices[23] = Vector3(x,-y,z);

    normals[20] = normals[21] = normals[22] = normals[23] = Vector3(1,0,0);

    indices[30] = 20;
    indices[31] = 22;
    indices[32] = 21;
    indices[33] = 20;
    indices[34] = 23;
    indices[35] = 22;

    return GE_NEW Mesh(1,
                       24, vertices, normals, 0, 0,
                       36, indices);
}

} // namespace Ge
