#include "Precompiled.h"
//------------------------------------------------------------------------------
// (c) 04-2001 Gottfried Chen
//------------------------------------------------------------------------------

#include "Camera.h"

namespace Ge
{

//------------------------------------------------------------------------------
Camera::Camera(Locator& l) :
//------------------------------------------------------------------------------
mAspectRatio(4.0f/3.0f),
mFov(PI_HALF),
mNear(1.0f),
mFar(1000.0f),
mLocator(&l)
{
    recalculateFrustum();
}

//------------------------------------------------------------------------------
void Camera::getProjectionMatrix(Matrix4x4& m) const
//------------------------------------------------------------------------------
{
    makeProjection(m, mNear, mFar, mFov, mAspectRatio);
}

//------------------------------------------------------------------------------
const Frustum& Camera::getWorldFrustum() const
//------------------------------------------------------------------------------
{
    if (mFrustumDirty)
    {
        recalculateFrustum();
    }

    return mFrustum;
}

//------------------------------------------------------------------------------
const Locator& Camera::getLocator() const
//------------------------------------------------------------------------------
{
    return *mLocator;
}

//------------------------------------------------------------------------------
Locator& Camera::getLocator()
//------------------------------------------------------------------------------
{
    mFrustumDirty = true;
    return *mLocator;
}

//------------------------------------------------------------------------------
Float Camera::getAspectRatio() const
//------------------------------------------------------------------------------
{
    return mAspectRatio;
}

//------------------------------------------------------------------------------
void Camera::setAspectRatio(Float a)
//------------------------------------------------------------------------------
{
    GE_ASSERT(a > 0);
    mAspectRatio = a;
    mFrustumDirty = true;
}

//------------------------------------------------------------------------------
Float Camera::getFieldOfView() const
//------------------------------------------------------------------------------
{
    return mFov;
}

//------------------------------------------------------------------------------
void Camera::setFieldOfView(Float fov)
//------------------------------------------------------------------------------
{
    GE_ASSERT(0<fov && fov<PI);
    mFov = fov;
    mFrustumDirty = true;
}

//------------------------------------------------------------------------------
Float Camera::getViewportWidth() const
//------------------------------------------------------------------------------
{
    return mViewportWidth;
}

//------------------------------------------------------------------------------
Float Camera::getNearPlane() const
//------------------------------------------------------------------------------
{
    return mNear;
}

//------------------------------------------------------------------------------
void Camera::setNearPlane(Float n)
//------------------------------------------------------------------------------
{
    GE_ASSERT(n < mFar);
    mNear = n;
    mFrustumDirty = true;
}

//------------------------------------------------------------------------------
Float Camera::getFarPlane() const
//------------------------------------------------------------------------------
{
    return mFar;
}

//------------------------------------------------------------------------------
void Camera::setFarPlane(Float f)
//------------------------------------------------------------------------------
{
    GE_ASSERT(f > mNear);
    mFar = f;
    mFrustumDirty = true;
}

//------------------------------------------------------------------------------
// Private members
//------------------------------------------------------------------------------

//------------------------------------------------------------------------------
void Camera::recalculateFrustum() const
//------------------------------------------------------------------------------
{
    mFrustum = Frustum(mNear, mFar, mFov, mFov/mAspectRatio).
               transform(mLocator->getGlobal());
    
    mViewportWidth = 2*mNear*tan(mFov*0.5f);

    mFrustumDirty = false;
}

} // namespace Ge