#include "Precompiled.h"
//------------------------------------------------------------------------------
// (c) Jul-2002 Gottfried Chen
//------------------------------------------------------------------------------

#include "Cluster.h"

namespace Ge
{

//------------------------------------------------------------------------------
void Cluster::add(Link& l)
//------------------------------------------------------------------------------
{
    mLinks.push_back(Pointer<Link>(&l));
}

//------------------------------------------------------------------------------
const Cluster::Links& Cluster::getLinks() const
//------------------------------------------------------------------------------
{
    return mLinks;
}

//------------------------------------------------------------------------------
Cluster::Links& Cluster::getLinks()
//------------------------------------------------------------------------------
{
    return mLinks;
}


//------------------------------------------------------------------------------
const Locator* Cluster::getLocator() const
//------------------------------------------------------------------------------
{
    return mLocator;
}

//------------------------------------------------------------------------------
Locator* Cluster::getLocator()
//------------------------------------------------------------------------------
{
    return mLocator;
}

//------------------------------------------------------------------------------
void Cluster::set(const Locator& l)
//------------------------------------------------------------------------------
{
    mLocator = &l;
}

//------------------------------------------------------------------------------
Cluster* Cluster::clone() const
//------------------------------------------------------------------------------
{
    Cluster* ret = GE_NEW Cluster;
    Locator* newHierarchy = mLocator->clone();
    ret->set(*newHierarchy);

    Locator::Iterator newIt(newHierarchy->getBegin());
    Locator::Iterator oldIt(mLocator->getBegin());
    while (newIt != newHierarchy->getEnd() &&
           ret->mLinks.size()<mLinks.size())
    {
        for (UInt i = 0; i<mLinks.size(); ++i)
        {
            if (mLinks[i]->getLocator() == &(*oldIt))
            {
                Link* link = mLinks[i]->clone();
                link->set(*newIt);
                ret->add(*link);
            }
        }
        ++newIt;
        ++oldIt;
    }

    GE_ASSERT(ret->mLinks.size() == mLinks.size());

    return ret;
}

//------------------------------------------------------------------------------
void Cluster::setShadowCaster(bool s)
//------------------------------------------------------------------------------
{
    for (Links::iterator i = mLinks.begin(); i != mLinks.end(); ++i)
    {
        (*i)->setShadowCaster(s);
    }
}

//------------------------------------------------------------------------------
bool Cluster::isShadowCaster() const
//------------------------------------------------------------------------------
{
    return mLinks[0]->isShadowCaster();
}

////------------------------------------------------------------------------------
//void Cluster::updateShadowVolume() const
////------------------------------------------------------------------------------
//{
//    for (Links::const_iterator i = mLinks.begin(); i != mLinks.end(); ++i)
//    {
//        (*i)->updateShadowVolume();
//    }
//}

//------------------------------------------------------------------------------
void Cluster::setLight(UInt index, const Vector4& light) const
//------------------------------------------------------------------------------
{
    for (Links::const_iterator i = mLinks.begin(); i != mLinks.end(); ++i)
    {
        (*i)->setLight(index, light);
    }
}


//------------------------------------------------------------------------------
void Cluster::calculateBound()
//------------------------------------------------------------------------------
{
    bool first = true;
    for (Links::iterator i = mLinks.begin(); i != mLinks.end(); ++i)
    {
        (*i)->calculateBound();

        if (first)
        {
            mBound = (*i)->getBound();
            first = false;
        }
        else
        {
            mBound.merge((*i)->getBound());
        }
    }
}

//------------------------------------------------------------------------------
const Sphere& Cluster::getBound() const
//------------------------------------------------------------------------------
{
    return mBound;
}

} // namespace Ge
