#ifndef ENGINE_CLUSTER_H
#define ENGINE_CLUSTER_H
#pragma once

//------------------------------------------------------------------------------
// (c) Jul-2002 Gottfried Chen
//------------------------------------------------------------------------------

#include "Link.h"
#include "Locator.h"

#include <System/Object.h>
#include <System/Pointer.h>
#include <System/NoCopy.h>

#include <Math/Sphere.h>



namespace Ge
{

// This is a container for Links
class Cluster : public Object, NoCopy
{
public:
    typedef std::vector<Pointer<Link> > Links;

    // The Link must live on the heap.
    void add(Link&);

    Links& getLinks();

    const Links& getLinks() const;

    Locator* getLocator();

    const Locator* getLocator() const;

    // The Locator of a cluster is the root node of the Locator
    // hierarchy for the attached Links. The Locator must live on the heap.
    void set(const Locator&);

    // Creates a copy of the Locator hierarchy and attaches Links to this
    // copy according to the original Links.
    Cluster* clone() const;

    // Set all Links to be shadow casters.
    void setShadowCaster(bool);

    // Returns true if the first Link in this Cluster is a shadow caster!
    // So it does not check all Links whether they are shadow casters.
    bool isShadowCaster() const;

//    void updateShadowVolume() const;

    void setLight(UInt index, const Vector4& light) const;

    const Vector4& getLight(UInt index) const;

    // - The global transformations of the Locators in the Link list must
    // be up to date before this is called since they are used when merging
    // the bounds of the Meshes attached to the Links!
    // - The bounds of the Links don't need to be calculated. This is done in
    // this routine.
    void calculateBound();

    const Sphere& getBound() const;

private:
    Pointer<Locator> mLocator;
    Links mLinks;
    Sphere mBound;
};

} // namespace Ge

#endif // !ENGINE_CLUSTER_H