//------------------------------------------------------------------------------
// (c) 04-2001 Gottfried Chen
//------------------------------------------------------------------------------

#ifndef ENGINE_COLOR_H
#define ENGINE_COLOR_H

#include <System/Types.h>

#include <Math/Vector.h>

namespace Ge
{

// Color in RGBA in [0, 1] if specified as Float
// or [0, 255] if specified as Int
class Color
{
public:
    enum Format
    {
        RGBA,
        BGRA,
        ARGB // DirectX format
    };

    enum Channel
    {
        CH_RED,
        CH_GREEN,
        CH_BLUE,
        CH_ALPHA
    };

    static const Color WHITE;
    static const Color BLACK;
    static const Color RED;
    static const Color GREEN;
    static const Color BLUE;
    static const Color YELLOW;

    // Initial color is opaque white
    Color();
    
    Color(Float r, Float g, Float b, Float a);

    Color(Format, const Vector4&);

    Color(const Color&);

    bool operator ==(const Color&) const;

    bool operator !=(const Color&) const;

    void setFloat(Float r, Float g, Float b, Float a);

    void setInt(UInt r, UInt g, UInt b, UInt a);
    
    void set(Format, UInt32);

    void set(Format, const Vector4&);
       
    void getFloat(Float& r, Float& g, Float& b, Float& a) const;

    UInt32 get(Format) const;

    UInt8 get(Channel) const;

private:
    // Stored in RGBA
    UInt32 mData;
};

// Linear interpolation between <c0> and <c1>. <value> = 0 returns c0.
Color lerp(const Color& c0, const Color& c1, Float value);

// Convert a normalized vector to a Color. I.e. performs range compression. 
void convertColorToVector(Vector3&, const Color&);

// Convert a Color to a vector. I.e. uncompress the vector.
void convertVectorToColor(Color&, const Vector3&, Float alpha = 1.0f);

}

#endif // ENGINE_COLOR_H

