#include "Precompiled.h"
//------------------------------------------------------------------------------
// (c) 04-2001 Gottfried Chen
//------------------------------------------------------------------------------

#include <Engine/DisplayStatistics.h>

namespace Ge
{

//------------------------------------------------------------------------------
DisplayStatistics& DisplayStatistics::instance()
//------------------------------------------------------------------------------
{
    static DisplayStatistics stats;
    return stats;
}

//------------------------------------------------------------------------------
void DisplayStatistics::tick(const Timer::Unit& time)
//------------------------------------------------------------------------------
{
    // Fps are reset every second
    mFrames++;
    if ((time - mLastTimeStamp) > 1000)
    {
        mFps = mFrames;
        mFrames = 0;
        mLastTimeStamp = time;
    }

    // Vertices and triangles are reset every frame
    mVerticesLastFrame = mVertices;
    mTrianglesLastFrame = mTriangles;
    mVertices = 0;
    mTriangles = 0;
}

//------------------------------------------------------------------------------
void DisplayStatistics::render(UInt vertices, UInt triangles)
//------------------------------------------------------------------------------
{
    mVertices += vertices;
    mTriangles += triangles;
}

//------------------------------------------------------------------------------
UInt DisplayStatistics::getFramesPerSecond() const
//------------------------------------------------------------------------------
{
    return mFps;
}

//------------------------------------------------------------------------------
UInt DisplayStatistics::getVertices() const
//------------------------------------------------------------------------------
{
    return mVerticesLastFrame;
}

//------------------------------------------------------------------------------
UInt DisplayStatistics::getTriangles() const
//------------------------------------------------------------------------------
{
    return mTrianglesLastFrame;
}


//------------------------------------------------------------------------------
DisplayStatistics::DisplayStatistics() :
//------------------------------------------------------------------------------
mLastTimeStamp(0),
mFrames(0),
mFps(0),
mVertices(0),
mTriangles(0),
mVerticesLastFrame(0),
mTrianglesLastFrame(0)
{
}


} // namespace Ge