//------------------------------------------------------------------------------
// (c) 04-2001 Gottfried Chen
//------------------------------------------------------------------------------

#ifndef ENGINE_DISPLAYSTATISTICS_H
#define ENGINE_DISPLAYSTATISTICS_H

#include <System/Timer.h>
#include <System/Types.h>

namespace Ge
{

// Collects statistical data about frame rate and vertex/triangle count
class DisplayStatistics
{
public:
    static
    DisplayStatistics& instance();

    // Called every frame with the current time
    void tick(const Timer::Unit& time);
    // Called each time geometry is rendered
    void render(UInt vertices, UInt triangles);

    UInt getFramesPerSecond() const;
    // Returns the number of vertices/triangles drawn last frame
    UInt getVertices() const;
    UInt getTriangles() const;

private:
    DisplayStatistics();

    Timer::Unit mLastTimeStamp;
    UInt mFrames;
    UInt mFps;
    UInt mVertices;
    UInt mTriangles;
    UInt mVerticesLastFrame;
    UInt mTrianglesLastFrame;
};

inline
//------------------------------------------------------------------------------
DisplayStatistics& theDisplayStatistics()
//------------------------------------------------------------------------------
{
    return DisplayStatistics::instance();
}

} // namespace Ge

#endif // ENGINE_DISPLAYSTATISTICS_H