#ifndef FPS_CAMERA_CONTROLLER_H
#define FPS_CAMERA_CONTROLLER_H
#pragma once

//------------------------------------------------------------------------------
// (c) Jul-2002 Gottfried Chen
//------------------------------------------------------------------------------

#include <System/Types.h>

#include <Math/Vector.h>

#include <Input/DeviceButton.h>
#include <Input/VirtualButton.h>

namespace Ge
{

class Camera;

// Standard WASD or cursor keys + mouse first person shooter camera
class FpsCameraController
{
public:
    // The controller only stores a reference to the camera.
    // Just make sure the camera lives at least as long as the
    // controller.
    //
    // Default: camera is placed according to the passed in camera
    FpsCameraController(Camera&);

    bool isInverted() const;

    void setInvert(bool invert);

    void update();

    // Default sensitivity is 1.
    void setSensitivity(Float);

    Float getSensitivity() const;

private:
    
    Float mSensitivity;
    Float mUnitsPerSecond;
    Int mInvert;
    Camera& mCamera;

    KeyboardButton mA;
    KeyboardButton mD;
    KeyboardButton mW;
    KeyboardButton mS;
    KeyboardButton mCursorLeft;
    KeyboardButton mCursorRight;
    KeyboardButton mCursorUp;
    KeyboardButton mCursorDown;
    KeyboardButton mCtrl;
    KeyboardButton mC;
    KeyboardButton mSpace;
    MouseButton mRightMouse;

    VirtualButton mLeft;
    VirtualButton mRight;
    VirtualButton mForward;
    VirtualButton mBackward;
    VirtualButton mCrouch;
    VirtualButton mJump;
};

} // namespace Ge

#endif // !FPS_CAMERA_CONTROLLER_H