#include "Precompiled.h"
//------------------------------------------------------------------------------
// (c) 06-2001 Gottfried Chen
//------------------------------------------------------------------------------

#include "HierarchyAnimation.h"
#include "Locator.h"

#include <System/StringTools.h>
#include <System/Log.h>

namespace Ge
{

//------------------------------------------------------------------------------
HierarchyAnimation::HierarchyAnimation(const char* name) :
//------------------------------------------------------------------------------
mName(name),
mRepeatType(TimeController::LOOP),
mDuration(0),
mOffset(0)
{
}

//------------------------------------------------------------------------------
bool HierarchyAnimation::update(const Timer::Unit& time)
//------------------------------------------------------------------------------
{
    for (UInt i = 0; i<mEntries.size(); ++i)
    {
        if (mEntries[i].mWasMapped == true)
        {
            mEntries[i].mController->tick(time);
        }
    }

    bool over((time - mOffset) > getDuration());
    if (over && getRepeatType() == TimeController::LOOP)
    {
        reset(time);
    }

    return getRepeatType() == TimeController::ONE_SHOT ? !over : true;
}

//------------------------------------------------------------------------------
void HierarchyAnimation::unmapAll()
//------------------------------------------------------------------------------
{
    for (UInt i = 0; i<mEntries.size(); i++)
    {
        mEntries[i].mWasMapped = false;
    }
}

//------------------------------------------------------------------------------
void HierarchyAnimation::add(KeyframeController& kfc, const char* prefix)
//------------------------------------------------------------------------------
{
    Entry entry;
    kfc.set(TimeController::ONE_SHOT);
    entry.mController = &kfc;
    entry.mPrefix = prefix;
    mEntries.push_back(entry);
    recalcDuration(kfc);
}

//------------------------------------------------------------------------------
void HierarchyAnimation::reset(const Timer::Unit& time)
//------------------------------------------------------------------------------
{
    for (UInt i = 0; i<mEntries.size(); i++)
    {
        mEntries[i].mController->reset(time);
    }

    mOffset = time;
}

//------------------------------------------------------------------------------
void HierarchyAnimation::set(TimeController::Repeat repeat)
//------------------------------------------------------------------------------
{
    mRepeatType = repeat;
}

//------------------------------------------------------------------------------
TimeController::Repeat HierarchyAnimation::getRepeatType() const
//------------------------------------------------------------------------------
{
    return mRepeatType;
}

//------------------------------------------------------------------------------
void HierarchyAnimation::recalcDuration(KeyframeController& kfc)
//------------------------------------------------------------------------------
{
    if (kfc.getDuration() > getDuration())
    {
        mDuration = kfc.getDuration();
    }
}

//------------------------------------------------------------------------------
const Timer::Unit& HierarchyAnimation::getDuration() const
//------------------------------------------------------------------------------
{
    return mDuration;
}

//------------------------------------------------------------------------------
void HierarchyAnimation::attachTo(Locator& l)
//------------------------------------------------------------------------------
{
    Locator::Iterator it = l.getBegin();
    while (it != l.getEnd())
    {
        attachMatchingControllers(*it);
        GE_TRACE("Attaching controllers to: "<<it->getName()<</*"("<<
                 it->mChildren.size()<<" children)"<<*/std::endl);
        ++it;
    }
}

//------------------------------------------------------------------------------
void HierarchyAnimation::attachMatchingControllers(Locator& locator)
//------------------------------------------------------------------------------
{
    for (UInt i = 0; i<mEntries.size(); i++)
    {
        // Only the first instance of an Locator with the correct prefix
        // gets the controller mapped
        if (!mEntries[i].mWasMapped &&
            StringTools::beginsWith(mEntries[i].mPrefix, locator.getName()))
        {
            //av.attach(*(mEntries[i].mController));
            GE_LOG_DEBUG("AudioVisual "<<locator.getName().c_str()<<" attached a controller"<<std::endl);
            mEntries[i].mWasMapped = true;
            mEntries[i].mController->attachTo(locator);
        }
    }

    //mToBeTicked.push_back(&locator);
}

} // namespace Ge


