//------------------------------------------------------------------------------
// (c) 04-2001 Gottfried Chen
//------------------------------------------------------------------------------

#ifndef ENGINE_LIGHT_H
#define ENGINE_LIGHT_H

#include "Color.h"

#include <System/Base.h>
#include <System/Object.h>

namespace Ge
{

// Basic light class. Represents directional and point lights.
// If the w-coordinate of the position is 0 then the light lies in
// infinity and is thus a directional light. Otherwise it's a point
// light.
// ToDo: If lights can be bound to objects (like a car's headlights)
// then later add support to position them with locators.
class Light : public Object
{
public:
    // Default:
    // - color is white
    // - position is (0,0,0,1)
    // - range is 1
    Light();

    void setColor(const Color&);

    const Color& getColor() const;

    Float getRange() const;

    void setRange(const Float&);

    const Vector4& getPosition() const;

    void setPosition(const Vector4&);

private:
    Color mColor;
    Vector4 mPosition;
    Float mRange;
};

} // namespace Ge

#endif // ENGINE_LIGHT_H