#include "Precompiled.h"
//------------------------------------------------------------------------------
// (c) Jul-2002 Gottfried Chen
//------------------------------------------------------------------------------

#include "Lines.h"

#include "Renderer.h"
#include "RenderCache.h"

namespace Ge
{

//------------------------------------------------------------------------------
Lines::Lines(UInt vertexCount,
             Vector3* vertices,
             Color* colors,
             UInt indexCount,
             UInt16* indices) :
//------------------------------------------------------------------------------
mDynamic(false)
{
    GE_ASSERT_MSG(vertexCount > 0, "Empty vertex list");
    GE_ASSERT_MSG(vertices != 0, "Invalid vertex pointer");
    GE_ASSERT_MSG(indexCount > 0, "Empty index list");
    GE_ASSERT_MSG(indices != 0, "Invalid index pointer");
    GE_ASSERT_MSG(vertexCount < 65536, "No 32 bit indices supported");
        
    mVertexCount = vertexCount;
    mVertices = vertices;
    mColors = colors;
    mIndexCount = indexCount;
    mIndices = indices;

    calculateBound();
}

//------------------------------------------------------------------------------
Lines::~Lines()
//------------------------------------------------------------------------------
{
    GE_DELETE_A mVertices;
    GE_DELETE_A mColors;
    GE_DELETE_A mIndices;
}

////------------------------------------------------------------------------------
//void Lines::render(Renderer& renderer) const
////------------------------------------------------------------------------------
//{
//    renderer.render(*this);
//}

//------------------------------------------------------------------------------
void Lines::setDynamic(bool d)
//------------------------------------------------------------------------------
{
    mDynamic = d;
}

//------------------------------------------------------------------------------
bool Lines::isDynamic() const
//------------------------------------------------------------------------------
{
    return mDynamic;
}

//------------------------------------------------------------------------------
UInt Lines::getVertexCount() const
//------------------------------------------------------------------------------
{
    return mVertexCount;
}

//------------------------------------------------------------------------------
const Vector3* Lines::getVertices() const
//------------------------------------------------------------------------------
{
    return mVertices;
}
    

//------------------------------------------------------------------------------
const Color* Lines::getColors() const
//------------------------------------------------------------------------------
{
    return mColors;
}
    
//------------------------------------------------------------------------------
UInt Lines::getIndexCount() const
//------------------------------------------------------------------------------
{
    return mIndexCount;
}


//------------------------------------------------------------------------------
const UInt16* Lines::getIndices() const
//------------------------------------------------------------------------------
{
    return mIndices;
}

    
//------------------------------------------------------------------------------
Sphere Lines::getBound() const
//------------------------------------------------------------------------------
{
    return mLocalBound;
}

//------------------------------------------------------------------------------
void Lines::calculateBound()
//------------------------------------------------------------------------------
{
    mLocalBound.merge(mVertices[0], getVertexCount());
}


} // namespace Ge
