//------------------------------------------------------------------------------
// (c) Jul-2002 Gottfried Chen
//------------------------------------------------------------------------------

#ifndef ENGINE_LINES_H
#define ENGINE_LINES_H

#include <System/Base.h>
#include <System/Pointer.h>

#include <Math/Vector.h>
#include <Math/Sphere.h>

namespace Ge
{

class Color;
class RenderCache;

// Represents a bunch of lines. Ownership is taken of passed vertex/index data.
class Lines : public Object
{
public:    
    //------------------------------------------------------------------------------
    // Construction
    //------------------------------------------------------------------------------
    
    Lines(UInt vertexCount,
          Vector3* vertices,
          Color* colors,
          UInt indexCount,
          UInt16* indices);

    ~Lines();

    //------------------------------------------------------------------------------
    // Accessors
    //------------------------------------------------------------------------------
//    // Rendering
//    virtual
//    void render(Renderer& renderer) const;
    
    // Vertex data
    void setDynamic(bool);
    bool isDynamic() const;

    UInt getVertexCount() const;
    const Vector3* getVertices() const;
    
    const Color* getColors() const;
    
    // Indices
    UInt getIndexCount() const;
    const UInt16* getIndices() const;

    // This is a model space bounding volume.
    virtual
    Sphere getBound() const;

private:
    // Calculates the model space bounding volume
    void calculateBound();

    
    UInt mVertexCount;
    Vector3* mVertices;

    Color* mColors;

    UInt mIndexCount;
    UInt16* mIndices;
    
    Sphere mLocalBound;

    bool mDynamic;
};

} // namespace Ge

#endif // ENGINE_LINES_H