#include "Precompiled.h"
//------------------------------------------------------------------------------
// (c) Jul-2002 Gottfried Chen
//------------------------------------------------------------------------------

#include "Link.h"
#include "RenderCache.h"

namespace Ge
{

//------------------------------------------------------------------------------
Link::Link() :
//------------------------------------------------------------------------------
mShadow(0),
mLight(0,0,0,0)
{
}

//------------------------------------------------------------------------------
Link::Link(Locator& l, Mesh& m) :
//------------------------------------------------------------------------------
mLocator(&l),
mMesh(&m),
mShadow(0),
mLight(0,0,0,0)
{
}

//------------------------------------------------------------------------------
Link::~Link()
//------------------------------------------------------------------------------
{
    if (isShadowCaster())
    {
        GE_DELETE mShadow;
    }
}

//------------------------------------------------------------------------------
void Link::set(Locator& l)
//------------------------------------------------------------------------------
{
    mLocator = &l;
}

//------------------------------------------------------------------------------
Locator* Link::getLocator()
//------------------------------------------------------------------------------
{
    GE_ASSERT(mLocator.get() != 0);
    return mLocator;
}

//------------------------------------------------------------------------------
const Locator* Link::getLocator() const
//------------------------------------------------------------------------------
{
    GE_ASSERT(mLocator.get() != 0);
    return mLocator;
}

//------------------------------------------------------------------------------
void Link::set(Mesh& m)
//------------------------------------------------------------------------------
{
    mMesh = &m;
    setShadowCaster(isShadowCaster());
}

//------------------------------------------------------------------------------
Mesh* Link::getMesh() const
//------------------------------------------------------------------------------
{
    GE_ASSERT(mMesh.get() != 0);
    return mMesh;
}

//------------------------------------------------------------------------------
void Link::setShadowCaster(bool c)
//------------------------------------------------------------------------------
{
    if (c != isShadowCaster())
    {
        if (isShadowCaster())
        {
            GE_DELETE mShadow;
        }

        if (c)
        {
            GE_ASSERT(mMesh.get() != 0);
            if (0 == mMesh->getActiveFrame().getFacePlanes())
            {
                mMesh->calculateFacePlanes();
                mMesh->lerp(0,0,0);
            }
            
            if (0 == mMesh->getConnectivity())
            {
                mMesh->calculateConnectivity();
            }

            mShadow = GE_NEW ShadowVolume(*mMesh);
        }
    }
}

//------------------------------------------------------------------------------
bool Link::isShadowCaster() const
//------------------------------------------------------------------------------
{
    return mShadow != 0;
}

//------------------------------------------------------------------------------
const ShadowVolume* Link::getShadowVolume() const
//------------------------------------------------------------------------------
{
    GE_ASSERT(mShadow != 0);
    return mShadow;
}

//------------------------------------------------------------------------------
void Link::updateShadowVolume() const
//------------------------------------------------------------------------------
{
    const Vector4& light = getLight(0);
    GE_ASSERT(mMesh.get() != 0 && mLocator.get() != 0 && mShadow);
    if (mMesh.get() != 0 && mLocator.get() != 0 && mShadow)
    {
        Matrix3x4 inverse;
        inverse = mLocator->getGlobal();
        invertAffine(inverse);
        Vector3 temp = inverse*light;
        Vector4 objLight;
        ::Ge::set(objLight, temp, light[3]);
        mShadow->update(objLight);
    }
}

//------------------------------------------------------------------------------
Material* Link::getMaterial()
//------------------------------------------------------------------------------
{
    return mMaterial.get();
}

//------------------------------------------------------------------------------
const Material* Link::getMaterial() const
//------------------------------------------------------------------------------
{
    return mMaterial.get();
}

//------------------------------------------------------------------------------
void Link::set(Material& m)
//------------------------------------------------------------------------------
{
    mMaterial = &m;
}

//------------------------------------------------------------------------------
const StateBlock& Link::getStateBlock() const
//------------------------------------------------------------------------------
{
    return mStateBlock;
}

//------------------------------------------------------------------------------
void Link::set(const StateBlock& s)
//------------------------------------------------------------------------------
{
    mStateBlock = s;
}

//------------------------------------------------------------------------------
Link* Link::clone() const
//------------------------------------------------------------------------------
{
    Link* ret = GE_NEW Link;
    ret->set(*mMesh);
    if (getMaterial())
    {
        ret->set(*getMaterial()->clone());
    }
    ret->set(getStateBlock());
    ret->setShadowCaster(isShadowCaster());
    return ret;
}

//------------------------------------------------------------------------------
void Link::calculateBound()
//------------------------------------------------------------------------------
{
    GE_ASSERT(getMesh() != 0 && getLocator() != 0);
    mBound.setCenter(getLocator()->getGlobal()*
                     getMesh()->getActiveFrame().getBound().getCenter());
    
    Vector3 x;
    getLocator()->getGlobal().getColumn(MatrixBase::RIGHT, x);
    mBound.setRadius(getMesh()->getActiveFrame().getBound().getRadius()*x.length());
}

//------------------------------------------------------------------------------
const Sphere& Link::getBound() const
//------------------------------------------------------------------------------
{
    return mBound;
}


//------------------------------------------------------------------------------
void Link::setLight(UInt index, const Vector4& l)
//------------------------------------------------------------------------------
{
    mLight = l;
}

//------------------------------------------------------------------------------
const Vector4& Link::getLight(UInt index) const
//------------------------------------------------------------------------------
{
    return mLight;
}

} // namespace Ge
