#ifndef ENGINE_LINK_H
#define ENGINE_LINK_H
#pragma once

//------------------------------------------------------------------------------
// (c) Jul-2002 Gottfried Chen
//------------------------------------------------------------------------------

#include "Locator.h"
#include "Mesh.h"
#include "ShadowVolume.h"
#include "Material.h"
#include "StateBlock.h"

#include <System/Object.h>
#include <System/Pointer.h>
#include <System/NoCopy.h>

#include <Math/Sphere.h>

namespace Ge
{

// Links bind Meshes to Locator hierarchies.
class Link : public Object, NoCopy
{
public:
    Link();

    // The Locator and the Mesh must live on the heap.
    Link(Locator&, Mesh&);

    ~Link();
    
    void set(Locator&);
    
    Locator* getLocator();

    const Locator* getLocator() const;

    void set(Mesh&);

    Mesh* getMesh() const;

    void setShadowCaster(bool);

    bool isShadowCaster() const;

    const ShadowVolume* getShadowVolume() const;

    Material* getMaterial();

    const Material* getMaterial() const;

    void set(Material&);

    const StateBlock& getStateBlock() const;

    void set(const StateBlock&);

    Link* clone() const;

    // This function does NOT calculate a Mesh's MorphFrames' bound.
    // It only reallocates the bound using the Locator. So you
    // already must have calculated the Mesh's bound correctly.
    void calculateBound();

    const Sphere& getBound() const;

    // Sets the currently single allowed light position for the Link.
    // That only applies for bump mapping and Shadow volumes.
    void setLight(UInt index, const Vector4&);

    const Vector4& getLight(UInt index) const;

    // Recalculates the shadow volume according to the light position.
    void updateShadowVolume() const;

private:

    // Put <light> into tangent space and put it into Mesh's color array.
    void updateTangentLight(const Vector4& light) const;

    Pointer<Locator> mLocator;
    Pointer<Mesh> mMesh;
    Pointer<Material> mMaterial;
    StateBlock mStateBlock;
    ShadowVolume* mShadow;
    Sphere mBound;
    Vector4 mLight;
};

} // namespace Ge

#endif // !ENGINE_LINK_H