#include "Precompiled.h"
//------------------------------------------------------------------------------
// (c) 04-2001 Gottfried Chen
//------------------------------------------------------------------------------

#include <Engine/LocalTransform.h>
#include <Engine/WorldTransform.h>

namespace Ge
{

//------------------------------------------------------------------------------
LocalTransform::LocalTransform() :
//------------------------------------------------------------------------------
mScale(1,1,1),
mTranslation(Vector3::ORIGIN())
{
    mRotation = Matrix3x3::IDENTITY();
}

//------------------------------------------------------------------------------
void LocalTransform::setTranslation(const Vector3& translation)
//------------------------------------------------------------------------------
{
    mTranslation = translation;
}


//------------------------------------------------------------------------------
void LocalTransform::setRotation(const Vector3& axis, const Float& angle)
//------------------------------------------------------------------------------
{
    GE_ASSERT(!axis.isAlmost(Vector3::ORIGIN()));
    if (!axis.isAlmost(Vector3::ORIGIN()))
        makeRotation(mRotation, axis, angle);
}


//------------------------------------------------------------------------------
void LocalTransform::setRotation(const Matrix3x3& rotation)
//------------------------------------------------------------------------------
{
    mRotation = rotation;
}

//------------------------------------------------------------------------------
void LocalTransform::setScale(const Vector3& scale)
//------------------------------------------------------------------------------
{
    GE_ASSERT(scale.lengthSquare() > EPSILON);
    if (scale.lengthSquare() >EPSILON)
        mScale = scale;
}

//------------------------------------------------------------------------------
const Vector3& LocalTransform::getTranslation() const
//------------------------------------------------------------------------------
{
    return mTranslation;
}

//------------------------------------------------------------------------------
const Matrix3x3& LocalTransform::getRotation() const
//------------------------------------------------------------------------------
{
    return mRotation;
}

//------------------------------------------------------------------------------
const Vector3& LocalTransform::getScale() const
//------------------------------------------------------------------------------
{
    return mScale;
}

//------------------------------------------------------------------------------
void LocalTransform::asWorldTransform(WorldTransform& world) const
//------------------------------------------------------------------------------
{
    for (int i = 0; i<3; i++)
        for (int j = 0; j<3; j++)
            world.mMatrix(j,i) = getScale()[i]*(getRotation()(j,i));

    world.mTranslation = mTranslation;
    world.mScale = mScale.maxNorm();
}

} // namespace Ge