//------------------------------------------------------------------------------
// (c) 06-2001 Gottfried Chen
//------------------------------------------------------------------------------

#ifndef ENGINE_LWS_IMPORTER_H
#define ENGINE_LWS_IMPORTER_H

#include <System/Base.h>
#include <System/TagLine.h>
#include <System/Timer.h>

#include <Engine/PositionController.h>
#include <Engine/RotationController.h>
#include <Engine/ScaleController.h>
#include <Engine/HierarchyAnimation.h>

#include <fstream>
#include <vector>
#include <map>

namespace Ge
{

class ModelPool;
class Cluster;
//class HierarchyAnimationSet;

// Import Lightwave .lws files. Currently this is only used for importing
// hierarchical keyframe animation.
class LwsImporter
{
public:
    LwsImporter(const char* fileName, ModelPool&);
    Cluster* importModel() const;
    HierarchyAnimation/*Set**/ importAnimation() const;

    
private:
    bool waitForTag(std::istream& file);
    // Verify that .lws file version is correct
    bool verifyVersion(std::istream& file);
    // Read .obj file
    void readObject(const char* fileName);
    // Read animation channels for one object
    void readChannels(std::istream& file);
    // Read keys for one controller (pos, rot, scale)
    void readKeys(std::istream& file,
                  Vector3Controller& controller);
    // Read one animation key
    void readKey(Float& value, Timer::Unit& time,
                 Float& tangentIn, Float& tangentOut);
    // Read in key values are copied to the <controller>
    void copyBuffersToController(std::map<Timer::Unit, Vector3>& values,
                                 std::map<Timer::Unit, Vector3>& tIn,
                                 std::map<Timer::Unit, Vector3>& tOut,
                                 Vector3Controller& controller);
    // Read the parent node's index if present.
    void readParent(std::istream& file);
    // Creates a tree hierarchy from the mEntries list.
    void createHierarchyMap() const;
    // Make sure that all AudioVisuals that will be parents in the final
    // hierarchy are converted to Nodes
//    void assureParentsAreNodes() const;
    // Returns the number of root objects (i.e. with no parent) in mHierarchyMap.
    UInt getRootObjectCount() const;
    // After creating the hierarchy map create a subtree that contains all
    // root nodes in the hierarchy as children
    Cluster* buildSubTree() const;

    struct Entry
    {
        Entry() : mObject(0), mParent(-1) {}
        // Need Pointer<Cluster>, so that on destruction no memory is
        // leaked.
        Pointer<Cluster> mObject;
        PositionController mPositionController;
        RotationController mRotationController;
        ScaleController mScaleController;
        Int mParent;
    };

    UInt mCurrentEntry;
    TagLine mCurrentLine;
    ModelPool& mModelPool;
    std::vector<Entry> mEntries;

    // Create controllers, add them to mAnimationSet, return pointer
    // to animation set at importAnimationSet()
//    HierarchyAnimationSet* mAnimationSet;

    // This map is used for creating pointer relationships between parents
    // and children. The index is the position of the AudioVisual during the
    // loading process. (Actually I could use a std::vector or just build the
    // hierarchy in mEntries in place.)
    typedef std::map<UInt, Cluster*> HierarchyMap;
    mutable HierarchyMap mHierarchyMap;
};

} // namespace Ge

#endif // ENGINE_LWS_IMPORTER_H