#ifndef ENGINE_MATERIAL_H
#define ENGINE_MATERIAL_H
#pragma once

//------------------------------------------------------------------------------
// (c) Jul-2002 Gottfried Chen
//------------------------------------------------------------------------------

#include "Color.h"
#include "Texture.h"

#include <System/Object.h>
#include <System/Pointer.h>
#include <System/NoCopy.h>

#include <Math/Matrix.h>

namespace Ge
{

class Link;    

class Material : public Object, NoCopy
{
public:
    enum
    {
        MAX_TEXTURES = 4
    };

    // Default is white without textures.
    Material(UInt passCount = 1);

    Color getColor() const;

    void set(const Color&);

    const Texture* getTexture(UInt index) const;

    // The texture must live on the heap.
    void set(UInt index, const Texture&);

    // Apply render state changes and blend modes and so on for <pass>
    virtual
    void apply(UInt pass, const Matrix4x4& view) const;

    UInt getPassCount() const;

    virtual
    Material* clone() const;

    void setLink(Link&);

    void setPreShadow(bool);

    void setPostShadow(bool);

protected:
    // Used by clone(). Does the actual data copying.
    virtual
    void copyDataFrom(const Material&);

    void setPassCount(UInt);

    const Link* getLink() const;

    bool isPreShadow() const;

    bool isPostShadow() const;

private:
    Material(const Material&);
    Material& operator =(const Material&);

    Color mColor;
    Pointer<const Texture> mTextures[MAX_TEXTURES];
    UInt mPassCount;

    Link* mLink;
    bool mPreShadow;
    bool mPostShadow;
};

} // namespace Ge

#endif // !ENGINE_MATERIAL_H