#ifndef ENGINE_MD3_MODEL_H
#define ENGINE_MD3_MODEL_H
#pragma once

//------------------------------------------------------------------------------
// (c) Aug-2002 Gottfried Chen
//------------------------------------------------------------------------------

#include "MorphAnimation.h"

#include <map>

namespace Ge
{

class Link;
class Cluster;
struct Md3Tag;

struct Md3Animation
{
    UInt mFrameStart;
    UInt mFrameCount;
    UInt mFrameLoop;
    UInt mFps;
};

class Md3Model
{
public:
    // One body part of an Md3Model
    struct Part
    {
        Part();
        ~Part();
        
        typedef std::map<std::string, Md3Animation> Animations;
        Animations mAnimations;
        std::string mCurrentAnimation;
        std::string mIdle;
        UInt mCurrentFrame;
        Float mCurrentDelta;
        std::map<std::string, Locator*> mJoints;
        std::vector<Link*> mLinks;
        Md3Tag* mTags;
        UInt mTagCount;
    };

    // All parameters must live on the heap.
    Md3Model(Cluster&, Part& lower, Part& upper, Part& head);

    ~Md3Model();

    const Cluster& getCluster() const;

    Cluster& getCluster();

    void update();

    void setLowerAnimation(const char*);

    void setUpperAnimation(const char*);

private:
    void setAnimation(Part&, const char* anim);

    void update(Part&);

    void updateMorphs(const Part&, UInt frame, UInt nextFrame, Float delta);

    void updateJoints(Part&, UInt frame, UInt nextFrame, Float delta);

    Cluster* mCluster;
    Part* mLower;
    Part* mUpper;
    Part* mHead;
};

} // namespace Ge

#endif // !ENGINE_MD3_MODEL_H