#ifndef ENGINE_MD3TYPES_H
#define ENGINE_MD3TYPES_H
#pragma once

//------------------------------------------------------------------------------
// (c) Aug-2002 Gottfried Chen
//------------------------------------------------------------------------------

namespace Ge
{

// File header
struct Md3Header
{ 
    UInt32  mFileId;                     // This stores the file ID - Must be "IDP3"
    Int     mVersion;                    // This stores the file version - Must be 15
    UInt8   mFile[68];                   // This stores the name of the file
    Int     mFrameCount;                 // This stores the number of animation frames
    Int     mTagCount;                   // This stores the tag count
    Int     mMeshCount;                  // This stores the number of sub-objects in the mesh
    Int     mSkinCount;                  // This stores the number of skins for the mesh
    Int     mHeaderSize;                 // This stores the mesh header size
    Int     mTagStart;                   // This stores the offset Into the file for tags
    Int     mTagEnd;                     // This stores the end offset Into the file for tags
    Int     mFileSize;                   // This stores the file size
};

// Info for each mesh in an .md3
struct Md3MeshInfo
{
    UInt8   mMeshId[4];                 // This stores the mesh ID (We don't care)
    UInt8   mName[68];                  // This stores the mesh name (We do care)
    Int     mFrameCount;                // This stores the mesh animation frame count
    Int     mSkinCount;                 // This stores the mesh skin count
    Int     mVertexCount;               // This stores the mesh vertex count
    Int     mTriangleCount;             // This stores the mesh face count
    Int     mTriStart;                  // This stores the starting offset for the triangles
    Int     mHeaderSize;                // This stores the header size for the mesh
    Int     mUvStart;                   // This stores the starting offset for the UV coordinates
    Int     mVertexStart;               // This stores the starting offset for the vertex indices
    Int     mMeshSize;                  // This stores the total mesh size
};

// Bone data
struct Md3Bone
{
    Float   mMins[3];                    // This is the min (x, y, z) value for the bone
    Float   mMaxs[3];                    // This is the max (x, y, z) value for the bone
    Float   mPosition[3];                // This supposedly stores the bone position???
    Float   mScale;                      // This stores the scale of the bone
    UInt8   mCreator[16];                // The modeler used to create the model (I.E. "3DS Max")
};

// A tag is a transformation matrix that stores keyframe animation data for
// hierarchical animations connecting the model's body parts.
struct Md3Tag
{
    UInt8       mName[64];              // This stores the name of the tag (I.E. "tag_torso")
    Float       mTranslation[3];        // This stores the translation that should be performed
    Float       mRotation[3][3];        // This stores the 3x3 rotation matrix for this frame
};

struct Md3Vertex
{
    Int16       mVertex[3];             // The vertex for this face (scale down by 64.0f)
    UInt8       mNormal[2];             // This stores some crazy normal values (not sure...)
};

struct Md3Skin 
{
    UInt8 mName[68]; // The name of the corresponding texture file
};

struct Md3Triangle
{
    Int mIndices[3];
};

} // namespace Ge

#endif // !ENGINE_MD3TYPES_H