//------------------------------------------------------------------------------
// (c) 04-2001 Gottfried Chen
//------------------------------------------------------------------------------

#include <Math/Tools.h>
#include <Math/Vector.h>


namespace Ge
{

//inline
////------------------------------------------------------------------------------
//const Vector3* Mesh::getVertices() const
////------------------------------------------------------------------------------
//{
//    return mVertices;
//}


inline
//------------------------------------------------------------------------------
UInt Mesh::getVertexCount() const
//------------------------------------------------------------------------------
{
    return mVertexCount;
}

inline
//------------------------------------------------------------------------------
const Vector2* Mesh::getUvs(UInt set) const
//------------------------------------------------------------------------------
{
    GE_ASSERT(set < UV_SET_COUNT);
    GE_ASSERT(mTextureUvs[set].first == UV_ARRAY);
    GE_ASSERT(mTextureUvs[set].second != 0);
    return mTextureUvs[set].second;
}

inline
//------------------------------------------------------------------------------
Mesh::UvType Mesh::getUvType(UInt set) const
//------------------------------------------------------------------------------
{
    GE_ASSERT(set < UV_SET_COUNT);
    return mTextureUvs[set].first;
}

inline
//------------------------------------------------------------------------------
UInt Mesh::getIndexCount() const
//------------------------------------------------------------------------------
{
    return mIndexCount;
}

inline
//------------------------------------------------------------------------------
Pointer<RenderCache>& Mesh::cache() const
//------------------------------------------------------------------------------
{
    return mCache;
}


inline
//------------------------------------------------------------------------------
UInt Mesh::getPrimitiveCount() const
//------------------------------------------------------------------------------
{
    UInt vertices(getIndexCount() == 0 ? getVertexCount() : getIndexCount());
    return vertices/3;
}

inline
//------------------------------------------------------------------------------
void Mesh::getVerticesStartingFromBase(unsigned int base,
        unsigned int& v0, unsigned int&v1, unsigned int& v2)
//------------------------------------------------------------------------------
{
    // Vertex addressing depends on existence of indices
    if (mIndexCount > 0)
    {
        v0 = mIndices[base];
        v1 = mIndices[base + 1];
        v2 = mIndices[base + 2];
    }
    else
    {
        v0 = base;
        v1 = base + 1;
        v2 = base + 2;
    }
}

inline
//------------------------------------------------------------------------------
void Mesh::getTriangleVertices(UInt triangle, UInt& v0, UInt& v1, UInt& v2)
//------------------------------------------------------------------------------
{
    GE_ASSERT(triangle < getPrimitiveCount());
    unsigned int base = 3*triangle;    
    getVerticesStartingFromBase(base, v0, v1, v2);
}

} // namespace Ge