#include "Precompiled.h"
//------------------------------------------------------------------------------
// (c) 06-2001 Gottfried Chen
//------------------------------------------------------------------------------

#pragma warning (disable:4786)

#include "ModelPool.h"
#include "ObjImporter.h"
//#include "Cluster.h"

#include <DirectXRenderer/XImporter.h>

#include <System/FilePath.h>


namespace Ge
{

//------------------------------------------------------------------------------
Pointer<ModelPool>& ModelPool::instancePtr()
//------------------------------------------------------------------------------
{
    static Pointer<ModelPool> mp;
    return mp;
}

//------------------------------------------------------------------------------
ModelPool::ModelPool(TexturePool& tp, const DirectXRenderer* dxr ) :
//------------------------------------------------------------------------------
mTexturePool(tp),
mDirectXRenderer(dxr),
mImportMaterialAndTexture(false),
mRecalculateNormals(false),
mFlipUvs(false)
{
}

//------------------------------------------------------------------------------
Cluster* ModelPool::get(const char* file)
//------------------------------------------------------------------------------
{
    Cluster* model = mModels[file].mModel;
    if (model == 0)
    {
        FilePath path(file);
        if (path.getExtension() == ".x")
        {
            GE_ASSERT(mDirectXRenderer != 0);
            if (mDirectXRenderer != 0)
            {
                XImporter x(file, *mDirectXRenderer);
                model = x.import(mImportMaterialAndTexture,
                                 mRecalculateNormals,
                                 mFlipUvs);
            }
        }
        else if (path.getExtension() == ".obj")
        {
            ObjImporter obj(file, mTexturePool);
            model = obj.import(mImportMaterialAndTexture,
                               mRecalculateNormals,
                               mFlipUvs);
        }
        else if (path.getExtension() == ".lwo")
        {
            GE_ASSERT_MSG(false, ".lwo import not supported yet");
        }
        else
            GE_ASSERT_MSG(false, "Unknown model file extension");
        
        GE_ASSERT(model != 0);
        mModels[file].mModel = model;
    }

    Cluster* ret = model->clone();
    
//    nameModel(*ret, file);

    return ret;
}

////------------------------------------------------------------------------------
//void ModelPool::nameModel(AudioVisual& model, const char* file)
////------------------------------------------------------------------------------
//{
//    // Cloned models' names will be numbered in a sequence
//    std::string name(mModels[file].mModel->getName());
//    if (mModels[file].mCounter > 0)
//        name += StringTools::toString(mModels[file].mCounter);
//
//    model.setName(name.c_str());
//    mModels[file].mCounter++;
//}

//------------------------------------------------------------------------------
void ModelPool::setImportMaterialAndTexture(bool import)
//------------------------------------------------------------------------------
{
    mImportMaterialAndTexture = import;
}

//------------------------------------------------------------------------------
void ModelPool::setRecalculateNormals(bool recalculate)
//------------------------------------------------------------------------------
{
    mRecalculateNormals = recalculate;
}

//------------------------------------------------------------------------------
void ModelPool::setFlipUvs(bool flip)
//------------------------------------------------------------------------------
{
    mFlipUvs = flip;
}

} // namespace Ge
