//------------------------------------------------------------------------------
// (c) 06-2001 Gottfried Chen
//------------------------------------------------------------------------------

#ifndef ENGINE_MODEL_POOL_H
#define ENGINE_MODEL_POOL_H

#include <System/Pointer.h>
#include <System/StringTools.h>

//#include <Engine/AudioVisual.h>
#include "Cluster.h"

#include <map>
#include <string>

namespace Ge
{

class DirectXRenderer;
class TexturePool;

class ModelPool;
class Cluster;

ModelPool& theModelPool();

// This class manages loading of models. Every loaded model is registered in
// the ModelPool. Thus no model will be loaded twice. .x files can only be loaded
// in conjunction with a DirectXRenderer.
// The difference in terminology between a mesh and a model in the GameEngine
// is, that a mesh can have only one material/texture set, so that sorting by
// materials is possible. A model is made up of a subtree of meshes. During import
// models are split up into meshes by the importers.
class ModelPool : public Object
{
public:
    ModelPool(TexturePool&, const DirectXRenderer* = 0);

    // Use this to set the one and only ModelPool from the application
    // framework.
    static Pointer<ModelPool>& instancePtr();

    Cluster* get(const char* file);

    // All these properties are set to false by default

    // Import both material and texture (normally only texture is imported).
    void setImportMaterialAndTexture(bool import);

    // Recalculate mesh normals.
    void setRecalculateNormals(bool recalculate);

    // Flip the the uv coordinates on import.
    void setFlipUvs(bool flip);

    bool importMaterialAndTexture() const;

    bool recalculateNormals() const;

    bool flipUvs() const;

private:
    struct Entry
    {
        Entry() : mCounter(0) {}
        UInt mCounter;
        Pointer<Cluster> mModel;
    };

//    void nameModel(AudioVisual& model, const char* file);

    typedef std::map<std::string, Entry, StringTools::Pred> Map;
    Map mModels;
    TexturePool& mTexturePool;
    const DirectXRenderer* mDirectXRenderer;

    bool mImportMaterialAndTexture;
    bool mRecalculateNormals;
    bool mFlipUvs;
};

inline
//------------------------------------------------------------------------------
ModelPool& theModelPool()
//------------------------------------------------------------------------------
{
    return *(ModelPool::instancePtr());
}

} // namespace Ge

#endif // ENGINE_MODEL_POOL_H