#ifndef MODEL_VIEW_CAMERA_CONTROLLER_H
#define MODEL_VIEW_CAMERA_CONTROLLER_H
#pragma once

//------------------------------------------------------------------------------
// (c) Mai-2002 Gottfried Chen
//------------------------------------------------------------------------------

#include <System/Types.h>

#include <Math/Vector.h>


namespace Ge
{

class Camera;

// The center of a ModelViewCameraController is defined as
// position + distance*at where <position> is the
// camera position and <at> is the camera transformation's
// at vector. So the camera rotates around the center (where the
// viewed model is supposed to be).
//
// With left mouse button down you rotate the camera.
// With right mouse button down you rotate the camera.
// With mouse wheel or middle mouse moving left/right you zoom the camera.
class ModelViewCameraController
{
public:
    // The controller only stores a reference to the camera.
    // Just make sure the camera lives at least as long as the
    // controller.
    // The default distance of the camera is 10.
    ModelViewCameraController(Camera&);

    Float getDistance() const;

    void setDistance(Float);

    bool isInverted() const;

    void setInvert(bool invert);

    // After changing the Locator of the camera call this function to
    // recalculate the camera parameters (currently only the
    // center is repositioned).
    void recalculate();

    void update();

    // Default sensitivity is 1.
    void setSensitivity(Float);

    Float getSensitivity() const;

private:
    
    Float mSensitivity;
    Float mDistance;
    Vector3 mCenter;
    Int mInvert;
    Camera& mCamera;
};

} // namespace Ge

#endif // !MODEL_VIEW_CAMERA_CONTROLLER_H