#ifndef ENGINE_MORPH_FRAME_H
#define ENGINE_MORPH_FRAME_H
#pragma once

//------------------------------------------------------------------------------
// (c) Aug-2002 Gottfried Chen
//------------------------------------------------------------------------------

#include <Math/Vector.h>
#include <Math/Matrix.h>
#include <Math/Sphere.h>

namespace Ge
{

// One frame in a Mesh's morphing animation. That's actually a helper class
// to Mesh so you mustn't instance one yourself.
class MorphFrame
{
public:
    MorphFrame();

    ~MorphFrame();
    
    void setVertices(Vector3*);

    const Vector3* getVertices() const;

    void setNormals(Vector3*);
    
    const Vector3* getNormals() const;

    // This is an object space bounding volume.
    void calculateBound(UInt vertexCount);

    const Sphere& getBound() const;

    // One face plane is given as a Vector4(a,b,c,d) per triangle with a point being
    // on the plane if ax + by +cz +dw = 0. (a,b,c) needs not to be normalized,
    // that's why not the Plane class is used.
    void calculateFacePlanes(UInt triangleCount, UInt16* indices);
    
    const Vector4* getFacePlanes() const;

    void setTangents(Matrix3x3*);

    const Matrix3x3* getTangents() const;

    void flipTangents(UInt vertexCount);

    void lerp(UInt vertexCount, UInt triangleCount,
              const MorphFrame& from, const MorphFrame& to, Float s);

private:
    friend class Mesh;

    void copyFrom(UInt vertexCount, UInt triangleCount,
                  const MorphFrame& source,
                  bool normals, bool facePlanes, bool tangents);

    Vector3* mVertices;
    Vector3* mNormals;
    Vector4* mFacePlanes;
    // Transforms to move light vector from object to tangent space
    Matrix3x3* mTangents; 
    Sphere mBound;
};

} // namespace Ge

#endif // !ENGINE_MORPH_FRAME_H