#include "Precompiled.h"
//------------------------------------------------------------------------------
// (c) 06-2001 Gottfried Chen
//------------------------------------------------------------------------------

#pragma warning(disable: 4786)

#include "MtlImporter.h"

#include <Engine/TexturePool.h>

#include <System/FilePath.h>

namespace Ge
{

//------------------------------------------------------------------------------
MtlImporter::MtlImporter(TexturePool& pool) :
//------------------------------------------------------------------------------
mTexturePool(pool)
{
}

//------------------------------------------------------------------------------
void MtlImporter::import(const char* fileName)
//------------------------------------------------------------------------------
{
    std::ifstream file(fileName, std::ios::in);
    GE_ASSERT_MSG(file != 0,
        (std::string(fileName)+" couldn't be opened").c_str());

    if (file)
    {
        while (waitForTag(file))
        {
            if ("newmtl" == mCurrentLine.getTag())
                addNewEntry();
//            else if ("Ns" == mCurrentLine.getTag())
//                setSpecularSharpness();
            else if ("Kd" == mCurrentLine.getTag())
                setDiffuse();
//            else if ("Ks" == mCurrentLine.getTag())
//                setSpecular();
//            else if ("Ka" == mCurrentLine.getTag())
//                setAmbient();
            else if ("map_Kd" == mCurrentLine.getTag())
            {
                FilePath path(fileName);
                setTexture(path.getDirectory().c_str());
            }
            else
                GE_ASSERT_MSG(false, "Unknown tag in .mtl file");

            mCurrentLine.clear();
        }
    }
}

//------------------------------------------------------------------------------
void MtlImporter::readColor(Vector4& rgba)
//------------------------------------------------------------------------------
{
    rgba[0] = StringTools::toFloat(mCurrentLine.getTag());
    mCurrentLine.split();
    rgba[1] = StringTools::toFloat(mCurrentLine.getTag());
    mCurrentLine.split();
    rgba[2] = StringTools::toFloat(mCurrentLine.getTag());
}

//------------------------------------------------------------------------------
void MtlImporter::setDiffuse()
//------------------------------------------------------------------------------
{
    mCurrentLine.split();
    Vector4 rgba;
    readColor(rgba);
    mCurrentEntry->mMaterial->set(Color(rgba[0], rgba[1], rgba[2], 1));
}

//------------------------------------------------------------------------------
void MtlImporter::setTexture(const char* dir)
//------------------------------------------------------------------------------
{
    FilePath fileName(mCurrentLine.getRest().c_str());
    fileName.setDirectory(dir);
    mCurrentEntry->mMaterial->set(0, *mTexturePool.get(fileName.get().c_str()));
}

//------------------------------------------------------------------------------
void MtlImporter::addNewEntry()
//------------------------------------------------------------------------------
{
    mCurrentEntry = &(mEntries[mCurrentLine.getRest()]);
    mCurrentEntry->mMaterial = GE_NEW Material;
}

//------------------------------------------------------------------------------
bool MtlImporter::waitForTag(std::istream& file)
//------------------------------------------------------------------------------
{
    while (!file.eof() &&
           "newmtl" != mCurrentLine.getTag() &&
//           "Ns" != mCurrentLine.getTag() &&
           "Kd" != mCurrentLine.getTag() &&
//           "Ks" != mCurrentLine.getTag() &&
//           "Ka" != mCurrentLine.getTag() &&
           "map_Kd" != mCurrentLine.getTag())
           mCurrentLine.read(file);

    // !file.eof() means a tag was found
    return !file.eof();
}

//------------------------------------------------------------------------------
Material* MtlImporter::getMaterial(const char* name)
//------------------------------------------------------------------------------
{
    if (mEntries.find(std::string(name)) != mEntries.end())
    {
        return mEntries[std::string(name)].mMaterial;
    }
    else
    {
        return 0;
    }
}

} // namespace Ge
