//------------------------------------------------------------------------------
// (c) 06-2001 Gottfried Chen
//------------------------------------------------------------------------------

#ifndef ENGINE_MTL_IMPORTER_H
#define ENGINE_MTL_IMPORTER_H

#include "Material.h"

#include <Math/Vector.h>

#include <System/Pointer.h>
#include <System/TagLine.h>
#include <System/StringTools.h>

#include <string>
#include <fstream>
#include <map>

namespace Ge
{

class TexturePool;

// Imports .mtl material libs that accompany .obj files
class MtlImporter
{
public:
    MtlImporter(TexturePool&);

    void import(const char* fileName);
    Material* getMaterial(const char* name);

private:
    bool waitForTag(std::istream&);
    // Adds a new entry into mCurrentEntry using the material name as index
    void addNewEntry();
    void createMaterial();
//    void setSpecularSharpness();
    void setDiffuse();
//    void setSpecular();
//    void setAmbient();
    void setTexture(const char* path);
    void readColor(Vector4&);

    struct Entry
    {
        Entry() : mMaterial(0) {}
        // Some .obj files don't use all materials in the lib. Those that are not
        // imported by getMaterial() would otherwise cause memory leaks -> use Pointer<>
        Pointer<Material> mMaterial;
    };

    typedef std::map<std::string, Entry, StringTools::Pred> Map;
    Map mEntries;
    Entry* mCurrentEntry;
    TexturePool& mTexturePool;
    TagLine mCurrentLine;
};

} // namespace Ge

#endif // ENGINE_MTL_IMPORTER_H