#include "Precompiled.h"
//------------------------------------------------------------------------------
// (c) Aug-2002 Gottfried Chen
//------------------------------------------------------------------------------

#include "Particle.h"
#include "Emitter.h"

#include <Math/Interpolation.h>
#include <Math/Random.h>

#include <Application/ScaledMetronome.h>

namespace Ge
{

//------------------------------------------------------------------------------
Particle::Particle() :
//------------------------------------------------------------------------------
mAge(-1),
mEmitter(0)
{
}

//------------------------------------------------------------------------------
void Particle::init(const Vector3& position, const Vector3& speed,
                    const Emitter* e)
//------------------------------------------------------------------------------
{
    GE_ASSERT(e != 0);
    if (e != 0)
    {
        mPosition = position;
        mSpeed = speed;
        mAge = 0;
        mEmitter = e;
    
        if (e->getAngle().first != e->getAngle().second)
        {
            mAngle = mStartAngle = frand(0, TWO_PI);
        }
        else
        {
            mAngle = mStartAngle = 0;
        }

        mSize = e->getSize().first;
        mColor = e->getColor().first;
    }
}

//------------------------------------------------------------------------------
bool Particle::update()
//------------------------------------------------------------------------------
{
    if (isAlive() && mEmitter != 0)
    {
        Float delta = theScaledMetronome().getDelta()/1000;
        mPosition += mSpeed*delta;
        mAge += delta;
        if (mAge > mEmitter->getLifetime())
        {
            mAge = -1;
        }
        else
        {
            Float factor = mAge/mEmitter->getLifetime();
            mSize = lerp(mEmitter->getSize().first, mEmitter->getSize().second,
                         factor); 

            mAngle = mStartAngle +
                     lerp(mEmitter->getAngle().first, mEmitter->getAngle().second,
                          factor); 

            mColor = lerp(mEmitter->getColor().first, mEmitter->getColor().second,
                          factor);
        }
    }

    return isAlive();
}

//------------------------------------------------------------------------------
bool Particle::isAlive() const
//------------------------------------------------------------------------------
{
    return mAge >= 0;
}

//------------------------------------------------------------------------------
Float Particle::getSize() const
//------------------------------------------------------------------------------
{
    return mSize;
}

//------------------------------------------------------------------------------
Float Particle::getAngle() const
//------------------------------------------------------------------------------
{
    return mAngle;
}

//------------------------------------------------------------------------------
const Vector3& Particle::getPosition() const
//------------------------------------------------------------------------------
{
    return mPosition;
}

//------------------------------------------------------------------------------
const Color& Particle::getColor() const
//------------------------------------------------------------------------------
{
    return mColor;
}

} // namespace Ge
