#include "Precompiled.h"
//------------------------------------------------------------------------------
// (c) Jun-2002 Gottfried Chen
//------------------------------------------------------------------------------

#include "Pick.h"

#include "Camera.h"

#include <System/Trace.h>
#include <Application/Metronome.h>

#include <Math/Vector.h>
#include <Math/Interpolation.h>

#include <Input/Mouse.h>

#include <windows.h>

namespace Ge
{

//------------------------------------------------------------------------------
Pick::Pick()
//------------------------------------------------------------------------------
{
}

//------------------------------------------------------------------------------
Pick::Pick(const Camera& c, Handle window) :
//------------------------------------------------------------------------------
mCamera(&c),
mWindow(window)
{
    GE_ASSERT(window != 0);
}

//------------------------------------------------------------------------------
const Ray& Pick::getMouseRay() const
//------------------------------------------------------------------------------
{
    return mRay;
}

//------------------------------------------------------------------------------
void Pick::setCamera(const Camera& c)
//------------------------------------------------------------------------------
{
    mCamera = &c;
}

//------------------------------------------------------------------------------
void Pick::setWindow(Handle window)
//------------------------------------------------------------------------------
{
    GE_ASSERT(window != 0);
    mWindow = window;
}

//------------------------------------------------------------------------------
void Pick::update()
//------------------------------------------------------------------------------
{   
    // Get mouse position in window client coordinates
    Vector2 screenPos;
    theMouse().getAbsolute(screenPos);

    POINT clientPos;
    clientPos.x = screenPos[0];
    clientPos.y = screenPos[1];
    bool ok = ScreenToClient(HWND(mWindow), &clientPos) != 0;
    GE_ASSERT_MSG(ok, "ScreenToClient failed");

    // Convert mouse position to camera space coordinates
    RECT clientSize;
    ok = GetClientRect(HWND(mWindow), &clientSize) != 0;
    GE_ASSERT_MSG(ok, "GetClientRect failed");

    Vector2 viewport;
    viewport[0] = mCamera->getViewportWidth()*0.5f;
    viewport[1] = (viewport[0] / mCamera->getAspectRatio());

    Vector3 camSpacePos;
    camSpacePos[0] = lerp(0, clientSize.right, -viewport[0], viewport[0], clientPos.x);
    camSpacePos[1] = lerp(0, clientSize.bottom, viewport[1], -viewport[1], clientPos.y);
    camSpacePos[2] = mCamera->getNearPlane();
                      
    // Convert camera space to world space
    Matrix3x3 rotation;
    mCamera->getLocator().get().getSubMatrix(rotation);
    camSpacePos = rotation*camSpacePos;

    Vector3 camPos;
    mCamera->getLocator().get().getColumn(MatrixBase::TRANSLATION, camPos);
    
    // Put result into ray.
    mRay.setStart(camPos);
    mRay.setDirection(camSpacePos);

    //GE_INTERVAL_TRACE("Client pos: "<<clientPos.x<<", "<<clientPos.y<<std::endl, 1000);
}

} // namespace Ge
