#ifndef ENGINE_RENDER_STATE_H
#define ENGINE_RENDER_STATE_H
#pragma once

//------------------------------------------------------------------------------
// (c) Jul-2002 Gottfried Chen
//------------------------------------------------------------------------------

namespace Ge
{

namespace RenderState
{

    enum Type
    {
        BLEND,
        FILL,
        SHADE,
        SPECULAR,
        DEPTH,
        DEPTH_COMPARE,
        STENCIL,
        COLOR_WRITE,
        CULLING,
        LIGHTING,
        TYPE_COUNT
    };

    // This is used in delta StateBlocks to identify if no change has
    // occured in a RenderState.
    enum
    {
        NO_CHANGE = -1
    };

    enum Blend
    {
        BLEND_OFF,
        BLEND_MULTIPLY,
        BLEND_ADD,
        BLEND_ADD_SQUARED,
        BLEND_SOURCE_ALPHA,
        BLEND_SOURCE_ALPHA_ADD, // Add with source alpha to destination.
        BLEND_DEFAULT = BLEND_OFF
    };

    enum Fill
    {
        FILL_SOLID,
        FILL_WIRE,
        FILL_POINT,
        FILL_DEFAULT = FILL_SOLID
    };

    enum Shade
    {
        SHADE_FLAT,
        SHADE_GOURAUD,
        SHADE_DEFAULT = SHADE_GOURAUD
    };

    enum Specular
    {
        SPECULAR_ON,
        SPECULAR_OFF,
        SPECULAR_DEFAULT = SPECULAR_OFF
    };

    enum Depth
    {
        DEPTH_ON,
        DEPTH_OFF,
        DEPTH_READ_ONLY,
        DEPTH_DEFAULT = DEPTH_ON
    };

    enum DepthCompare
    {
        DEPTH_COMPARE_EQUAL,
        DEPTH_COMPARE_LESS_OR_EQUAL,
        DEPTH_COMPARE_LESS,
        DEPTH_COMPARE_DEFAULT = DEPTH_COMPARE_LESS_OR_EQUAL
    };

    enum Stencil
    {
        STENCIL_ON,
        STENCIL_OFF,
        STENCIL_DEFAULT = STENCIL_OFF
    };

    enum ColorWrite
    {
        COLOR_WRITE_ON,
        COLOR_WRITE_OFF,
        COLOR_WRITE_DEFAULT = COLOR_WRITE_ON
    };

    enum Culling
    {
        CULLING_CLOCKWISE,
        CULLING_COUNTERCLOCKWISE,
        CULLING_OFF,
        CULLING_DEFAULT = CULLING_COUNTERCLOCKWISE
    };

    enum Lighting
    {
        LIGHTING_ON,
        LIGHTING_OFF,
        LIGHTING_DEFAULT = LIGHTING_ON
    };

} // namespace RenderState

} // namespace Ge

#endif // !ENGINE_RENDER_STATE_H