#include "Precompiled.h"
//------------------------------------------------------------------------------
// (c) Jul-2002 Gottfried Chen
//------------------------------------------------------------------------------

#include "ShadowVolume.h"

#include "Mesh.h"

#include <System/Trace.h>

namespace
{

    using namespace Ge;

    //------------------------------------------------------------------------------
    Vector4 convert(const Vector3& v)
    //------------------------------------------------------------------------------
    {
        return Vector4(v[0], v[1], v[2], 1);
    }

    Float INFINITY = 10000;

} // namespace

namespace Ge
{

//------------------------------------------------------------------------------
ShadowVolume::ShadowVolume()
//------------------------------------------------------------------------------
{
}

//------------------------------------------------------------------------------
ShadowVolume::ShadowVolume(const Mesh& m) :
//------------------------------------------------------------------------------
mMesh(&m)
{
}

//------------------------------------------------------------------------------
void ShadowVolume::set(const Mesh& m)
//------------------------------------------------------------------------------
{
    mMesh = &m;
}

//------------------------------------------------------------------------------
void ShadowVolume::update(const Vector4& lightPos)
//------------------------------------------------------------------------------
{
    mVertices.clear();
    mVertices.reserve(mMesh->getIndexCount());

    bool* front = GE_NEW bool[mMesh->getPrimitiveCount()];
    const MorphFrame& frame = mMesh->getActiveFrame();
    UInt t = 0;
    for (t = 0; t<mMesh->getPrimitiveCount(); ++t)
    {
        front[t] = ( lightPos*(frame.getFacePlanes()[t]) ) > 0;
    }

    const UInt16* triangle = 0;
    for (t = 0; t<mMesh->getPrimitiveCount(); ++t)
    {
        triangle = &(mMesh->getIndices()[t*3]);
        if (front[t])
        {
            // Triangle is front facing
            mVertices.push_back(convert(frame.getVertices()[triangle[0]]));
            mVertices.push_back(convert(frame.getVertices()[triangle[1]]));
            mVertices.push_back(convert(frame.getVertices()[triangle[2]]));

            // Check the triangle's neighbours
            for (UInt n = 0; n<3; ++n)
            {
                if (!front[ mMesh->getConnectivity()[t][n] ])
                {
                    // Neighbour is back facing

                    // Corresponding edge in front facing triangle's
                    // winding order.
                    Vector3 a = frame.getVertices()[triangle[n]];
                    Vector3 b = frame.getVertices()[triangle[(n+1)%3]];
                    mVertices.push_back(Vector4(b[0], b[1], b[2], 1));
                    mVertices.push_back(Vector4(a[0], a[1], a[2], 1));
                    Vector4 al(
                        a[0]*lightPos[3] - lightPos[0],
                        a[1]*lightPos[3] - lightPos[1],
                        a[2]*lightPos[3] - lightPos[2],
                        0);
                    al *= INFINITY;
                    mVertices.push_back(al);

                    mVertices.push_back(Vector4(b[0], b[1], b[2], 1));
                    mVertices.push_back(al);
                    mVertices.push_back(Vector4(
                        b[0]*lightPos[3] - lightPos[0],
                        b[1]*lightPos[3] - lightPos[1],
                        b[2]*lightPos[3] - lightPos[2],
                        0)*INFINITY);
                }
            }
        }
        else
        {
            // Triangle is back facing
            Vector3 v;
            for (UInt i = 0; i<3; ++i)
            {
                v = frame.getVertices()[triangle[i]];
                mVertices.push_back(Vector4(
                    v[0]*lightPos[3] - lightPos[0],
                    v[1]*lightPos[3] - lightPos[1],
                    v[2]*lightPos[3] - lightPos[2],
                    0)*INFINITY);
            }
        }
    }

    GE_DELETE_A front;
}


//------------------------------------------------------------------------------
const Vector4* ShadowVolume::getVertices() const
//------------------------------------------------------------------------------
{
    return &(mVertices[0]);
}

//------------------------------------------------------------------------------
UInt ShadowVolume::getVertexCount() const
//------------------------------------------------------------------------------
{
    return mVertices.size();
}

} // namespace Ge
