#ifndef ENGINE_SHADOW_VOLUME_H
#define ENGINE_SHADOW_VOLUME_H
#pragma once

//------------------------------------------------------------------------------
// (c) Jul-2002 Gottfried Chen
//------------------------------------------------------------------------------

#include <Math/Vector.h>

#include <vector>

namespace Ge
{

class Mesh;

// Every Link can have a ShadowVolume, recalculated every frame from the
// light position.
class ShadowVolume
{
public:
    ShadowVolume();

    ShadowVolume(const Mesh&);

    // Sets the corresponding mesh.
    void set(const Mesh&);

    // <lightPos> must be in object space. If <lightPos>[3] == 0 then the
    // light is a directional light, otherwise it's a point light.
    // The mesh's face normals must be up to date.
    void update(const Vector4& lightPos);

    const Vector4* getVertices() const;

    UInt getVertexCount() const;

private:
    std::vector<Vector4> mVertices;
    const Mesh* mMesh;
};

} // namespace Ge

#endif // !ENGINE_SHADOW_VOLUME_H