#include "Precompiled.h"
//------------------------------------------------------------------------------
// (c) Jul-2002 Gottfried Chen
//------------------------------------------------------------------------------

#include "StateBlock.h"

namespace Ge
{

    
//------------------------------------------------------------------------------
StateBlock StateBlock::createDelta(const StateBlock& from, const StateBlock& to)
//------------------------------------------------------------------------------
{
    StateBlock block = StateBlock::createDelta();
    for (UInt i = 0; i<RenderState::TYPE_COUNT; ++i)
    {
        if (from.mStates[i] != to.mStates[i])
        {
            block.mStates[i] = to.mStates[i];
        }
    }

    return block;
}

//------------------------------------------------------------------------------
StateBlock StateBlock::createDelta()
//------------------------------------------------------------------------------
{
    StateBlock block;

    for (UInt i = 0; i<RenderState::TYPE_COUNT; ++i)
    {
        block.mStates[i] = RenderState::NO_CHANGE;
    }

    return block;
}

//------------------------------------------------------------------------------
StateBlock::StateBlock()
//------------------------------------------------------------------------------
{
    mStates[RenderState::BLEND] = RenderState::BLEND_DEFAULT;
    mStates[RenderState::FILL] = RenderState::FILL_DEFAULT;
    mStates[RenderState::SHADE] = RenderState::SHADE_DEFAULT;
    mStates[RenderState::SPECULAR] = RenderState::SPECULAR_DEFAULT;
    mStates[RenderState::DEPTH] = RenderState::DEPTH_DEFAULT;
    mStates[RenderState::DEPTH_COMPARE] = RenderState::DEPTH_COMPARE_DEFAULT;
    mStates[RenderState::STENCIL] = RenderState::STENCIL_DEFAULT;
    mStates[RenderState::COLOR_WRITE] = RenderState::COLOR_WRITE_DEFAULT;
    mStates[RenderState::CULLING] = RenderState::CULLING_DEFAULT;
    mStates[RenderState::LIGHTING] = RenderState::LIGHTING_DEFAULT;
}

//------------------------------------------------------------------------------
void StateBlock::set(RenderState::Blend rs)
//------------------------------------------------------------------------------
{
    mStates[RenderState::BLEND] = rs;
}

//------------------------------------------------------------------------------
void StateBlock::set(RenderState::Fill rs)
//------------------------------------------------------------------------------
{
    mStates[RenderState::FILL] = rs;
}

//------------------------------------------------------------------------------
void StateBlock::set(RenderState::Shade rs)
//------------------------------------------------------------------------------
{
    mStates[RenderState::SHADE] = rs;
}

//------------------------------------------------------------------------------
void StateBlock::set(RenderState::Specular rs)
//------------------------------------------------------------------------------
{
    mStates[RenderState::SPECULAR] = rs;
}

//------------------------------------------------------------------------------
void StateBlock::set(RenderState::Depth rs)
//------------------------------------------------------------------------------
{
    mStates[RenderState::DEPTH] = rs;
}

//------------------------------------------------------------------------------
void StateBlock::set(RenderState::DepthCompare rs)
//------------------------------------------------------------------------------
{
    mStates[RenderState::DEPTH_COMPARE] = rs;
}

//------------------------------------------------------------------------------
void StateBlock::set(RenderState::Stencil rs)
//------------------------------------------------------------------------------
{
    mStates[RenderState::STENCIL] = rs;
}

//------------------------------------------------------------------------------
void StateBlock::set(RenderState::ColorWrite rs)
//------------------------------------------------------------------------------
{
    mStates[RenderState::COLOR_WRITE] = rs;
}

//------------------------------------------------------------------------------
void StateBlock::set(RenderState::Culling rs)
//------------------------------------------------------------------------------
{
    mStates[RenderState::CULLING] = rs;
}

//------------------------------------------------------------------------------
void StateBlock::set(RenderState::Lighting rs)
//------------------------------------------------------------------------------
{
    mStates[RenderState::LIGHTING] = rs;
}

//------------------------------------------------------------------------------
Int8 StateBlock::get(RenderState::Type type) const
//------------------------------------------------------------------------------
{
    return mStates[type];
}

////------------------------------------------------------------------------------
//void StateBlock::apply(const StateBlock& delta)
////------------------------------------------------------------------------------
//{
//    for (UInt i = 0; i<RenderState::TYPE_COUNT; ++i)
//    {
//        if (delta.mStates[i] != RenderState::NO_CHANGE)
//        {
//            mStates[i] = delta.mStates[i];
//        }
//    }
//}

} // namespace Ge
