#ifndef ENGINE_STATE_BLOCK_H
#define ENGINE_STATE_BLOCK_H
#pragma once

//------------------------------------------------------------------------------
// (c) Jul-2002 Gottfried Chen
//------------------------------------------------------------------------------

#include "RenderState.h"

#include <System/Base.h>

namespace Ge
{

class StateBlock
{
public:
    // Creates a delta StateBlock.
    // - <from> is the currently active StateBlock in the engine.
    // - <to> are the desired RenderStates for the next primitives to be
    //   rendered.
    // - out: a StateBlock where all RenderStates needed to be set are
    //   marked accordingly and all states that are already OK are set to
    //   RenderState::NO_CHANGE.
    static
    StateBlock createDelta(const StateBlock& from, const StateBlock& to);

    // Creates an empty delta StateBlock that can be modified and then
    // applied to other StateBlocks.
    static
    StateBlock createDelta();

    // All RenderStates are set to their defaults on construction.
    StateBlock();

    void set(RenderState::Blend);
    void set(RenderState::Fill);
    void set(RenderState::Shade);
    void set(RenderState::Specular);
    void set(RenderState::Depth);
    void set(RenderState::DepthCompare);
    void set(RenderState::Stencil);
    void set(RenderState::ColorWrite);
    void set(RenderState::Culling);
    void set(RenderState::Lighting);

    // You have to cast to the correct enum type yourself.
    Int8 get(RenderState::Type) const;

//    // Apply a <delta> StateBlock. I.e. all states that are not
//    // RenderState::NO_CHANGE in <delta> are copied into this StateBlock.
//    void apply(const StateBlock& delta);

private:
    Int8 mStates[RenderState::TYPE_COUNT];
};

} // namespace Ge

#endif // !ENGINE_STATE_BLOCK_H