//------------------------------------------------------------------------------
// (c) 06-2001 Gottfried Chen
//------------------------------------------------------------------------------

#ifndef ENGINE_TEXTURE_POOL_H
#define ENGINE_TEXTURE_POOL_H

#include <Engine/Texture.h>
#include <System/Pointer.h>
#include <System/StringTools.h>
#include <map>

namespace zip
{
    class izipfile;
}


namespace Ge
{

class Image;

class TexturePool;
TexturePool& theTexturePool();

// This class manages loading of textures. Every loaded texture is registered in
// the TexturePool. Thus no textures will be loaded twice.
class TexturePool : public Object
{
public:
    // Gives access to the one and only TexturePool instance. Should be
    // set only once on application startup by the application framework.
    // Use theTexturePool to access it. It is cleaned up automatically at
    // application shutdown.
    static Pointer<TexturePool>& instancePtr();

    void setDefaultFiltering(Texture::Filtering);
    Texture::Filtering getDefaultFiltering() const;

    virtual
    Texture* get(const char* file) = 0;

    virtual
    Texture* get(zip::izipfile&, const char* file) = 0;
    
    virtual
    Texture* get(const Image&, const char* name) = 0;

    // Delete the texture with the corresponding <name> from the pool.
    void clear(const char* name);
   
protected:
    TexturePool(Texture::Filtering);
    typedef std::map<std::string, Pointer<Texture>, StringTools::Pred> Map;
    Map mTextures;

private:
    Texture::Filtering mDefaultFiltering;
};

inline
//------------------------------------------------------------------------------
TexturePool& theTexturePool()
//------------------------------------------------------------------------------
{
    return *(TexturePool::instancePtr());
}

} // namespace Ge

#endif // ENGINE_TEXTURE_POOL_H