#include "Precompiled.h"
//------------------------------------------------------------------------------
// (c) 06-2001 Gottfried Chen
//------------------------------------------------------------------------------

#include <Engine/TimeController.h>
#include <Math/Interpolator.h>
#include <System/Assert.h>

namespace Ge
{

//------------------------------------------------------------------------------
TimeController::TimeController(Interpolator& interpolator) :
//------------------------------------------------------------------------------
mRepeatType(DEFAULT),
mInterpolator(&interpolator),
mOffset(0),
mReverse(false)
{
}

//------------------------------------------------------------------------------
void TimeController::set(Interpolator& interpolator)
//------------------------------------------------------------------------------
{
    mInterpolator = &interpolator;
}

//------------------------------------------------------------------------------
void TimeController::set(Repeat repeat)
//------------------------------------------------------------------------------
{
    mRepeatType = repeat;
}

//------------------------------------------------------------------------------
TimeController::Repeat TimeController::getRepeatType() const
//------------------------------------------------------------------------------
{
    return mRepeatType;
}

//------------------------------------------------------------------------------
const Interpolator& TimeController::getInterpolator() const
//------------------------------------------------------------------------------
{
    GE_ASSERT(mInterpolator != 0);
    return *mInterpolator;
}

//------------------------------------------------------------------------------
void TimeController::reset(const Timer::Unit& time)
//------------------------------------------------------------------------------
{
    mOffset = time;
}

//------------------------------------------------------------------------------
bool TimeController::isReverse() const
//------------------------------------------------------------------------------
{
    return mReverse;
}

//------------------------------------------------------------------------------
void TimeController::flipReverse()
//------------------------------------------------------------------------------
{
    mReverse = !mReverse;
}

//------------------------------------------------------------------------------
const Timer::Unit& TimeController::getOffset() const
//------------------------------------------------------------------------------
{
    return mOffset;
}

} // namespace Ge
