//------------------------------------------------------------------------------
// (c) 06-2001 Gottfried Chen
//------------------------------------------------------------------------------

#ifndef ENGINE_TIME_CONTROLLER_H
#define ENGINE_TIME_CONTROLLER_H

#include <System/Timer.h>
#include <System/Pointer.h>

#include <Math/HermiteInterpolator.h>

namespace Ge
{

class Locator;
class Interpolator;

// A TimeController is a general base class for all aspects of an AudioVisual
// that can change over time. It can be used for all different kinds of
// animation effects.
class TimeController : public Object
{
public:
    enum Repeat
    {
        ONE_SHOT,
        LOOP,
        REVERSE,
        DEFAULT = LOOP
    };

    TimeController(Interpolator& = *(GE_NEW HermiteInterpolator));

    // Resets the TimeController's offset time to <time>. The current
    // local time of the TimeController is calculated using the offset.
    void reset(const Timer::Unit& time);

    // <time> in ms should be game time so all animations are uniform.
    // Returns false if the animation is over (for one shot anims)
    virtual
    bool tick(const Timer::Unit& time) = 0;

    virtual
    void attachTo(Locator&) = 0;

    // The interpolator must be on the heap
    void set(Interpolator&);

    void set(Repeat);
    Repeat getRepeatType() const;

protected:
    const Interpolator& getInterpolator() const;
    bool isReverse() const;
    void flipReverse();
    const Timer::Unit& getOffset() const;

private:
    Pointer<Interpolator> mInterpolator;
    Repeat mRepeatType;
    Timer::Unit mOffset;
    bool mReverse;
};

} // namespace Ge

#endif // ENGINE_TIME_CONTROLLER_H