//------------------------------------------------------------------------------
// (c) 06-2001 Gottfried Chen
//------------------------------------------------------------------------------

#ifndef ENGINE_VECTOR3_CONTROLLER_H
#define ENGINE_VECTOR3_CONTROLLER_H

#include <Engine/TimeController.h>
#include <Engine/Vector3ControllerData.h>

#include <System/Pointer.h>

namespace Ge
{

class Vector3Key;

// Vector3Controllers are TimeControllers that use  Vector3s as animation
// keys. A KeyframeController is an example of a Vector3Controller. The
// protected interface provides the function calcCurrentVector() that
// calculates the correct interpolated vector for a given animation time.
// Animation data is shared using Vector3ControllerData (as in Geometry
// and GeometryData).
class Vector3Controller : public TimeController
{
public:    
    Vector3Controller();

    // Adds a key to the controller key sequence
    void add(const Vector3Key&);
    UInt getKeyCount() const;

    // Returns the duration of the animation in ms
    const Timer::Unit& getDuration() const;

protected:
    // Verifies whether the key is correct (e.g. no negative scaling)
    virtual
    bool verifyKey(const Vector3Key&) = 0;

    // Calculates the current interpolated <vector> for a given Metronome <time>
    bool calcCurrentVector(const Timer::Unit& time, Vector3& vector);


private:
    // Returns the current time (between 0 and the duration) of the animation
    // for a given Metronome <time>
    const Timer::Unit getCurrentAnimationTime(const Timer::Unit& time);
    const Vector3ControllerData& getData() const;
    // Returns the current animation key for a given <animationTime>
    // (Interpolation is always between the current key and its successor)
    UInt getCurrentKey(const Timer::Unit& animationTime);

    Pointer<Vector3ControllerData> mData;
    UInt mCurrentKey;
    Timer::Unit mCurrentTime;
};

} // namespace Ge

#endif // ENGINE_VECTOR3_CONTROLLER_H