//------------------------------------------------------------------------------
// (c) 07-2001 Gottfried Chen
//------------------------------------------------------------------------------

#ifndef ENGINE_VERTEX_SHADER_H
#define ENGINE_VERTEX_SHADER_H

#include <Math/Vector.h>
#include <Math/Matrix.h>
#include <vector>

namespace Ge
{


// This stuff is not working yet. So don't use it!!!
// A simple VertexShader class
class VertexShader
{
public:
    struct Constant
    {
        enum Type
        {
            // This type of constant is set only once
            CUSTOM,

            // These two types change per mesh
            WORLD,
            P_V_W, // Projection*View*World

            // These types change per frame
            VIEW,
            PROJECTION,
            CAMERA_POSITION,
            LIGHT_POSITION
        };

        Constant() : mType(CUSTOM) {}
        UInt32 mRegister;
        Vector4 mValue;
        Type mType;
    };

    // Create a shader from a file. If <isBinary> is true, then the shader is
    // already precompiled (e.g. with nvasm). <declaration> contains the DirectX
    // shader declaration.
    VertexShader(const char* name, const UInt32* declaration,
                 const char* shaderFile, bool isBinary);
    // <shader> is the shader in text form
    VertexShader(const char* name, const UInt32* declaration,
                 const char* shaderCode);
    // <shader> is the shader in binary form
    VertexShader(const char* name, const UInt32* declaration,
                 const UInt32* shader);

    // These functions facilitate constant definitions, e.g. a Matrix4x4 will be
    // split into 4 Vector4s.
    void loadConstant(UInt32 position, const Vector4&);
    void loadConstant(UInt32 position, const Vector3&);
    void loadConstant(UInt32 position, const Matrix4x4&);
    // This is used for constants that change e.g. per model
    void loadConstant(UInt32 position, Constant::Type);
    

    // Returns the number of constants
    UInt getConstantCount();
    // Returns the constant at <index>
    const Constant& getConstant(UInt index) const;

    Handle getHandle() const;
    void setHandle(Handle); 

    const char* getCode() const;
    const UInt32* getTokens() const;
    const UInt32* getDeclaration();

private:
    std::string mCode;
    UInt32* mBinary;
    std::vector<Constant> mConstants;
    Handle mHandle;
};

} // namespace Ge

#endif // ENGINE_VERTEX_SHADER_H