#include "Precompiled.h"
//------------------------------------------------------------------------------
// (c) Jun-2002 Gottfried Chen
//------------------------------------------------------------------------------

#include "VideoCard.h"

#include <System/Assert.h>
#include <System/Log.h>

#include <d3d8.h>

namespace
{

using namespace Ge;
#define X(a,b) b
D3DFORMAT d3dDepthStencilFormat[] =
{
    DEPTH_STENCIL_FORMATS
};
#undef X

} // namespace

namespace Ge
{

//------------------------------------------------------------------------------
VideoCard& theVideoCard(UInt cardId)
//------------------------------------------------------------------------------
{
    // Later think of something cleaner, but for now that's good enough.
    static VideoCard c(0);
    return c;
}

struct VideoCard::Impl
{
    Impl(UInt card);

    D3DCAPS8 mCaps;
    bool mDepthStencilFormats[DS_COUNT];
};

//------------------------------------------------------------------------------
VideoCard::Impl::Impl(UInt cardId)
//------------------------------------------------------------------------------
{
    IDirect3D8* d3d = Direct3DCreate8(D3D_SDK_VERSION);
    GE_ASSERT(d3d != 0);
    
    if (d3d)
    {
        // Check that the cardId is valid
        if (cardId >= d3d->GetAdapterCount())
        {
            cardId = 0;
            GE_ASSERT_MSG(false, "Invalid cardId passed");
        }

        // Get device caps
        UInt card = cardId == 0 ? D3DADAPTER_DEFAULT : cardId;
        HRESULT ok = d3d->GetDeviceCaps(card, D3DDEVTYPE_HAL, &mCaps);
        GE_ASSERT(ok == D3D_OK);
        if (FAILED(ok))
        {
            GE_LOG_ERROR("Error getting device caps for a HAL device.\n"<<
                "Software rendering is not supported"<<std::endl);
        }

        // Check stencil formats
        D3DDISPLAYMODE displayMode;
        ok = d3d->GetAdapterDisplayMode(card, &displayMode);
        GE_ASSERT(ok == D3D_OK);
        if (SUCCEEDED(ok))
        {
            for (UInt i = 0; i<DS_COUNT; ++i)
            {
                mDepthStencilFormats[i] = d3d->CheckDeviceFormat(
                    card,
                    D3DDEVTYPE_HAL,
                    displayMode.Format,
                    D3DUSAGE_DEPTHSTENCIL,
                    D3DRTYPE_SURFACE,
                    d3dDepthStencilFormat[i]) == D3D_OK;
            }
        }

        d3d->Release();
    }
    else
    {
        GE_LOG_ERROR("Couldn't obtain Direct3D interface");
    }
}


//------------------------------------------------------------------------------
// VideoCard members
//------------------------------------------------------------------------------

//------------------------------------------------------------------------------
VideoCard::VideoCard(UInt card) : 
//------------------------------------------------------------------------------
mImpl(new Impl(card))
{
}

//------------------------------------------------------------------------------
VideoCard::~VideoCard()
//------------------------------------------------------------------------------
{
    if (mImpl != 0)
    {
        delete mImpl;
    }
}

//------------------------------------------------------------------------------
bool VideoCard::supportsHwTl() const
//------------------------------------------------------------------------------
{
    return (mImpl->mCaps.DevCaps & D3DDEVCAPS_HWTRANSFORMANDLIGHT) != 0;
}

//------------------------------------------------------------------------------
bool VideoCard::supports(VideoCard::DepthStencilFormat f) const
//------------------------------------------------------------------------------
{
    return mImpl->mDepthStencilFormats[UInt(f)];
}

//------------------------------------------------------------------------------
bool VideoCard::supportsDot3() const
//------------------------------------------------------------------------------
{
    return (mImpl->mCaps.TextureOpCaps & D3DTEXOPCAPS_DOTPRODUCT3) != 0;
}

//------------------------------------------------------------------------------
UInt VideoCard::getTextureUnitCount() const
//------------------------------------------------------------------------------
{
    return mImpl->mCaps.MaxSimultaneousTextures;
}

//------------------------------------------------------------------------------
bool VideoCard::supportsWindowedRendering() const
//------------------------------------------------------------------------------
{
    return (mImpl->mCaps.Caps2&D3DCAPS2_CANRENDERWINDOWED) != 0;
}

} // namespace Ge
