#ifndef ENGINE_VIDEO_CARD_H
#define ENGINE_VIDEO_CARD_H
#pragma once

//------------------------------------------------------------------------------
// (c) Jun-2002 Gottfried Chen
//------------------------------------------------------------------------------

namespace Ge
{

#define DEPTH_STENCIL_FORMATS \
X(DS_32, D3DFMT_D32), \
X(DS_24_8, D3DFMT_D24S8), \
X(DS_24_4, D3DFMT_D24X4S4), \
X(DS_24_0, D3DFMT_D24X8), \
X(DS_16, D3DFMT_D16), \
X(DS_15_1, D3DFMT_D15S1), \
X(DS_COUNT, D3DFMT_UNKNOWN)
    

// Represents the capabilities of a video card. If the card is not
// a HAL (capable of hardware shading) device then the constructor will assert.
class VideoCard
{
public:
#define X(a, b) a
    enum DepthStencilFormat
    {
        DEPTH_STENCIL_FORMATS
    };
#undef X

    ~VideoCard();

    // Does the device support hardware transformation & lighting?
    bool supportsHwTl() const;

    bool supports(DepthStencilFormat) const;

    bool supportsDot3() const;

    bool supportsWindowedRendering() const;

    UInt getTextureUnitCount() const;

private:
    friend VideoCard& theVideoCard(UInt cardId);

    // <cardId> identifies which videocard's caps are requested.
    VideoCard(UInt cardId);

    struct Impl;
    Impl* mImpl;
};

// Currently only id 0 is allowed.
VideoCard& theVideoCard(UInt cardId = 0);

} // namespace Ge

#endif // !ENGINE_VIDEO_CARD_H