//------------------------------------------------------------------------------
// (c) 04-2001 Gottfried Chen
//------------------------------------------------------------------------------

#ifndef ENGINE_WORLD_H
#define ENGINE_WORLD_H

#include <System/Pointer.h>

#include <Engine/Link.h>

#include <vector>

namespace Ge
{

class Camera;
class Mesh;
class Link;
class Cluster;
class Light;

// Represents the scene tree. Contains the active camera and the world root.
class World
{
public:
    explicit
    World();

    ~World();

 
    Camera* getCamera() const;
    void setCamera(Camera* camera);

    // The Mesh must live on the heap.
    void add(Mesh&);

    // The Link must live on the heap.
    void add(const Cluster&);

    const std::vector<Pointer<const Cluster> >& getClusters() const;

    // The Link must live on the heap.
    void add(Link&);

    const std::vector<Pointer<Link> >& getLinks() const;

    void add(const Light&);

    const std::vector<Pointer<const Light> >& getLights() const;

private:
    Camera* mCamera;

    std::vector<Pointer<Link> > mLinks;
    std::vector<Pointer<const Cluster> > mClusters;
    std::vector<Pointer<const Light> > mLights;
};

} // namespace Ge

#endif // ENGINE_WORLD_H