#include "Precompiled.h"
//------------------------------------------------------------------------------
// (c) 07-2001 Gottfried Chen
//------------------------------------------------------------------------------

#include "XzGrid.h"
#include "Lines.h"

namespace Ge
{

//------------------------------------------------------------------------------
Lines* XzGrid::create(UInt size, bool leaveAxesGap, const Color& color)
//------------------------------------------------------------------------------
{
    // Needed because I can't use -size otherwise
    const Int sSize(size);
    // 2*size + 1 lines, *2 -> in x and z direction, *2 -> two points per line
    const UInt vertexCount(2*2*(2*size + 1));
    Vector3* vertices = GE_NEW Vector3[vertexCount];
    Color* colors = GE_NEW Color[vertexCount];

    // Fill lines parallel to z
    for (Int i = -sSize; i<sSize+1; i++)
    {
        vertices[2*(i+sSize)] = Vector3(i,0,sSize);
        vertices[2*(i+sSize) + 1] = Vector3(i,0,-sSize);
    }

    // Lines parallel to x (start at (2*(size+size)+1) + 1 -> <size> is
    // last index above)
    UInt offset((2*(sSize+sSize)+1)+1);
    for (i = -sSize; i<sSize+1; i++)
    {
        vertices[2*(i+sSize) + offset] = Vector3(-sSize,0,i);
        vertices[2*(i+sSize) + 1 + offset] = Vector3(sSize,0,i);
    }

    if (leaveAxesGap)
    {
        vertices[2*sSize][2] = 0;
        vertices[2*sSize+1+offset][0] = 0;
    }
    
    for (i = 0; i<vertexCount; i++)
    {
        colors[i] = color;
    }

    // Hack: That's stupid, but I just wanted to get it to compile after I've
    // introduced the Lines class
    UInt16* indices = GE_NEW UInt16[vertexCount];
    for(UInt j = 0; j<vertexCount; ++j)
    {
        indices[j] = j;
    }

    return GE_NEW Lines(vertexCount, vertices, colors,
                        vertexCount, indices);
}

} // namespace Ge
