//------------------------------------------------------------------------------
// (c) 05-2001 Gottfried Chen
//------------------------------------------------------------------------------

#ifndef INPUT_BUTTON_H
#define INPUT_BUTTON_H

#include <System/Timer.h>

namespace Ge
{

// This class represents a hardware key. There is no constructor, so
// keys can only be created using the static create() member
class Button
{
public:
    Button();

    virtual
    ~Button();

    // Button is pressed this frame
    bool isPressed() const;

    bool isButtonDown() const;

    bool isButtonUp() const;

    // Call this every frame to update the key's state. Before this is
    // called. theMetronome() and all used input devices (e.g. theKeyboard())
    // must also be updated.
    void update();

protected:
    // These two reset mPressed of a component button in their
    // override() function.
    friend class ModButton;
    friend class MultiButton;

    // Implement the update function in a derived class. This function must
    // set the mPressed variable!
    virtual
    void implementUpdate() = 0;

    // This function is called after the first tick path, so that
    // MultiButtons and ModButtons can cancel their sub keys pressed/triggered
    // states
    virtual
    void override();

    bool mPressed;

private:
    UInt mLastUpdate;
    bool mPressedLastUpdate;
};

} // namespace Ge

#endif // INPUT_BUTTON_H