//------------------------------------------------------------------------------
// (c) 05-2001 Gottfried Chen
//------------------------------------------------------------------------------

#ifndef INPUT_COMBO_BUTTON_H
#define INPUT_COMBO_BUTTON_H

#include "Button.h"

#include <System/Timer.h>

#include <vector>

namespace Ge
{

// ComboButtons are buttons that represent a sequence of buttons that have to be pressed
// within a given time frame. Since ComboButtons use the abstract Button interface,
// you can construct combos that contain MultiButtons and ComboButtons.
// Combos are tolerant in this implementation (i.e. you only have to press the
// sequence in the tolerance time. If you press other buttons in between, the combo
// is not stopped
class ComboButton : public Button
{
public:
    // <tolerance> must be given in milliseconds. The whole sequence must be
    // finished in <tolerance> milliseconds.
    ComboButton(Button& first, Timer::Unit tolerance);
        
    // Adds <button> to the list of buttons that have to be pressed
    // in a row.
    void add(Button& button);

protected:
    virtual
    void implementUpdate();

private:
    std::vector<Button*> mKeys;
    Timer::Unit mFirstPress;
    Timer::Unit mTolerance;
    UInt mPendingKey;
};

} // namespace Ge

#endif // INPUT_COMBO_BUTTON_H