#include "Precompiled.h"
//------------------------------------------------------------------------------
// (c) Mai-2002 Gottfried Chen
//------------------------------------------------------------------------------

#include "Keyboard.h"

#include <System/Assert.h>

#include <dinput.h>


namespace
{

// Helper class encapsulation DirecInput keyboard device
class Device
{
public:

    static
    Device& instance()
    {
        static Device d;
        return d;
    }

    ~Device()
    {
        if (mKeyboard)
        {
            mKeyboard->Unacquire();
            mKeyboard->Release();
        }

        if (mDirectInput)
        {
            mDirectInput->Release();
        }
    }

    IDirectInputDevice8& get()
    {
        return *mKeyboard;
    }

private:

    Device()
    {
        HRESULT ok = DirectInput8Create(GetModuleHandle(0), DIRECTINPUT_VERSION,
            IID_IDirectInput8, reinterpret_cast<void**>(&mDirectInput), NULL);
        GE_ASSERT(!FAILED(ok));

        ok = mDirectInput->CreateDevice(GUID_SysKeyboard, &mKeyboard, NULL); 
        GE_ASSERT(!FAILED(ok));

        mKeyboard->SetDataFormat(&c_dfDIKeyboard);
        mKeyboard->SetCooperativeLevel(GetForegroundWindow(),
            DISCL_NONEXCLUSIVE | DISCL_BACKGROUND);
        mKeyboard->Acquire();
    }

    IDirectInputDevice8* mKeyboard;
    IDirectInput8* mDirectInput;
};

#define X(a, b) b
Ge::UInt keymap[Ge::Keyboard::KEY_COUNT] =
{
#include "KeyCodes.h"
};
#undef X

} // namespace





namespace Ge
{

//------------------------------------------------------------------------------
Keyboard& Keyboard::instance()
//------------------------------------------------------------------------------
{
    static Keyboard k;
    return k;
}

//------------------------------------------------------------------------------
Keyboard::Keyboard()
//------------------------------------------------------------------------------
{
    update();
}


//------------------------------------------------------------------------------
void Keyboard::update()
//------------------------------------------------------------------------------
{
    UInt8 keys[256];
    if (Device::instance().get().GetDeviceState(sizeof(keys), keys) == DI_OK)
    {
        for (UInt i = 0; i<KEY_COUNT; ++i)
        {
            mPressed[i] = (keys[keymap[i]] & 0x80) != 0;
        }
    }
    else
    {
        GE_ASSERT_MSG(false, "Error retrieving keyboard state.");
    }
}

//------------------------------------------------------------------------------
bool Keyboard::isPressed(Keyboard::Button b) const
//------------------------------------------------------------------------------
{
    return mPressed[b];
}

} // namespace Ge
