//------------------------------------------------------------------------------
// (c) 05-2001 Gottfried Chen
//------------------------------------------------------------------------------

#ifndef INPUT_MAPPED_COMMAND_H
#define INPUT_MAPPED_COMMAND_H

#include <string>
#include <vector>

#include <System/Pointer.h>

#include <Input/Button.h>

namespace Ge
{

class Command;

// MappedCommands are Commands that are mapped onto Keys.
// Usage:
// - Create the appropriate MappedCommand
// - Tick all MappedCommands in a loop
// - Execute all MappedCommands
// - Reset all MappedCommands
class MappedCommand : public Object
{
public:
    // <continuous> buttons are e.g.: move forward, non continuous buttons
    // are e.g. fire buttons. <command> must be on the heap
    MappedCommand(Command& command, Button& key, bool continuous = false);


    // Returns the name of the command associated with this MappedCommand
    const std::string getString() const;
    
    // Ticks the button associated with the command
    void update();

    // Execute the command bound to the button, depending on button status
    // Returns true if executed
    bool execute();

private:
    Button* mKey;
    Pointer<Command> mCommand;
    bool mContinuous;
};

} // namespace Ge

#endif // INPUT_MAPPED_COMMAND_H