#include "Precompiled.h"
//------------------------------------------------------------------------------
// (c) 03-2001 Gottfried Chen
//------------------------------------------------------------------------------

#include "Mouse.h"
#include <System/Assert.h>

#include <windows.h>
#include <dinput.h>

namespace
{

// Helper class encapsulation DirecInput mouse device
class Device
{
public:

    static
    Device& instance()
    {
        static Device d;
        return d;
    }

    ~Device()
    {
        if (mMouse)
        {
            mMouse->Unacquire();
            mMouse->Release();
        }

        if (mDirectInput)
        {
            mDirectInput->Release();
        }
    }

    IDirectInputDevice8& get()
    {
        return *mMouse;
    }

private:

    Device()
    {
        HRESULT ok = DirectInput8Create(GetModuleHandle(0), DIRECTINPUT_VERSION,
            IID_IDirectInput8, reinterpret_cast<void**>(&mDirectInput), NULL);
        GE_ASSERT(!FAILED(ok));

        ok = mDirectInput->CreateDevice(GUID_SysMouse, &mMouse, NULL); 
        GE_ASSERT(!FAILED(ok));

        mMouse->SetDataFormat(&c_dfDIMouse);
        mMouse->SetCooperativeLevel(GetForegroundWindow(),
                                    DISCL_NONEXCLUSIVE | DISCL_BACKGROUND);
        mMouse->Acquire();
    }

    IDirectInputDevice8* mMouse;
    IDirectInput8* mDirectInput;
};

} // namespace


namespace Ge
{

//------------------------------------------------------------------------------
Mouse::Mouse()
//------------------------------------------------------------------------------
{
    update();
}

//------------------------------------------------------------------------------
Mouse& Mouse::instance()
//------------------------------------------------------------------------------
{
    static Mouse mouse;
    return mouse;
}

//------------------------------------------------------------------------------
void Mouse::update()
//------------------------------------------------------------------------------
{
    // Read absolute mouse position
    POINT point;
    if (GetCursorPos(&point) == 0)
    {
        GE_ASSERT_MSG(false, "Couldn't retrieve absolute mouse position.");
    }

    mAbsolute[0] = point.x;
    mAbsolute[1] = point.y;

    // Read DirectInput mouse info
    DIMOUSESTATE mouseInfo;
    bool mouseInfoOk = Device::instance().get().
        GetDeviceState(sizeof(mouseInfo), &mouseInfo) == DI_OK;
    GE_ASSERT_MSG(mouseInfoOk, "Couldn't retrieve DIMOUSESTATE");

    // Read relative mouse position
    if (mouseInfoOk)
    {
        mDelta[0] = mouseInfo.lX;
        mDelta[1] = mouseInfo.lY;
        mWheel = mouseInfo.lZ;
    }
    
    // Read button states
    if (mouseInfoOk)
    {
        mPressed[LEFT] = (mouseInfo.rgbButtons[LEFT] & 0x80) != 0;
        mPressed[MIDDLE] = (mouseInfo.rgbButtons[MIDDLE] & 0x80) != 0;
        mPressed[RIGHT] = (mouseInfo.rgbButtons[RIGHT] & 0x80) != 0;
    }
}

//------------------------------------------------------------------------------
void Mouse::getAbsolute(Vector2& pos) const
//------------------------------------------------------------------------------
{
    pos = mAbsolute;
}


//------------------------------------------------------------------------------
void Mouse::getDelta(Vector2& pos) const
//------------------------------------------------------------------------------
{
    pos = mDelta;
}

//------------------------------------------------------------------------------
Float Mouse::getWheelDelta() const
//------------------------------------------------------------------------------
{
    return mWheel;
}

//------------------------------------------------------------------------------
bool Mouse::isPressed(Mouse::Button b) const
//------------------------------------------------------------------------------
{
    return mPressed[b];
}

} // namespace Ge

