#ifndef INPUT_REPEAT_BUTTON_H
#define INPUT_REPEAT_BUTTON_H
#pragma once

//------------------------------------------------------------------------------
// (c) Jun-2002 Gottfried Chen
//------------------------------------------------------------------------------

#include "Button.h"

#include <System/Timer.h>

namespace Ge
{

// A RepeatButton takes a reference for a button and triggers isButtonDown()
// regularily like a repeating key under Windows.
class RepeatButton : public Button
{
public:
    // Default:
    // The default delay until the repeats start is 500ms.
    // The repeat rate is 50ms.
    // 
    // <button> is the button that will be checked for repeats.
    RepeatButton(Button& button);

    //------------------------------------------------------------------------------
    // Accessors
    //------------------------------------------------------------------------------

    // Time passed/returned is in ms.

    Timer::Unit getDelay() const;

    void setDelay(const Timer::Unit&);

    Timer::Unit getRate() const;

    void setRate(const Timer::Unit&);

protected:
    virtual
    void implementUpdate();

private:
    Timer::Unit mDelay;
    Timer::Unit mRate;
    
    // Time to wait until isButtonDown() is triggered.
    Timer::Unit mWait;
    // Absolute time waiting has started.
    Timer::Unit mWaitStart;

    Button* mButton;
};

} // namespace Ge

#endif // !INPUT_REPEAT_BUTTON_H