//------------------------------------------------------------------------------
// (c) 02-2002 Gottfried Chen
//------------------------------------------------------------------------------

#ifndef FFT_H
#define FFT_H

#include "FftRoutines.h"

// Comment this out if you don't want to use FFTW but rather my own FFT
// routines (they are slower but are not bound by the FFTW open source
// license). You can also undefine this, if you run into troubles using
// FFTW.
#define USE_FFTW

#ifdef USE_FFTW
#   include <fftw.h>
#   include <rfftw.h>
#   ifndef FFTW_ENABLE_FLOAT
#       error You have add "#define FFTW_ENABLE_FLOAT" to fftw.h
#   endif
#endif



// Usage: Don't use any of the classes below!!!
//
// Use the classes:
// Fft, Fft2D, DFft and DFft2D.
//
// They are typedefed at the bottom of this file.
// This ensures that the correct FFT implementation will be used depending
// on the USE_FFTW #define.

namespace Ge
{

// The array must contain 2^<sizePower> elements.
template<class T>
class MyFft
{
public:
    MyFft(UInt sizePower, Int sign = 1);
    void calculate(std::complex<T>* data) const;

private:
    UInt mSizePower;
    Int mSign;
};

// Works on an x*y array, where x is the number of elements in a row and y
// the number of elements in a column. The row index varries fastest, i.e.
// the array is an array of rows.
template<class T>
class MyFft2D
{
public:
    MyFft2D(UInt xSizePower, UInt ySizePower, Int sign = 1);
    void calculate(std::complex<T>* data) const;

private:
    UInt mXSizePower;
    UInt mYSizePower;
    Int mSign;
};

#ifdef USE_FFTW

    class Fftw
    {
    public:
        Fftw(UInt sizePower, Int sign = 1);
        ~Fftw();

        void calculate(std::complex<Float>* data) const;

    private:
        fftw_plan mPlan;
    };

    class Fftw2D
    {
    public:
        Fftw2D(UInt xSizePower, UInt ySizePower, Int sign = 1);
        ~Fftw2D();

        void calculate(std::complex<Float>* data) const;

    private:
        fftwnd_plan mPlan;
    };

    typedef Fftw Fft;
    typedef Fftw2D Fft2D;

#else // USE_FFTW

    typedef MyFft<Float> Fft;
    typedef MyFft2D<Float> Fft2D;

#endif // USE_FFTW


// Double versions of the above (no FFTW used for that now)
typedef MyFft<Double> DFft;
typedef MyFft2D<Double> DFft2D;

} // namespace Ge

#include "Fft.inl"

#endif // FFT_H