//------------------------------------------------------------------------------
// (c) 01-2002 Gottfried Chen
//------------------------------------------------------------------------------

#ifndef FILTER_H
#define FILTER_H

namespace Ge
{

namespace Filter
{

// Filter direction
enum Direction
{
    FORWARD,
    REVERSE,
    BIDIRECTIONAL
};

} // namespace Filter


// Applies a FIR filter to <array>. <constant> denotes c below.
// If array is x_0,...,x_n then the filtered sequence y_0,...,y_n is
// calculated by y_i = c*y_(i-1) + (1-c)*x_i. <stride> is the stride
// from x_i to x_i+1.
template<class T> inline
void fir(T* array, unsigned int size, unsigned int stride, float constant,
         Filter::Direction fd);

// Applies an FIR filter to the 2-dimensional <array>. The filter is first
// applied to all rows, then all columns.
template<class T> inline
void firSquare(T* array, unsigned int xSize, unsigned int ySize, float constant,
         Filter::Direction fd);

} // namespace Ge

#include "Filter.inl"
#endif // FILTER_H
