//------------------------------------------------------------------------------
// (c) 01-2002 Gottfried Chen
//------------------------------------------------------------------------------

#include <System/Assert.h>

namespace Ge
{

template<class T> inline
//------------------------------------------------------------------------------
void fir(T* array, unsigned int size, unsigned int stride, float constant,
         Filter::Direction fd)
//------------------------------------------------------------------------------
{
    GE_ASSERT(size >= 1);
    GE_ASSERT(0<=constant && constant<=1);

    T* current = 0;
    T* previous = 0;
    if (fd == Filter::FORWARD || fd == Filter::BIDIRECTIONAL)
    {
        previous = array;
        current = previous + stride;

        for (unsigned int i = 0; i<size-1; ++i)
        {
            *current = constant*float(*previous) + (1-constant)*float(*current);
            current += stride;
            previous += stride;
        }
    }

    if (fd == Filter::REVERSE || fd == Filter::BIDIRECTIONAL)
    {
        previous = array + stride*(size-1);
        current = previous - stride;

        for (unsigned int i = 0; i<size-1; ++i)
        {
            *current = constant*float(*previous) + (1-constant)*float(*current);
            current -= stride;
            previous -= stride;
        }
    }    
}

template<class T> inline
//------------------------------------------------------------------------------
void firSquare(T* array, unsigned int xSize, unsigned int ySize, float constant,
               Filter::Direction fd)
//------------------------------------------------------------------------------
{
    unsigned int x(0), y(0);
    // Apply to rows
    for (y = 0; y<ySize; ++y)
    {
        fir(array + y*xSize, xSize, 1, constant, fd);
    }

    // Apply to columns
    for (x = 0; x<xSize; ++x)
    {
        fir(array + x, ySize, xSize, constant, fd);
    }
}

} // namespace Ge