#include "Precompiled.h"

//------------------------------------------------------------------------------
// (c) 06-2001 Gottfried Chen
//------------------------------------------------------------------------------

#include <Math/Frustum.h>
#include <Math/Sphere.h>

namespace Ge
{

//------------------------------------------------------------------------------
Frustum::Frustum()
//------------------------------------------------------------------------------
{
}

//------------------------------------------------------------------------------
Frustum::Frustum(const Float& nearPlane, const Float& farPlane,
                 const Float& fovHorizontal, const Float& fovVertical)
//------------------------------------------------------------------------------
{
    recalculate(nearPlane, farPlane, fovHorizontal, fovVertical);
}

//------------------------------------------------------------------------------
void Frustum::recalculate(const Float& nearPlane, const Float& farPlane,
                          const Float& fovHorizontal, const Float& fovVertical)
//------------------------------------------------------------------------------
{
    GE_ASSERT(nearPlane>EPSILON);
    GE_ASSERT(farPlane>nearPlane);
    GE_ASSERT(EPSILON<fovHorizontal && fovHorizontal<PI);
    GE_ASSERT(EPSILON<fovVertical && fovVertical<PI);
   
    mSpaces[NEAR_PLANE].set(Plane(Vector3(0,0,1), nearPlane));
    // -farPlane, because normal points to viewer
    mSpaces[FAR_PLANE].set(Plane(Vector3(0,0,-1), -farPlane));
    
    Float ch(cos(fovHorizontal/2));
    Float sh(sin(fovHorizontal/2));
    mSpaces[LEFT_PLANE].set(Plane(Vector3(ch,0,sh), 0));
    mSpaces[RIGHT_PLANE].set(Plane(Vector3(-ch,0,sh), 0));

    Float cv(cos(fovVertical/2));
    Float sv(sin(fovVertical/2));
    mSpaces[TOP_PLANE].set(Plane(Vector3(0,-cv,sv), 0));
    mSpaces[BOTTOM_PLANE].set(Plane(Vector3(0,cv,sv), 0));
}

//------------------------------------------------------------------------------
Frustum::Frustum(const Plane* planes)
//------------------------------------------------------------------------------
{
    for (UInt i = 0; i<6; i++)
        mSpaces[i].set(planes[i]);
}


//------------------------------------------------------------------------------
Intersect::Type Frustum::intersects(const Sphere& sphere) const
//------------------------------------------------------------------------------
{
    Intersect::Type ret(Intersect::CONTAINS);
    Intersect::Type temp;
    for (UInt i = 0; i<6 && ret != Intersect::DISJOINT; i++)
    {
        temp = mSpaces[i].intersects(sphere);
        if (temp < ret)
        {
            ret = temp;
        }
    }

    return ret;
}

} // namespace Ge

