//------------------------------------------------------------------------------
// (c) 06-2001 Gottfried Chen
//------------------------------------------------------------------------------

#include <Math/HalfSpace.h>

namespace Ge
{

// Represents a viewing frustum. Used for visibility checks.
// The frustum is positioned at the origin and looks down the positive z-Axis
// after construction/recalculate(...). It can then be transformed for
// positioning in world space.
class Frustum
{
public:
    // Doesn't do anything. So the frustum is uninitialized!
    Frustum();

    Frustum(const Float& nearPlane, const Float& farPlane,
            const Float& fovHorizontal, const Float& fovVertical);
    
    void recalculate(const Float& nearPlane, const Float& farPlane,
                     const Float& fovHorizontal, const Float& fovVertical);
    
    template<class MATRIX>
    Frustum transform(const MATRIX& matrix) const
    {
        Plane planes[6];
        for (UInt i = 0; i<6; i++)
        {
            planes[i] = mSpaces[i].getPlane().transform(matrix);
        }
    
        return Frustum(planes);
    }

    Intersect::Type intersects(const Sphere&) const;

private:
    // Pass an array of 6 planes
    Frustum(const Plane*);

    // Added _PLANE, because NEAR, FAR are Microsoft macros, and changing
    // #include<...> order didn't work in one special case (MFC apps are not
    // allowed to #include<windows.h>!)
    enum
    {
        NEAR_PLANE,
        FAR_PLANE,
        LEFT_PLANE,
        RIGHT_PLANE,
        TOP_PLANE,
        BOTTOM_PLANE
    };

    // Order is near, far, left, right, top, bottom
    HalfSpace mSpaces[6];
};


} // namespace Ge