#include "Precompiled.h"

//------------------------------------------------------------------------------
// (c) 05-2001 Gottfried Chen
//------------------------------------------------------------------------------

#include <Math/HalfSpace.h>
#include <Math/Sphere.h>
#include <Math/Vector.h>

namespace Ge
{

//------------------------------------------------------------------------------
HalfSpace::HalfSpace() :
//------------------------------------------------------------------------------
mPlane(Vector3(0,0,1), 0)
{
}

//------------------------------------------------------------------------------
HalfSpace::HalfSpace(const Plane& plane) :
//------------------------------------------------------------------------------
mPlane(plane)
{
}

//------------------------------------------------------------------------------
void HalfSpace::set(const Plane& plane)
//------------------------------------------------------------------------------
{
    mPlane = plane;
}

//------------------------------------------------------------------------------
Intersect::Type HalfSpace::intersects(const Sphere& sphere) const
//------------------------------------------------------------------------------
{
    // Algorithm: sphere.center*plane.normal is the offset of the sphere's
    // center (i.e. the length of normal projection of the center onto
    // the normal), because the plane's normal is normalized
    // If offset > plane.offset + radius, then the sphere is contained in the
    // halfspace
    // If offset < plane.offset - radius, then the sphere is disjoint from the
    // halfspace
    // centerOffsetMinPlaneOffset subtracts the plane.offset, to save one
    // subtraction in the two if statements.
    Intersect::Type ret(Intersect::INTERSECTS);
    Float centerOffsetMinPlaneOffset(sphere.getCenter()*mPlane.getNormal() -
                                     mPlane.getOffset());
    if (centerOffsetMinPlaneOffset > sphere.getRadius())
    {
        ret = Intersect::CONTAINS;
    }
    else if (centerOffsetMinPlaneOffset < - sphere.getRadius())
    {
        ret = Intersect::DISJOINT;
    }

    return ret;    
}

//------------------------------------------------------------------------------
bool HalfSpace::contains(const Vector3& vector) const
//------------------------------------------------------------------------------
{
    return vector*mPlane.getNormal() > mPlane.getOffset();
}

//------------------------------------------------------------------------------
const Plane& HalfSpace::getPlane() const
//------------------------------------------------------------------------------
{
    return mPlane;
}

} // namespace Ge

