#include "Precompiled.h"

//------------------------------------------------------------------------------
// (c) 06-2001 Gottfried Chen
//------------------------------------------------------------------------------

#include <Math/Interpolation.h>
#include <System/Assert.h>

namespace Ge
{

//------------------------------------------------------------------------------
Float lerp(Float x0, Float y0, Float x1, Float y1,
           Float f00, Float f10, Float f11, Float f01,
           Float x, Float y)
//------------------------------------------------------------------------------
{
    // Lerp in x direction on upper and lower side.
    Float flx0 = lerp(x0, x1, f00, f10, x);
    Float flx1 = lerp(x0, x1, f01, f11, x);

    // Lerp in y direction between the result values.
    return lerp(y0, y1, flx0, flx1, y);
}


//------------------------------------------------------------------------------
Float lerp(Float x0, Float x1, Float f0, Float f1, Float x)
//------------------------------------------------------------------------------
{
    return f0 != f1 ? lerp(f0, f1, toUnitInterval(x0, x1, x)) : f0;
}

//------------------------------------------------------------------------------
Float lerp(Float f0, Float f1, Float s)
//------------------------------------------------------------------------------
{
    GE_ASSERT(0<=s && s<=1);
    return f0 != f1 ? f0*(1-s) + f1*s : f0;
}

//------------------------------------------------------------------------------
Float hermite(Float f0, Float f1,
              Float df0, Float df1,
              Float s)
//------------------------------------------------------------------------------
{
    GE_ASSERT(0<=s && s<=1);

//    Float s2(s*s);
//    Float s3(s*s*s);
//    Float h0(2*s3-3*s2+1);
//    Float h1(-2*s3+3*s2);
//    Float h2(s3-2*s2+s);
//    Float h3(s3-s2);
//
//    return h0*f0 + h1*f1 + h2*df0 + h3*df1;
    static Float hermite[1001][4];
    static bool init(true);
    if (init)
    {
        Float v(0), v2(0), v3(0);
        for (UInt i = 0; i<1001; i++)
        {
            v = Float(i)/1000;
            v2 = v*v;
            v3 = v2*v;
            hermite[i][0] = 2*v3 - 3*v2 + 1;
            hermite[i][1] = -2*v3 + 3*v2;
            hermite[i][2] = v3 -2*v2 + v;
            hermite[i][3] = v3 -v2;
        }

        init = false;
    }

    Int sInt(Int(s*1000.0f));
    return hermite[sInt][0]*f0 + hermite[sInt][1]*f1 +
           hermite[sInt][2]*df0 + hermite[sInt][3]*df1;
}

//------------------------------------------------------------------------------
Float toUnitInterval(const Float& t0, const Float& t1, const Float& t)
//------------------------------------------------------------------------------
{
    GE_ASSERT(t0<=t && t<=t1);
    return (t-t0)/(t1-t0);
}

} // namespace Ge
