//------------------------------------------------------------------------------
// (c) 06-2001 Gottfried Chen
//------------------------------------------------------------------------------

#ifndef MATH_INTERPOLATION_H
#define MATH_INTERPOLATION_H

#include <System/Types.h>

#include "Vector.h"
#include "Matrix.h"

namespace Ge
{

// 0 -> f0
// 1 -> f1
// s in [0, 1]
Float lerp(Float f0, Float f1, Float s);

// Lerp two vectors componentwise
template<typename T, UInt DIM>
void lerp(BasicVector<T, DIM>& result, const BasicVector<T, DIM>& f0,
          const BasicVector<T, DIM>& f1, Float s);

// Lerp two matrices componentwise
template<typename T, UInt R, UInt C>
void lerp(BasicMatrix<T, R, C>& result, const BasicMatrix<T, R, C>& f0,
          const BasicMatrix<T, R, C>& f1, Float s);


// x0 -> f0
// x1 -> f1
// x in [x0, x1], x0<x1
Float lerp(Float x0, Float x1, Float f0, Float f1, Float x);

// 2 dimensional lerp (first in x, than y direction)
// (x0, y0) -> f00, ... , (x1, y1) -> f11
Float lerp(Float x0, Float y0, Float x1, Float y1,
           Float f00, Float f10, Float f11, Float f01,
           Float x, Float y);

// Perform interpolation with Hermite splines.
// <f0>, <f1> have the same meaning as in the linear case.
// <df0>, <df1> denote the fist derivatives f'(0) and f'(1).
// <s> must be between 0 and 1.
Float hermite(Float f0, Float f1,
              Float df0, Float df1,
              Float s);

// All interpolation methods expect a parameter <s> in the unit interval (0,1).
// During most processes like keyframe interpolation you have two given time
// values <t0>, <t1> and a current time <t> with the function/tangent values
// at <t0> and <t1>. Use this function to convert <t> into the needed parameter
// <s>. (The function and tangent values don't need to be converted).
Float toUnitInterval(const Float& t0, const Float& t1, const Float& t);

} // namespace Ge

#include "Interpolation.inl"
#endif // MATH_INTERPOLATION_H
