#include "Precompiled.h"
//------------------------------------------------------------------------------
// (c) Jun-2002 Gottfried Chen
//------------------------------------------------------------------------------

#include "Intersection.h"

#include "Plane.h"
#include "Ray.h"


namespace Ge
{

//------------------------------------------------------------------------------
Intersect::Type intersects(const Plane& p, const Ray& r, Vector3* point)
//------------------------------------------------------------------------------
{
    Intersect::Type ret = Intersect::DISJOINT;
    Float nd = p.getNormal()*r.getDirection();
    
    if (nd == 0)
    {
        // Plane is parallel to ray or contains it
        if (isAlmost(p.getDistance(r.getStart()), 0.0f))
        {
            ret = Intersect::CONTAINS;
        }
    }
    else
    {
        // Either ray or negative ray intersect.
        Float t = (p.getOffset() - (p.getNormal()*r.getStart())) / nd;
        if (t >= 0)
        {
            ret = Intersect::INTERSECTS;

            if (point != 0)
            {
                *point = r.getStart() + t*r.getDirection();
            }
        }
    }

    return ret;
}

//------------------------------------------------------------------------------
Intersect::Type intersects(const Ray& r, const Plane& p, Vector3* point)
//------------------------------------------------------------------------------
{
    return intersects(p, r, point);
}

} // namespace Ge
