//------------------------------------------------------------------------------
// (c) Apr-2002 Gottfried Chen
//------------------------------------------------------------------------------

#include "Quaternion.h"

namespace Ge
{

template<typename T, UInt R, UInt C>
//------------------------------------------------------------------------------
BasicMatrix<T, R, C>& BasicMatrix<T, R, C>::IDENTITY()
//------------------------------------------------------------------------------
{
    static BasicMatrix<T, R, C> id;
    if (id(0,0) == 0)
    {
        // Didn't use minimum(R,C), because that didn't compile with VC6.
        for (UInt i = 0; i<(R<C?R:C); ++i)
        {
            id(i,i) = 1;
        }
    }

    return id;
}

template<typename T, UInt R, UInt C>
//------------------------------------------------------------------------------
BasicMatrix<T, R, C>& BasicMatrix<T, R, C>::ZERO()
//------------------------------------------------------------------------------
{
    static BasicMatrix<T, R, C> z;
    return z;
}


template<typename T, UInt R, UInt C>
//------------------------------------------------------------------------------
BasicMatrix<T, R, C>::BasicMatrix()
//------------------------------------------------------------------------------
{
}


template<typename T, UInt R, UInt C>
//------------------------------------------------------------------------------
BasicMatrix<T, R, C>::BasicMatrix(const BasicMatrix<T, R, C>& rhs)
//------------------------------------------------------------------------------
{
    memcpy(mData, rhs.mData, sizeof(T)*R*C);
}

 
template<typename T, UInt R, UInt C>
//------------------------------------------------------------------------------
const T& BasicMatrix<T, R, C>::operator()(UInt row, UInt column) const
//------------------------------------------------------------------------------
{
    GE_ASSERT(row<R && column<C);
    return mData[row][column];
}


template<typename T, UInt R, UInt C>
//------------------------------------------------------------------------------
T& BasicMatrix<T, R, C>::operator()(UInt row, UInt column)
//------------------------------------------------------------------------------
{
    GE_ASSERT(row<R && column<C);
    return mData[row][column];
}


template<typename T, UInt R, UInt C>
//------------------------------------------------------------------------------
void BasicMatrix<T, R, C>::getRow(UInt row, BasicVector<T, C>& v) const
//------------------------------------------------------------------------------
{
    GE_ASSERT(row<R);
    memcpy(&v, &(*this)(row, 0), sizeof(T)*C);
}


template<typename T, UInt R, UInt C>
//------------------------------------------------------------------------------
void BasicMatrix<T, R, C>::setRow(UInt row, const BasicVector<T, C>& v)
//------------------------------------------------------------------------------
{
    GE_ASSERT(row<R);
    memcpy(&(*this)(row, 0), &v, sizeof(T)*C);
}

template<typename T, UInt R, UInt C>
//------------------------------------------------------------------------------
void BasicMatrix<T, R, C>::addRow(UInt row, const BasicVector<T, C>& v)
//------------------------------------------------------------------------------
{
    for (UInt i = 0; i<C; ++i)
    {
        (*this)(row, i) += v[i];
    }
}

template<typename T, UInt R, UInt C>
//------------------------------------------------------------------------------
void BasicMatrix<T, R, C>::normalizeRow(UInt row)
//------------------------------------------------------------------------------
{
    T l = 0;
    for (UInt i = 0; i<C; ++i)
    {
        l += square((*this)(row, i));
    }
    
    GE_ASSERT(l > 0);
    if (l>0)
    {
        l = sqrt(l);
        for (UInt i = 0; i<C; ++i)
        {
            (*this)(row, i) /= l;
        }
    }
}


template<typename T, UInt R, UInt C>
//------------------------------------------------------------------------------
void BasicMatrix<T, R, C>::getColumn(UInt column, BasicVector<T, R>& v) const
//------------------------------------------------------------------------------
{
    GE_ASSERT(column<C);
    for (UInt i = 0; i<R; ++i)
    {
        v[i] = (*this)(i,column);
    }
}


template<typename T, UInt R, UInt C>
//------------------------------------------------------------------------------
void BasicMatrix<T, R, C>::setColumn(UInt column, const BasicVector<T, R>& v)
//------------------------------------------------------------------------------
{
    GE_ASSERT(column<C);
    for (UInt i = 0; i<R; ++i)
    {
        (*this)(i, column) = v[i];
    }
}

template<typename T, UInt R, UInt C>
//------------------------------------------------------------------------------
void BasicMatrix<T, R, C>::addColumn(UInt column, const BasicVector<T, R>& v)
//------------------------------------------------------------------------------
{
    for (UInt i = 0; i<R; ++i)
    {
        (*this)(i, column) += v[i];
    }
}



template<typename T, UInt R, UInt C>
//------------------------------------------------------------------------------
void BasicMatrix<T, R, C>::normalizeColumn(UInt column)
//------------------------------------------------------------------------------
{
    T l = 0;
    for (UInt i = 0; i<R; ++i)
    {
        l += square((*this)(i, column));
    }
    
    GE_ASSERT(l > 0);
    if (l>0)
    {
        l = sqrt(l);
        for (UInt i = 0; i<R; ++i)
        {
            (*this)(i, column) /= l;
        }
    }
}


template<typename T>
//------------------------------------------------------------------------------
void setColumn(BasicMatrix<T, 4, 4>& m, UInt column,
               const T& x, const T& y, const T& z, const T& w)
//------------------------------------------------------------------------------
{
    GE_ASSERT(column<4);
    m(0,column) = x;
    m(1,column) = y;
    m(2,column) = z;
    m(3,column) = w;
}

template<typename T>
//------------------------------------------------------------------------------
void setRow(BasicMatrix<T, 4, 4>& m, UInt row,
            const T& x, const T& y, const T& z, const T& w)
//------------------------------------------------------------------------------
{
    GE_ASSERT(row<4);
    m(row, 0) = x;
    m(row, 1) = y;
    m(row, 2) = z;
    m(row, 3) = w;
}


template<typename T>
//------------------------------------------------------------------------------
void setColumn(BasicMatrix<T, 3, 4>& m, UInt column,
               const T& x, const T& y, const T& z)
//------------------------------------------------------------------------------
{
    GE_ASSERT(column<4);
    m(0,column) = x;
    m(1,column) = y;
    m(2,column) = z;
}

template<typename T>
//------------------------------------------------------------------------------
void setColumn(BasicMatrix<T, 3, 3>& m, UInt column,
               const T& x, const T& y, const T& z)
//------------------------------------------------------------------------------
{
    GE_ASSERT(column<3);
    m(0,column) = x;
    m(1,column) = y;
    m(2,column) = z;
}

template<typename T>
//------------------------------------------------------------------------------
void set(BasicMatrix<T, 3, 3>& m,
         const T& _00, const T& _01, const T& _02,
         const T& _10, const T& _11, const T& _12,
         const T& _20, const T& _21, const T& _22)
//------------------------------------------------------------------------------
{
    m(0,0) = _00;
    m(0,1) = _01;
    m(0,2) = _02;

    m(1,0) = _10;
    m(1,1) = _11;
    m(1,2) = _12;

    m(2,0) = _20;
    m(2,1) = _21;
    m(2,2) = _22;
}

template<typename T>
//------------------------------------------------------------------------------
void set(BasicMatrix<T, 4, 4>& m,
         const T& _00, const T& _01, const T& _02, const T& _03,
         const T& _10, const T& _11, const T& _12, const T& _13,
         const T& _20, const T& _21, const T& _22, const T& _23,
         const T& _30, const T& _31, const T& _32, const T& _33)
//------------------------------------------------------------------------------
{
    m(0,0) = _00;
    m(0,1) = _01;
    m(0,2) = _02;
    m(0,3) = _03;

    m(1,0) = _10;
    m(1,1) = _11;
    m(1,2) = _12;
    m(1,3) = _13;

    m(2,0) = _20;
    m(2,1) = _21;
    m(2,2) = _22;
    m(2,3) = _23;

    m(3,0) = _30;
    m(3,1) = _31;
    m(3,2) = _32;
    m(3,3) = _33;
}

template<typename T, UInt R, UInt C>
//------------------------------------------------------------------------------
BasicMatrix<T, R, C>&
BasicMatrix<T, R, C>::operator =(const BasicMatrix<T, R, C>& rhs)
//------------------------------------------------------------------------------
{
    if (&rhs != this)
    {
        memcpy(mData, rhs.mData, sizeof(T)*R*C);
    }

    return *this;
}


template<typename T, UInt R, UInt C>
//------------------------------------------------------------------------------
bool BasicMatrix<T, R, C>::operator ==(const BasicMatrix& rhs) const
//------------------------------------------------------------------------------
{
    for (UInt r = 0; r<R; ++r)
    {
        for (UInt c = 0; c<C; ++c)
        {
            if (mData[r][c] != rhs.mData[r][c])
            {
                return false;
            }
        }
    }

    return true;
}


template<typename T, UInt R, UInt C>
//------------------------------------------------------------------------------
bool BasicMatrix<T, R, C>::operator !=(const BasicMatrix& rhs) const
//------------------------------------------------------------------------------
{
    return !(*this == rhs);
}


template<typename T, UInt R, UInt C>
//------------------------------------------------------------------------------
bool BasicMatrix<T, R, C>::isAlmost(const BasicMatrix& rhs, Float tolerance) const
//------------------------------------------------------------------------------
{
    for (UInt r = 0; r<R; ++r)
    {
        for (UInt c = 0; c<C; ++c)
        {
            if (!::Ge::isAlmost(mData[r][c], rhs.mData[r][c], tolerance))
            {
                return false;
            }
        }
    }

    return true;
}





//------------------------------------------------------------------------------
// Globals
//------------------------------------------------------------------------------

template<typename T>
//------------------------------------------------------------------------------
BasicVector<T, 3> operator *(const BasicMatrix<T, 4, 4>& lhs,
                             const BasicVector<T, 3>& rhs)
//------------------------------------------------------------------------------
{
    T w = lhs(3,0)*rhs[0] + lhs(3,1)*rhs[1] + lhs(3,2)*rhs[2] + lhs(3,3);
    GE_ASSERT(w != 0);
    w = 1.0f/w;

    return BasicVector<T, 3>(
        w*(lhs(0,0)*rhs[0] + lhs(0,1)*rhs[1] + lhs(0,2)*rhs[2] + lhs(0,3)),
        w*(lhs(1,0)*rhs[0] + lhs(1,1)*rhs[1] + lhs(1,2)*rhs[2] + lhs(1,3)),
        w*(lhs(2,0)*rhs[0] + lhs(2,1)*rhs[1] + lhs(2,2)*rhs[2] + lhs(2,3)));
}

template<typename T>
//------------------------------------------------------------------------------
BasicVector<T, 3> operator *(const BasicMatrix<T, 3, 4>& lhs,
                             const BasicVector<T, 3>& rhs)
//------------------------------------------------------------------------------
{
    return BasicVector<T, 3>(
        lhs(0,0)*rhs[0] + lhs(0,1)*rhs[1] + lhs(0,2)*rhs[2] + lhs(0,3),
        lhs(1,0)*rhs[0] + lhs(1,1)*rhs[1] + lhs(1,2)*rhs[2] + lhs(1,3),
        lhs(2,0)*rhs[0] + lhs(2,1)*rhs[1] + lhs(2,2)*rhs[2] + lhs(2,3));
}

template<typename T>
//------------------------------------------------------------------------------
BasicMatrix<T, 3, 3> operator *(const BasicMatrix<T, 3, 3>& lhs,
                                const BasicMatrix<T, 3, 3>& rhs)
//------------------------------------------------------------------------------
{
    return lhs.multiplyHelper(rhs);
}

template<typename T>
//------------------------------------------------------------------------------
BasicMatrix<T, 4, 4> operator *(const BasicMatrix<T, 4, 4>& lhs,
                                const BasicMatrix<T, 4, 4>& rhs)
//------------------------------------------------------------------------------
{
    return lhs.multiplyHelper(rhs);
}

template<typename T>
//------------------------------------------------------------------------------
BasicMatrix<T, 3, 4> operator *(const BasicMatrix<T, 3, 4>& lhs,
                                const BasicMatrix<T, 3, 4>& rhs)
//------------------------------------------------------------------------------
{
    BasicMatrix<T, 3, 4> ret;
    
    ret(0,0) = lhs(0,0)*rhs(0,0) + lhs(0,1)*rhs(1,0) + lhs(0,2)*rhs(2,0);
    ret(0,1) = lhs(0,0)*rhs(0,1) + lhs(0,1)*rhs(1,1) + lhs(0,2)*rhs(2,1);
    ret(0,2) = lhs(0,0)*rhs(0,2) + lhs(0,1)*rhs(1,2) + lhs(0,2)*rhs(2,2);
    ret(0,3) = lhs(0,0)*rhs(0,3) + lhs(0,1)*rhs(1,3) + lhs(0,2)*rhs(2,3) + lhs(0,3);

    ret(1,0) = lhs(1,0)*rhs(0,0) + lhs(1,1)*rhs(1,0) + lhs(1,2)*rhs(2,0);
    ret(1,1) = lhs(1,0)*rhs(0,1) + lhs(1,1)*rhs(1,1) + lhs(1,2)*rhs(2,1);
    ret(1,2) = lhs(1,0)*rhs(0,2) + lhs(1,1)*rhs(1,2) + lhs(1,2)*rhs(2,2);
    ret(1,3) = lhs(1,0)*rhs(0,3) + lhs(1,1)*rhs(1,3) + lhs(1,2)*rhs(2,3) + lhs(1,3);

    ret(2,0) = lhs(2,0)*rhs(0,0) + lhs(2,1)*rhs(1,0) + lhs(2,2)*rhs(2,0);
    ret(2,1) = lhs(2,0)*rhs(0,1) + lhs(2,1)*rhs(1,1) + lhs(2,2)*rhs(2,1);
    ret(2,2) = lhs(2,0)*rhs(0,2) + lhs(2,1)*rhs(1,2) + lhs(2,2)*rhs(2,2);
    ret(2,3) = lhs(2,0)*rhs(0,3) + lhs(2,1)*rhs(1,3) + lhs(2,2)*rhs(2,3) + lhs(2,3);

    return ret;
}

template<typename T, UInt R, UInt C>
//------------------------------------------------------------------------------
std::ostream& operator <<(std::ostream& os, const BasicMatrix<T, R, C>& m)
//------------------------------------------------------------------------------
{
    os << "(\n";

    BasicVector<T, R> column;
    for (UInt i = 0; i<C; ++i)
    {
        m.getColumn(i, column);
        os << column << '\n';
    }

    os << ')' << std::endl;
    return os;
}

template<typename T>
//------------------------------------------------------------------------------
void makeProjection(BasicMatrix<T, 4, 4>& m,
                    const T& nearPlane, const T& farPlane,
                    const T& fovHorizontal, const T& aspectRatio)
//------------------------------------------------------------------------------
{
    GE_ASSERT(nearPlane>EPSILON);
    GE_ASSERT(farPlane>nearPlane);
    GE_ASSERT(EPSILON<fovHorizontal && fovHorizontal<PI);
    GE_ASSERT(0<aspectRatio);

    T q = farPlane/(farPlane - nearPlane);
    T c = cot(fovHorizontal*T(0.5));

    m = BasicMatrix<T, 4, 4>::IDENTITY();
    
    m(0,0) = c;
    m(1,1) = aspectRatio*c;
    m(2,2) = q;
    m(3,3) = 0;
    m(2,3) = -q*nearPlane;
    m(3,2) = 1;
}


template<typename T>
//------------------------------------------------------------------------------
void makeProjection(BasicMatrix<T, 4, 4>& m,
                    const T& nearPlane,
                    const T& fovHorizontal, const T& aspectRatio)
//------------------------------------------------------------------------------
{
    GE_ASSERT(nearPlane>EPSILON);
    GE_ASSERT(EPSILON<fovHorizontal && fovHorizontal<PI);
    GE_ASSERT(0<aspectRatio);

    T c = cot(fovHorizontal*T(0.5));

    m = BasicMatrix<T, 4, 4>::IDENTITY();
    
    m(0,0) = c;
    m(1,1) = aspectRatio*c;
    m(2,2) = 1;
    m(3,3) = 0;
    m(2,3) = -nearPlane;
    m(3,2) = 1;
}


template<typename T, UInt R, UInt C>
//------------------------------------------------------------------------------
bool makeLookAt(BasicMatrix<T, R, C>& m, const BasicVector<T, 3>& from,
                const BasicVector<T, 3>& at, const BasicVector<T, 3>& up)
//------------------------------------------------------------------------------
{
    GE_CTASSERT(C == 4 && (R == 3 || R == 4));

    bool ret(true);

    // Check that <up> is not nearly 0
    GE_ASSERT_MSG(!up.isAlmost(BasicVector<T, 3>::ORIGIN()), "Up vector nearly 0");
    if (up.isAlmost(BasicVector<T, 3>::ORIGIN()))
    {
        ret = false;
    }
    
    // Check that <from> and <at> have reasonable distance
    BasicVector<T, 3> dirVec(at-from);
    if (ret)
    {
        GE_ASSERT_MSG(!dirVec.isAlmost(BasicVector<T, 3>::ORIGIN()),
            "Distance between eye and lookat point too close");

        if (dirVec.isAlmost(BasicVector<T, 3>::ORIGIN()))
        {
            ret = false;
        }
    }

    // Both checks were OK
    if (ret)
    {
        m = BasicMatrix<T, R, C>::IDENTITY();

        dirVec.normalize();

        BasicVector<T, 3> rightVec(cross(up, dirVec));
        rightVec.normalize();

        BasicVector<T, 3> upVec(cross(dirVec, rightVec));

        m(0,0) = rightVec[0];
        m(1,0) = rightVec[1];
        m(2,0) = rightVec[2];
        
        m(0,1) = upVec[0];
        m(1,1) = upVec[1];
        m(2,1) = upVec[2];
        
        m(0,2) = dirVec[0];
        m(1,2) = dirVec[1];
        m(2,2) = dirVec[2];
        
        m(0,3) = from[0];
        m(1,3) = from[1];
        m(2,3) = from[2];
    }

    return ret;
}

template<typename T, UInt R, UInt C>
//------------------------------------------------------------------------------
bool makeView(BasicMatrix<T, R, C>& m, const BasicVector<T, 3>& from,
              const BasicVector<T, 3>& at, const BasicVector<T, 3>& up)
//------------------------------------------------------------------------------
{
    GE_CTASSERT(C == 4 && (R == 3 || R == 4));
    
    bool ret(true);

    // Check that <up> is not nearly 0
    GE_ASSERT_MSG(!up.isAlmost(BasicVector<T, 3>::ORIGIN()), "Up vector nearly 0");
    if (up.isAlmost(BasicVector<T, 3>::ORIGIN()))
    {
        ret = false;
    }
    
    // Check that <from> and <at> have reasonable distance
    BasicVector<T, 3> dirVec(at-from);
    if (ret)
    {
        GE_ASSERT_MSG(!dirVec.isAlmost(BasicVector<T, 3>::ORIGIN()),
            "Distance between eye and lookat point too close");
        if (dirVec.isAlmost(BasicVector<T, 3>::ORIGIN()))
        {
            ret = false;
        }
    }

    // Both checks were OK
    if (ret)
    {
        m = BasicMatrix<T, R, C>::IDENTITY();

        dirVec.normalize();
        BasicVector<T, 3> rightVec(cross(up, dirVec));
        rightVec.normalize();
        BasicVector<T, 3> upVec(cross(dirVec, rightVec));

        // Inverse of view matrix transforms x, y, z-axes
        // into rightVec, upVec, dirVec, so transpose it to get
        // inverse and add translation into origin.
        
        // 1st row = rightVec
        m(0, 0) = rightVec[0];
        m(0, 1) = rightVec[1];
        m(0, 2) = rightVec[2];

        // 2nd row = upVec
        m(1, 0) = upVec[0];
        m(1, 1) = upVec[1];
        m(1, 2) = upVec[2];

        // 3rd row = dirVec
        m(2, 0) = dirVec[0];
        m(2, 1) = dirVec[1];
        m(2, 2) = dirVec[2];

        // Last column = translation of from to origin (dot product term
        // because first translation is done, then rotation)
        m(0, 3) = -rightVec*from;
        m(1, 3) = -upVec*from;
        m(2, 3) = -dirVec*from;
    }

    return ret;
}


template<class MATRIX_OUT, class MATRIX_IN>
//------------------------------------------------------------------------------
void makeBillboardHelper(MATRIX_OUT& out, const MATRIX_IN& view)
//------------------------------------------------------------------------------
{
    out = MATRIX_OUT::IDENTITY();
    for (UInt r = 0; r<3; ++r)
    {
        for (UInt c = 0; c<3; ++c)
        {
            out(c,r) = view(r,c);
        }
    }
}


template<typename T, UInt R, UInt C>
//------------------------------------------------------------------------------
void makeBillboard(BasicMatrix<T, R, C>& out, const BasicMatrix<T, 4, 4>& view)
//------------------------------------------------------------------------------
{
    GE_CTASSERT((R==3 && C==3) || (R==3 && C==4) || (R==4 && C==4));
    makeBillboardHelper(out,view);
}


template<typename T, UInt R, UInt C>
//------------------------------------------------------------------------------
void makeBillboard(BasicMatrix<T, R, C>& out, const BasicMatrix<T, 3, 4>& view)
//------------------------------------------------------------------------------
{
    GE_CTASSERT((R==3 && C==3) || (R==3 && C==4) || (R==4 && C==4));
    makeBillboardHelper(out, view);
}


template<typename T, UInt R, UInt C>
//------------------------------------------------------------------------------
void makeXRotation(BasicMatrix<T, R, C>& out, const T& angle)
//------------------------------------------------------------------------------
{
    GE_CTASSERT((R==3 && C==3) || (R==3 && C==4) || (R==4 && C==4));
    
    out = BasicMatrix<T, R, C>::IDENTITY();

    T c = cos(angle);
    T s = sin(angle);

    out(1, 1) = c;
    out(1, 2) = -s;
    out(2, 1) = s;
    out(2, 2) = c;
}


template<typename T, UInt R, UInt C>
//------------------------------------------------------------------------------
void makeYRotation(BasicMatrix<T, R, C>& out, const T& angle)
//------------------------------------------------------------------------------
{
    GE_CTASSERT((R==3 && C==3) || (R==3 && C==4) || (R==4 && C==4));

    out = BasicMatrix<T, R, C>::IDENTITY();
    
    T c(cos(angle));
    T s(sin(angle));

    out(0, 0) = c;
    out(0, 2) = s;
    out(2, 0) = -s;
    out(2, 2) = c;
}


template<typename T, UInt R, UInt C>
//------------------------------------------------------------------------------
void makeZRotation(BasicMatrix<T, R, C>& out, const T& angle)
//------------------------------------------------------------------------------
{
    GE_CTASSERT((R==3 && C==3) || (R==3 && C==4) || (R==4 && C==4));

    out = BasicMatrix<T, R, C>::IDENTITY();
    
    T c(cos(angle));
    T s(sin(angle));

    out(0, 0) = c;
    out(0, 1) = -s;
    out(1, 0) = s;
    out(1, 1) = c;
}


template<typename T, UInt R, UInt C>
//------------------------------------------------------------------------------
void makeRotation(BasicMatrix<T, R, C>& out,
                  const T& head, const T& pitch, const T& roll)
//------------------------------------------------------------------------------
{
    GE_CTASSERT((R==3 && C==3) || (R==3 && C==4) || (R==4 && C==4));

    // The matrix is obtained by multiplying R_z(roll)*R_x(pitch)*R_y(head)
    out = BasicMatrix<T, R, C>::IDENTITY();
    Float sR(sin(roll)), cR(cos(roll));
    Float sH(sin(head)), cH(cos(head));
    Float sP(sin(pitch)), cP(cos(pitch));
    out(0,0) = cR*cH - sR*sP*sH;
    out(0,1) = -sR*cP;
    out(0,2) = cR*sH + sR*sP*cH;
    out(1,0) = sR*cH + cR*sP*sH;
    out(1,1) = cR*cP;
    out(1,2) = sR*sH - cR*sP*cH;
    out(2,0) = -cP*sH;
    out(2,1) = sP;
    out(2,2) = cP*cH;
}

template<typename T, UInt R, UInt C>
//------------------------------------------------------------------------------
void makeRotation(BasicMatrix<T, R, C>& out,
                  const BasicVector<T, 3>& axis, const T& angle)
//------------------------------------------------------------------------------
{
    BasicQuaternion<T> q;
    q.makeRotation(axis, angle);
    q.convertTo(out);
}

template<typename T, UInt R, UInt C>
//------------------------------------------------------------------------------
bool isAffine(const BasicMatrix<T, R, C>& m)
//------------------------------------------------------------------------------
{
    GE_CTASSERT((R==3 && C==3) || (R==3 && C==4) || (R==4 && C==4));

    const Float TOLERANCE = 0.001f;

    return
    isAlmost(square(m(0,0)) + square(m(1,0)) + square(m(2,0)), T(1), TOLERANCE) &&
    isAlmost(square(m(0,1)) + square(m(1,1)) + square(m(2,1)), T(1), TOLERANCE) &&
    isAlmost(square(m(0,2)) + square(m(1,2)) + square(m(2,2)), T(1), TOLERANCE) &&
    isAlmost(m(0,0)*m(0,1) + m(1,0)*m(1,1) + m(2,0)*m(2,1), T(0), TOLERANCE) &&
    isAlmost(m(0,0)*m(0,2) + m(1,0)*m(1,2) + m(2,0)*m(2,2), T(0), TOLERANCE) &&
    isAlmost(m(0,1)*m(0,2) + m(1,1)*m(1,2) + m(2,1)*m(2,2), T(0), TOLERANCE);
}


template<typename T, UInt R, UInt C>
//------------------------------------------------------------------------------
void invertAffine(BasicMatrix<T, R, C>& m)
//------------------------------------------------------------------------------
{
    GE_CTASSERT((R==3 && C==3) || (R==3 && C==4) || (R==4 && C==4));
    GE_ASSERT_MSG(isAffine(m), "Tried to invert non affine matrix.");
    
    // Transpose rotation
    std::swap(m(1,0), m(0,1));
    std::swap(m(2,0), m(0,2));
    std::swap(m(1,2), m(2,1));
    
    // Rotate inverted translation
    if (C==4)
    {
        Vector3 t;
        t[0] = m(0,3);
        t[1] = m(1,3);
        t[2] = m(2,3);
        m(0,3) = -m(0,0)*t[0] - m(0,1)*t[1] - m(0,2)*t[2];
        m(1,3) = -m(1,0)*t[0] - m(1,1)*t[1] - m(1,2)*t[2];
        m(2,3) = -m(2,0)*t[0] - m(2,1)*t[1] - m(2,2)*t[2];
    }
}

} // namespace Ge