#include "Precompiled.h"

//------------------------------------------------------------------------------
// (c) 05-2001 Gottfried Chen
//------------------------------------------------------------------------------

#include <Math/Perlin.h>
#include <Math/Interpolation.h>

#include <math.h>

namespace Ge
{

//------------------------------------------------------------------------------
Perlin::Perlin(UInt iterations, Float dampening) :
//------------------------------------------------------------------------------
mIterations(iterations),
mDampening(dampening)
{
}

//------------------------------------------------------------------------------
Float Perlin::noise(Float x, Float y) const
//------------------------------------------------------------------------------
{
    UInt frequency(0);
    double amplitude(0);
    Float ret(0);
    for (UInt i = 0; i<mIterations; i++)
    {
        frequency = pow(2, i);
        amplitude = pow(mDampening, i);
        ret += amplitude*interpolatedRandom(x*frequency, y*frequency);
    }
    return ret;
}

    
//------------------------------------------------------------------------------
Float Perlin::interpolatedRandom(Float x, Float y) const
//------------------------------------------------------------------------------
{
    Int xInt(x), yInt(y);
    Float xFrac(x-xInt), yFrac(y-yInt);
    
    Float r(random(xInt, yInt));
    Float rX(random(xInt+1, yInt));
    Float rY(random(xInt, yInt+1));
    Float rXY(random(xInt+1, yInt+1));

    Float iX(lerp(r, rX, xFrac));
    Float iXY(lerp(rY, rXY, xFrac));

    return lerp(iX, iXY, yFrac);
}

//------------------------------------------------------------------------------
Float Perlin::random(Int x, Int y) const
//------------------------------------------------------------------------------
{
    Int n = x + y * 57;
    n = n<<13 ^ n;
    return ( 1.0f - ( (n * (n * n * 15731 + 789221) + 1376312589) & 0x7fffffff) /
        1073741824.0);    
}

} // namespace Ge