#include "Precompiled.h"

//------------------------------------------------------------------------------
// (c) 05-2001 Gottfried Chen
//------------------------------------------------------------------------------

#include <Math/Plane.h>
#include <Math/Sphere.h>
#include <Math/Matrix.h>

namespace Ge
{

//------------------------------------------------------------------------------
Plane::Plane()
//------------------------------------------------------------------------------
{
}
    
//------------------------------------------------------------------------------
Plane::Plane(const Vector3& normal, const Float& offset) :
//------------------------------------------------------------------------------
mNormal(normal),
mOffset(offset)
{
    GE_ASSERT(!normal.isAlmost(Vector3::ORIGIN(), EPSILON));
    mNormal.normalize();
}

//------------------------------------------------------------------------------
Plane::Plane(const Vector3& normal, const Vector3& planePoint)
//------------------------------------------------------------------------------
{
    set(normal, planePoint);
}

//------------------------------------------------------------------------------
void Plane::set(const Vector3& normal, const Vector3& planePoint)
//------------------------------------------------------------------------------
{
    GE_ASSERT(!normal.isAlmost(Vector3::ORIGIN(), EPSILON));
    mNormal = normal;
    mNormal.normalize();
    mOffset = planePoint*mNormal;
}

//------------------------------------------------------------------------------
bool Plane::intersects(const Sphere& sphere) const
//------------------------------------------------------------------------------
{
    return getDistance(sphere.getCenter()) <= sphere.getRadius();
}

//------------------------------------------------------------------------------
Float Plane::getDistance(const Vector3& vector) const
//------------------------------------------------------------------------------
{
    return fabs((vector*mNormal)-mOffset);
}


//------------------------------------------------------------------------------
const Vector3& Plane::getNormal() const
//------------------------------------------------------------------------------
{
    return mNormal;
}

//------------------------------------------------------------------------------
const Float& Plane::getOffset() const
//------------------------------------------------------------------------------
{
    return mOffset;
}

} // namespace Ge

