//------------------------------------------------------------------------------
// (c) 05-2001 Gottfried Chen
//------------------------------------------------------------------------------

#ifndef MATH_PLANE_H
#define MATH_PLANE_H

#include <Math/Vector.h>
#include <Math/Matrix.h>

namespace Ge
{

class Sphere;

// ### Could make this into a template class HyperPlane, but it's not really
// necessary
//
// The normal vector of a plane will always be normalized, when set.
// The offset of the plane is in direction of the plane normal.
// I.e.:
//
// ^
// |<-     offset     ->|
// |                    |normal  
// +---->               |-->    
//                      |
//                    Plane
//
// So the plane equation is normal*v - offset = 0
class Plane
{
public:
    // No initialisation is done!
    Plane();

    // <normal> must not be the origin
    explicit
    Plane(const Vector3& normal, const Float& offset);

    Plane(const Vector3& normal, const Vector3& planePoint);

    bool intersects(const Sphere&) const;    

    Float getDistance(const Vector3&) const;
    
    void set(const Vector3& normal, const Vector3& planePoint);
    
    const Vector3& getNormal() const;

    const Float& getOffset() const;

    template<typename T, UInt R, UInt C>
    Plane transform(const BasicMatrix<T, R, C>& transform) const
    {
        GE_CTASSERT(C == 4);
        Plane ret;
        Matrix3x3 rotation;
        transform.getSubMatrix(rotation);
        ret.mNormal = rotation*mNormal;
        Vector3 translation(transform(0,3), transform(1,3), transform(2,3));

        // Basically this is (transform*planePoint)*transformedNormal.
        ret.mOffset = mOffset + translation*ret.mNormal;
        return ret;
    }

private:
    Vector3 mNormal;
    Float mOffset;
};

} // namespace Ge

#endif // MATH_PLANE_H
