#include "Precompiled.h"

//------------------------------------------------------------------------------
// (c) 05-2001 Gottfried Chen
//------------------------------------------------------------------------------

#include <Math/Sphere.h>

namespace Ge
{

//------------------------------------------------------------------------------
Sphere::Sphere(const Vector3& center, const Float& radius) :
//------------------------------------------------------------------------------
mCenter(center),
mRadius(radius)
{
}

//------------------------------------------------------------------------------
bool Sphere::intersects(const Sphere& sphere) const
//------------------------------------------------------------------------------
{
    Float radiusSum(getRadius() + sphere.getRadius());
    return (getCenter()-sphere.getCenter()).lengthSquare() <= square(radiusSum);
}

//------------------------------------------------------------------------------
bool Sphere::contains(const Vector3& point) const
//------------------------------------------------------------------------------
{
    return (point - getCenter()).lengthSquare() <= square(getRadius());
}

//------------------------------------------------------------------------------
bool Sphere::contains(const Sphere& sphere) const
//------------------------------------------------------------------------------
{
    Float r((getRadius()-sphere.getRadius()));
    return r >= 0 ?
           (getCenter()-sphere.getCenter()).lengthSquare() <= r*r :
           false;
}

//------------------------------------------------------------------------------
const Vector3& Sphere::getCenter() const
//------------------------------------------------------------------------------
{
    return mCenter;
}

//------------------------------------------------------------------------------
void Sphere::setCenter(const Vector3& center)
//------------------------------------------------------------------------------
{
    mCenter = center;
}

//------------------------------------------------------------------------------
const Float& Sphere::getRadius() const
//------------------------------------------------------------------------------
{
    return mRadius;
}

//------------------------------------------------------------------------------
void Sphere::setRadius(const Float& radius)
//------------------------------------------------------------------------------
{
    mRadius = radius;
}

//------------------------------------------------------------------------------
void Sphere::merge(const Sphere& sphere)
//------------------------------------------------------------------------------
{
    if (!sphere.contains(*this))
    {
        Vector3 dir(sphere.getCenter() - getCenter());
        Float dirLength(dir.length());
        Float distance(dirLength + getRadius() + sphere.getRadius());
        Float centerOffset(distance/2 - getRadius());
        if (centerOffset > 0)
        {
            setRadius(distance/2 + EPSILON);
            dir *= 1.0f/dirLength;
            setCenter(getCenter() + centerOffset*dir);
        }
    }
    else
        *this = sphere;
}

//------------------------------------------------------------------------------
void Sphere::merge(const Vector3& vertices, UInt vertexCount)
//------------------------------------------------------------------------------
{
    GE_ASSERT(vertexCount > 0);
    const Vector3* v = &vertices;
    UInt i(0);
    
    // Center is mean of all vertices
    Vector3 center(vertices);
    for (i = 1; i<vertexCount; i++)
        center += v[i];
    setCenter(center*(1.0f/vertexCount));

    // Radius is biggest distance
    Float radiusSquare((getCenter()-vertices).lengthSquare());
    Float temp(0);
    for (i = 1; i<vertexCount; i++)
    {
        temp = (getCenter()-v[i]).lengthSquare();
        if (temp > radiusSquare)
            radiusSquare = temp;
    }

    setRadius(sqrt(radiusSquare) + 0.001f);
}

} // namespace Ge
