//------------------------------------------------------------------------------
// (c) Mai-2002 Gottfried Chen
//------------------------------------------------------------------------------

#include "QuaternionTest.h"

#include <Math/Quaternion.h>
#include <Math/Matrix.h>

namespace
{

using namespace Ge;

//------------------------------------------------------------------------------
void fill(Matrix3x3& m,
          Float _00, Float _01, Float _02,
          Float _10, Float _11, Float _12,
          Float _20, Float _21, Float _22)
//------------------------------------------------------------------------------
{
    m(0,0) = _00;
    m(0,1) = _01;
    m(0,2) = _02;
    m(1,0) = _10;
    m(1,1) = _11;
    m(1,2) = _12;
    m(2,0) = _20;
    m(2,1) = _21;
    m(2,2) = _22;
}

} // namespace

namespace Ge
{

//------------------------------------------------------------------------------
void QuaternionTest::run() const
//------------------------------------------------------------------------------
{
    // Test axis/angle rotation
    Vector3 yAxis(0,1,0);
    Quaternion q;
    q.makeRotation(yAxis, PI_HALF);
    Matrix3x3 m;
    q.convertTo(m);
    Matrix3x3 result;
    fill(result, 0,0,1,
                 0,1,0,
                 -1,0,0);
    GE_ASSERT(m.isAlmost(result, EPSILON));

    // Test Euler angles
    q.makeRotation(2,1,4);
    makeRotation(result, 2.0f,1.0f,4.0f);
    q.convertTo(m);
    GE_ASSERT(m.isAlmost(result, EPSILON));

    // Test convertFrom
    q.makeRotation(1,2,3);
    q.convertTo(m);
    Quaternion converted;
    converted.convertFrom(m);
    GE_ASSERT(q.isAlmost(converted, EPSILON) || q.isAlmost(-converted, EPSILON));
}

} // namespace Ge
