//------------------------------------------------------------------------------
// (c) Apr-2002 Gottfried Chen
//------------------------------------------------------------------------------

#include "VectorTest.h"

#include <Math/Vector.h>



namespace Ge
{

//------------------------------------------------------------------------------
void VectorTest::run() const
//------------------------------------------------------------------------------
{
    const Float TEST_TOLERANCE = 0.0001f;

    // Test construction

    Vector3 v(1,2,3);
    GE_ASSERT(v[0] == 1);
    GE_ASSERT(v[1] == 2);
    GE_ASSERT(v[2] == 3);

    // Test comparison

    GE_ASSERT(v == Vector3(1,2,3));
    GE_ASSERT(v != Vector3(2,2,3));

    GE_ASSERT(Vector3(0,0,0) == Vector3::ORIGIN());

    set(v,3.0f,4.0f,5.0f);
    GE_ASSERT(v == Vector3(3,4,5));
    
    Vector3 assigned;
    assigned = v;
    GE_ASSERT(assigned == v);
    Vector3 copy(v);
    GE_ASSERT(copy == v);

    // Test operators
  
    set(v,1.0f,2.0f,3.0f);
    v += assigned;
    GE_ASSERT(v == Vector3(4,6,8));
    v -= assigned;
    GE_ASSERT(v == Vector3(1,2,3));    
    v *= 2;
    GE_ASSERT(v == Vector3(2,4,6));
    assigned = -v;
    GE_ASSERT(assigned == Vector3(-2,-4,-6));
    set(v,1.0f,2.0f,3.0f);
    Vector3 v2(3,5,4);
    GE_ASSERT(25 == v*v2);
    GE_ASSERT(Vector3(4,7,7) == v+v2);
    GE_ASSERT(Vector3(-2,-3,-1) == v-v2);
    GE_ASSERT(Vector3(2,4,6) == v*2.0f);
    GE_ASSERT(Vector3(2,4,6) == 2.0f*v);

    // Test utility
  
    set(v,1.0f,2.0f,3.0f);
    GE_ASSERT(v.lengthSquare() == 14);
    GE_ASSERT(isAlmost(v.length(), 3.74165f, TEST_TOLERANCE));

    set(v,1.0f,2.0f,-3.0f);
    GE_ASSERT(v.maxNorm() == 3);
    GE_ASSERT(v.sumNorm() == 6);

    set(v,1.0f,2.0f,3.0f);
    v.normalize();
    GE_ASSERT(v.isAlmost(Vector3(0.267262f,0.534524f,0.801785f), TEST_TOLERANCE));

    set(v,1.0f,10.0f,0.0f);
    Vector3 target(2,0,0);
    Vector3 projection = v.projectOn(target);
    GE_ASSERT(projection == Vector3(1,0,0));

    set(v,4.0f,2.0f,3.0f);
    set(v2,9.0f,7.0f,5.0f);
    GE_ASSERT(cross(v, v2) == Vector3(-11,7,10));

    set(v,1.0f,1.0f,0.0f);
    set(v2,1.0f,0.0f,0.0f);
    GE_ASSERT(isAlmost(v.getAngleTo(v2), PI_QUARTER));

    // Vector2 set

    Vector2 v2dim;
    set(v2dim,2.0f,3.0f);
    GE_ASSERT(v2dim == Vector2(2,3));
}

} // namespace Ge
