//------------------------------------------------------------------------------
// (c) 03-2001 Gottfried Chen
//------------------------------------------------------------------------------

#ifndef MATH_TOOLS_H
#define MATH_TOOLS_H

#include <System/Types.h>

#include <stdlib.h>

// ### Maybe rewrite stuff to template functions later. Right now
// I think control over datatypes is more important than
// generality

// Conversion from double to Float -> loss of data
#pragma warning(disable: 4244)

namespace Ge
{
// A left hand coordinate is used (z points into the screen).
// Vectors are multiplied from the right onto matrices.

//------------------------------------------------------------------------------
// Constants
//------------------------------------------------------------------------------
extern const Float PI;
extern const Float PI_HALF;
extern const Float PI_QUARTER;
extern const Float TWO_PI;
extern const Float EPSILON;
extern const Float SQRT_TWO;
extern const Float INV_SQRT_TWO;


//------------------------------------------------------------------------------
// Trigonometry
//------------------------------------------------------------------------------
Float degToRad(Float);

Float radToDeg(Float);

Float sin(const Float&);

Float cos(const Float&);

Float tan(const Float&);

Float cot(const Float&);

Float acos(const Float&);

//------------------------------------------------------------------------------
// Misc
//------------------------------------------------------------------------------
Float sqrt(const Float&);

Float square(const Float&);

// Calculates a fast inverse square root. (Taken from Q3).
Float invSqrt(Float); 

Float fabs(const Float&);

// Returns the sign (= 0, 1 or -1) of a number.
Float sign(const Float&);

// Returns (-1)^n
Int neg1Pow(Int n); 

// Only use this for b = 2^n! Returns a%b
Int mod(Int a, UInt b);

// Calculates the determinante of a 3x3 matrix.
Float det(const Float& m00, const Float& m01, const Float& m02,
          const Float& m10, const Float& m11, const Float& m12, 
          const Float& m20, const Float& m21, const Float& m22);

// Returns true if fabs(<lhs> - <rhs>) < <tolerance>.
bool isAlmost(const Float& lhs, const Float& rhs, Float tolerance = EPSILON);

// Clamp <value> to the interval [<low>, <up>].
template<typename T>
void clamp(T& value, const T& low, const T& up);

// Wraps <value> to the interval [<low>, <up>] similar to texture wrapping.
template<typename T>
void wrap(T& value, const T& lower, const T& upper);

} // namespace Ge

#include <Math/Tools.inl>
#endif // MATH_TOOLS_H