//------------------------------------------------------------------------------
// (c) 03-2001 Gottfried Chen
//------------------------------------------------------------------------------

#include <math.h>
#include <stdlib.h>

namespace Ge
{

inline
//------------------------------------------------------------------------------
Float degToRad(Float angle)
//------------------------------------------------------------------------------
{
    static Float piBy180 = PI/180.0f;
    return piBy180*angle;
}

inline
//------------------------------------------------------------------------------
Float radToDeg(Float angle)
//------------------------------------------------------------------------------
{
    static Float _180ByPi = 180.0f/PI;
    return _180ByPi*angle;
}

inline
//------------------------------------------------------------------------------
Float sin(const Float& value)
//------------------------------------------------------------------------------
{
    return Float(::sin(double(value)));
}


inline
//------------------------------------------------------------------------------
Float cos(const Float& value)
//------------------------------------------------------------------------------
{
    return Float(::cos(double(value)));
}


inline
//------------------------------------------------------------------------------
Float tan(const Float& value)
//------------------------------------------------------------------------------
{
    return Float(::tan(double(value)));
}


inline
//------------------------------------------------------------------------------
Float cot(const Float& value)
//------------------------------------------------------------------------------
{
    return Float(cos(value)/sin(value));
}


inline
//------------------------------------------------------------------------------
Float acos(const Float& value)
//------------------------------------------------------------------------------
{
    return Float(::acos(double(value)));
}

inline
//------------------------------------------------------------------------------
Float sqrt(const Float& value)
//------------------------------------------------------------------------------
{
    return Float(::sqrt(double(value)));
}

inline
//------------------------------------------------------------------------------
Float square(const Float& value)
//------------------------------------------------------------------------------
{
    return value*value;
}

inline
//------------------------------------------------------------------------------
Float invSqrt(Float value)
//------------------------------------------------------------------------------
{
    // This is a fast inverse square root implementation. For an explanation
    // look at http://www.magic-software.com/3DGEDInvSqrt.html
    Float vhalf = 0.5f*value;
    int i = *(int*)&value;
    i = 0x5f3759df - (i >> 1);  // This line hides a LOT of math!
    value = *(Float*)&i;
    value = value*(1.5f - vhalf*square(value));  // repeat this statement for a better approximation
    return value;
}

inline
//------------------------------------------------------------------------------
Float fabs(const Float& number)
//------------------------------------------------------------------------------
{
    return Float(::fabs(double(number)));
}

inline
//------------------------------------------------------------------------------
Float sign(const Float& number)
//------------------------------------------------------------------------------
{
    if (number == 0)
        return 0;
    else
        return number > 0 ? 1 : -1;
}

inline
//------------------------------------------------------------------------------
Float det(const Float& m00, const Float& m01, const Float& m02,
          const Float& m10, const Float& m11, const Float& m12, 
          const Float& m20, const Float& m21, const Float& m22)
//------------------------------------------------------------------------------
{
    return m00*m11*m22 + m01*m12*m20 + m02*m10*m21 -
           m20*m11*m02 - m21*m12*m00 - m22*m10*m01;
}

inline
//------------------------------------------------------------------------------
bool isAlmost(const Float& lhs, const Float& rhs, Float tolerance)
//------------------------------------------------------------------------------
{
    return fabs(lhs-rhs) < tolerance;
}

inline
//------------------------------------------------------------------------------
Int neg1Pow(Int n)
//------------------------------------------------------------------------------
{
    static Int pow[2] = { 1, -1 };
    return pow[n&1];
}

inline
//------------------------------------------------------------------------------
Int mod(Int a, UInt b)
//------------------------------------------------------------------------------
{
    return a&(b-1);
}

template<typename T> inline
//------------------------------------------------------------------------------
void clamp(T& value, const T& low, const T& up)
//------------------------------------------------------------------------------
{
    GE_ASSERT(low<up);
    value = minimum(maximum(value, low), up);
}

template<typename T> inline
//------------------------------------------------------------------------------
void wrap(T& value, const T& low, const T& up)
//------------------------------------------------------------------------------
{
    GE_ASSERT(low<up);
    if (value > up)
    {
        value = value - UInt((value - low)/(up-low))*(up-low);
    }
    else if (value < low)
    {
        value *= -1;
        wrap(value, -low, -up);
        value *= -1;
    }
}

} // namespace Ge