#ifndef MATH_UNIT_ROOT_H
#define MATH_UNIT_ROOT_H
#pragma once

//------------------------------------------------------------------------------
// (c) 01-2002 Gottfried Chen
//------------------------------------------------------------------------------


#include "Math.h"
#include <complex>

namespace Ge
{

// Represents complex unit roots. The template paramater T denotes the type
// used for the underlying complex numbers (i.e. Float or Double).
template<class T>
class UnitRoot
{
public:
    // Generates the powers of the <n>-th unit root.
    UnitRoot(unsigned int n);

    ~UnitRoot();

    // Gets the power <p> of the unit root. <p> must be in [0, n].
    // All other powers are periodic. If <p> is not within the valid range
    // the results are undefined.
    const std::complex<T>& pow(unsigned int p) const;
    
    // Returns pow(1).
    operator const std::complex<T>&() const;

private:
    std::complex<T>* mPowers;
    unsigned int mN;
};

} // namespace Ge

#include "UnitRoot.inl"
#endif // MATH_UNIT_ROOT_H