//------------------------------------------------------------------------------
// (c) 10-2001 Gottfried Chen
//------------------------------------------------------------------------------

#ifndef SYSTEM_BIT_MANIPULATION_H
#define SYSTEM_BIT_MANIPULATION_H


#include <System/Types.h>

// Just some helper functions for bit manipulation. I'm really bad at reading
// all these bit shifting and &, |, ^ stuff, so I use these helper functions
// to make code more readable.
namespace Ge
{

namespace Bit
{
    // Bit positions are from the left (i.e. 00000001 would have bit 0 set to 1)
    template<class T> inline
    void setBit(T& value, UInt position, bool on);

    template<class T> inline
    bool getBit(const T& value, UInt position);

    // If you reflect 0x11010001 you get 0x10001011.
    template<class T> inline
    T reflect(const T& value);

    // Byte positions are from the left (i.e. 0x00ff would have byte 0 set to 0xff)
    template<class T> inline
    void setByte(T& value, UInt position, UInt8 byte);

    template<class T> inline
    UInt8 getByte(const T& value, UInt position);

    // Reflect bytewise. I.e. 0xabcd becomes 0xcdab.
    template<class T> inline
    T reflectByte(const T& value);
} // namespace Bit

} // namespace Ge

#include "BitManipulation.inl"
#endif // SYSTEM_BIT_MANIPULATION_H