//------------------------------------------------------------------------------
// (c) 10-2001 Gottfried Chen
//------------------------------------------------------------------------------

#include "Base.h"

namespace Ge
{

namespace Bit
{

// Bit positions are from the left (i.e. 00000001 would have bit 0 set to 1)
template<class T> inline
//------------------------------------------------------------------------------
void setBit(T& value, UInt position, bool on)
//------------------------------------------------------------------------------
{
    GE_ASSERT(position < sizeof(T)*8);
    value = on ? (value | (1<<position)) : (value & ~(1<<position));
}

template<class T> inline
//------------------------------------------------------------------------------
bool getBit(const T& value, UInt position)
//------------------------------------------------------------------------------
{
    GE_ASSERT(position < sizeof(T)*8);
    return (value & (1<<position)) != 0;
}

template<class T> inline
//------------------------------------------------------------------------------
T reflect(const T& value)
//------------------------------------------------------------------------------
{
    T reflected(0);
    const UInt len(sizeof(T)*8);
    for (UInt i = 0; i<len; ++i)
        setBit(reflected, len-1-i, getBit(value, i));

    return reflected;
}

// Byte positions are from the left (i.e. 0x00ff would have byte 0 set to 0xff)
template<class T> inline
//------------------------------------------------------------------------------
void setByte(T& value, UInt position, UInt8 byte)
//------------------------------------------------------------------------------
{
    GE_ASSERT(position < sizeof(T));
    value = (value & ~(0xff<<(position*8))) | byte<<(position*8);
}

template<class T> inline
//------------------------------------------------------------------------------
UInt8 getByte(const T& value, UInt position)
//------------------------------------------------------------------------------
{
    GE_ASSERT(position < sizeof(T));
    return UInt8((value>>(position*8))&0xff);
}

template<class T> inline
//------------------------------------------------------------------------------
T reflectByte(const T& value)
//------------------------------------------------------------------------------
{
    T reflected(0);
    const UInt len(sizeof(T));
    for (UInt i = 0; i<len; ++i)
        setByte(reflected, len-1-i, getByte(value, i));

    return reflected;
}

} // namespace Bit

} // namespace Ge