//------------------------------------------------------------------------------
// (c) 12-2001 Gottfried Chen
//------------------------------------------------------------------------------

#ifndef BYTE_ORDER_H
#define BYTE_ORDER_H

#include "BitManipulation.h"

#define theByteOrder ByteOrder::instance()

namespace Ge
{

class ByteOrder
{
public:
    enum Type
    {
        BIG_ENDIAN, // Motorola
        LITTLE_ENDIAN, // Intel
        MSB = BIG_ENDIAN,
        LSB = LITTLE_ENDIAN,
        NETWORK = BIG_ENDIAN
    };

    static
    ByteOrder& instance();

    // Transform <value> from <from> to <to> byte order.
    template<class T>
    void convert(T& value, Type from, Type to) const
    {
        if (from != to)
            value = Bit::reflectByte(value);
    }

    // Transform <value> from local to <to>.
    template<class T>
    void to(T& value, Type to) const
    {
        convert(value, local(), to);
    }

    // Transform <value> from <from> to local.
    template<class T>
    void from(T& value, Type from) const
    {
        convert(value, from, local());
    }

    // Get byte order of local machine.
    Type local() const;

private:
    ByteOrder();

    Type mLocal;
};

} // namespace Ge

#endif // BYTE_ORDER_H