//------------------------------------------------------------------------------
// (c) 10-2001 Gottfried Chen
//------------------------------------------------------------------------------

#ifndef SYSTEM_CRC_H
#define SYSTEM_CRC_H

namespace Ge
{

// This class can be configured to calculate all kinds of different Crcs.
// Those I know of have been typedefed below, so normally it should suffice
// to use those.
template<class Reg, bool REFLECTED, Reg POLY, Reg INIT, Reg OUT>
class Crc
{
public:
    typedef Reg Register;

    Crc();

    // Call this subsequently to process further bytes for the Crc
    void process(const char* stream, unsigned int length);
    
    // Returns the checksum
    Register get() const;

private:
    static
    Register getTableEntry(unsigned char index);

    static
    void calculateTable();

    Register mRegister;
    static Register mTable[256];
};

// The check of a Crc is the code when process("123456789", 9) is called.
// It can be used to verify the correct implementation of the Crc.
typedef Crc<unsigned short, true, 0x8005, 0x0000, 0x0000> Crc16; // Check: 0xbb3d
typedef Crc<unsigned short, false, 0x1021, 0xffff, 0x0000> CrcCcitt; // Check: 0x29b1
typedef Crc<unsigned long, true, 0x04c11db7, 0xffffffff, 0xffffffff> Crc32; // Check: 0xcbf43926

} // namespace Ge

#include "Crc.inl"
#endif // SYSTEM_CRC_H