//------------------------------------------------------------------------------
// (c) 10-2001 Gottfried Chen
//------------------------------------------------------------------------------

#include "BitManipulation.h"

namespace Ge
{

template<class Reg, bool REFLECTED, Reg POLY, Reg INIT, Reg OUT>
Reg Crc<Reg, REFLECTED, POLY, INIT, OUT>::mTable[256];



template<class Reg, bool REFLECTED, Reg POLY, Reg INIT, Reg OUT>
//------------------------------------------------------------------------------
Reg Crc<Reg, REFLECTED, POLY, INIT, OUT>::getTableEntry(unsigned char index)
//------------------------------------------------------------------------------
{
    static bool tableCalculated(false);
    if (!tableCalculated)
        calculateTable();

    return mTable[index];
}

template<class Reg, bool REFLECTED, Reg POLY, Reg INIT, Reg OUT>
//------------------------------------------------------------------------------
void Crc<Reg, REFLECTED, POLY, INIT, OUT>::calculateTable()
//------------------------------------------------------------------------------
{ 
    // For all possible values in (0, 255) compute a register number depending
    // on the set bits in the value. This is done by shifting and XORing with
    // the generator polynom. Since reflected algorithms actually shift to the
    // right, the incoming value is reflected before it's fed into the register
    // and then the result is reflected again.
    unsigned char byte;
    Reg entry(0);
    for (unsigned int i = 0; i<256; ++i)
    {
        byte = REFLECTED ? Bit::reflect(unsigned char(i)) : i;
        // Move byte to the leftmost position
        Bit::setByte(entry, sizeof(Reg)-1, byte);
        for (unsigned int j = 0; j<8; ++j)
        {
            // If leftmost register bit is set, then XOR with polynom,
            // else just shift left.
            if (Bit::getBit(entry, sizeof(Reg)*8-1))
                entry = (entry<<1) ^ POLY;
            else
                entry <<= 1;
        }

        mTable[i] = REFLECTED ? Bit::reflect(entry): entry;
        entry = 0;
    }
}


template<class Reg, bool REFLECTED, Reg POLY, Reg INIT, Reg OUT>
//------------------------------------------------------------------------------
Crc<Reg, REFLECTED, POLY, INIT, OUT>::Crc() :
//------------------------------------------------------------------------------
mRegister(INIT)
{
}

template<class Reg, bool REFLECTED, Reg POLY, Reg INIT, Reg OUT>
//------------------------------------------------------------------------------
Reg Crc<Reg, REFLECTED, POLY, INIT, OUT>::get() const
//------------------------------------------------------------------------------
{
    // XOR with OUT to finalize the checksum.
    return mRegister^OUT;
}

template<class Reg, bool REFLECTED, Reg POLY, Reg INIT, Reg OUT>
//------------------------------------------------------------------------------
void Crc<Reg, REFLECTED, POLY, INIT, OUT>::process(const char* stream,
                                                   unsigned int length)
//------------------------------------------------------------------------------
{
    Reg entry;
    for (unsigned int i = 0; i<length; ++i)
    {
        // XOR the current stream byte with the rightmost/leftmost byte of
        // the register and use this as a key into the precalculated
        // polynom XOR-table.
        entry = REFLECTED ? getTableEntry(*stream ^ Bit::getByte(mRegister, 0)) :
                            getTableEntry(*stream ^ Bit::getByte(mRegister, sizeof(Reg)-1));
        // Advance the stream.
        ++stream;
        // Shift the register depending on REFLECTED to right/left.
        mRegister = REFLECTED ? mRegister>>8 : mRegister<<8; 
        // XOR the entry into the register.
        mRegister ^= entry;
    }
}

} // namespace Ge