#include "Precompiled.h"
//------------------------------------------------------------------------------
// (c) 05-2001 Gottfried Chen
//------------------------------------------------------------------------------

#include "FilePath.h"
#include "Base.h"
#include "StringTools.h"

namespace
{

inline
//------------------------------------------------------------------------------
bool isSlash(char c)
//------------------------------------------------------------------------------
{
    return (c == '/') || (c == '\\');
}

//------------------------------------------------------------------------------
void convertSlashes(std::string& str)
//------------------------------------------------------------------------------
{
    using namespace Ge;
    char* newStr = GE_NEW char[str.size()+1];
    memset(newStr, 0 , str.size()+1);
    UInt pos = 0;
    bool firstSlash = true;
    for (UInt i = 0; i<str.size(); i++)
    {
        if (!isSlash(str[i]))
        {
            newStr[pos] = str[i];
            firstSlash = true;
            ++pos;
        }

        if (isSlash(str[i]) && firstSlash)
        {
            newStr[pos] = '/';
            firstSlash = false;
            ++pos;
        }
    }

    str = newStr;
    GE_DELETE_A newStr;
}


} // namespace

namespace Ge
{

//------------------------------------------------------------------------------
FilePath::FilePath(const char* path)
//------------------------------------------------------------------------------
{
    if (path)
    {
        set(path);
    }
}


//------------------------------------------------------------------------------
FilePath::FilePath(const std::string& path)
//------------------------------------------------------------------------------
{
    set(path.c_str());
}


//------------------------------------------------------------------------------
const std::string& FilePath::getExtension() const
//------------------------------------------------------------------------------
{
    return mExtension;
}

//------------------------------------------------------------------------------
const std::string& FilePath::getName() const
//------------------------------------------------------------------------------
{
    return mName;
}

//------------------------------------------------------------------------------
const std::string& FilePath::getDirectory() const
//------------------------------------------------------------------------------
{
    return mDirectory;
}

//------------------------------------------------------------------------------
const std::string& FilePath::get() const
//------------------------------------------------------------------------------
{
    return mPath;
}

//------------------------------------------------------------------------------
void FilePath::setExtension(const char* extension)
//------------------------------------------------------------------------------
{
    mExtension = (extension[0] == '.') ? extension : std::string(".")+extension;
    mFile = mName + (mExtension != "" ? '.' + mExtension : "");
    merge();
}


//------------------------------------------------------------------------------
void FilePath::setDirectory(const char* dir)
//------------------------------------------------------------------------------
{
    mDirectory = dir;
    convertSlashes(mDirectory);
    merge();
}

//------------------------------------------------------------------------------
void FilePath::setName(const char* name)
//------------------------------------------------------------------------------
{
    mName = name;
    mFile = mName + (mExtension != "" ? '.' + mExtension : "");
    merge();
}

//------------------------------------------------------------------------------
const std::string& FilePath::getFile() const
//------------------------------------------------------------------------------
{
    return mFile;
}

//------------------------------------------------------------------------------
void FilePath::setFile(const char* file)
//------------------------------------------------------------------------------
{
    mFile = file;
    UInt i(mFile.find('.'));
    mName = mFile.substr(0, i);
    mExtension = mFile.substr(i+1);
    merge();
}

//------------------------------------------------------------------------------
void FilePath::set(const char* path)
//------------------------------------------------------------------------------
{
    mPath = path;
    convertSlashes(mPath);
    split();
}

//------------------------------------------------------------------------------
void FilePath::split()
//------------------------------------------------------------------------------
{
    if (mPath.size() > 0)
    {
        char drive[_MAX_DRIVE];
        char dir[_MAX_DIR];
        char name[_MAX_FNAME];
        char ext[_MAX_EXT];

        _splitpath(mPath.c_str(), drive, dir, name, ext);
        mDirectory = std::string(drive) + dir;
        mName = name;
        mFile = mName+ext;
        mExtension = ext;
    }
}


//------------------------------------------------------------------------------
void FilePath::merge()
//------------------------------------------------------------------------------
{
    char path[_MAX_PATH];
    _makepath(path, 0, mDirectory.c_str(), mName.c_str(), mExtension.c_str());
    mPath = path;
    convertSlashes(mPath);
}

//------------------------------------------------------------------------------
FilePath& FilePath::operator +=(const char* path)
//------------------------------------------------------------------------------
{
    if (!isSlash(path[0]))
    {
        mPath += '/';
    }

    mPath += path;
    convertSlashes(mPath);
    split();
    return *this;
}

//------------------------------------------------------------------------------
FilePath& FilePath::operator +=(const std::string& path)
//------------------------------------------------------------------------------
{
    return operator +=(path.c_str());
}

//------------------------------------------------------------------------------
FilePath operator +(const FilePath& lhs, const char* rhs)
//------------------------------------------------------------------------------
{
    FilePath ret(lhs);
    ret += rhs;
    return ret;
}

//------------------------------------------------------------------------------
FilePath operator +(const char* lhs, const FilePath& rhs)
//------------------------------------------------------------------------------
{
    return rhs+lhs;
}

//------------------------------------------------------------------------------
FilePath operator +(const FilePath& lhs, const std::string& rhs)
//------------------------------------------------------------------------------
{
    return lhs+rhs.c_str();
}

//------------------------------------------------------------------------------
FilePath operator +(const std::string& lhs, const FilePath& rhs)
//------------------------------------------------------------------------------
{
    return lhs.c_str()+rhs;
}

} // namespace Ge