//------------------------------------------------------------------------------
// (c) 04-2001 Gottfried Chen
//------------------------------------------------------------------------------

#ifndef SYSTEM_FILEPATH_H
#define SYSTEM_FILEPATH_H

#include <string>

namespace Ge
{

// Example: "c:/bla/test.exe"
// - Extension is ".exe"
// - Name is "test"
// - File is "test.exe"
// - Directory is "c:/bla/"
// Take care: backslashes are converted into forward slashes
//            on construction
class FilePath
{
public:
    explicit
    FilePath(const char* path = 0);

    explicit
    FilePath(const std::string&);

    // Access parts
    const std::string& getExtension() const;
    void setExtension(const char*);

    const std::string& getName() const;
    void setName(const char*);

    const std::string& getDirectory() const;
    void setDirectory(const char*);

    const std::string& getFile() const;
    void setFile(const char*);

    // Access the full path
    const std::string& get() const;
    void set(const char*);

    // Operators
    FilePath& operator +=(const char* path);
    FilePath& operator +=(const std::string& path);

private:
    void split();
    void merge();

    std::string mPath;
    std::string mDirectory;
    std::string mName;
    std::string mExtension;
    std::string mFile;
};

FilePath operator +(const FilePath& lhs, const char* rhs);

FilePath operator +(const char* lhs, const FilePath& rhs);

FilePath operator +(const FilePath& lhs, const std::string& rhs);

FilePath operator +(const std::string& lhs, const FilePath& rhs);

} // namespace Ge

#endif // SYSTEM_FILEPATH_H