#include "Precompiled.h"

//------------------------------------------------------------------------------
// (c) 03-2001 Gottfried Chen
//------------------------------------------------------------------------------

#include <System/Log.h>
#include <System/Timer.h>
#include <System/Assert.h>
#include <fstream>
#include <string>
#include <iomanip>

namespace Ge
{

//------------------------------------------------------------------------------
Log::Log(std::ostream& stream) :
//------------------------------------------------------------------------------
mStream(stream)
{
    log(INFO)<<"------------------------------------------------------------\n";
    log(INFO)<<"Begin logging\n";
    log(INFO)<<"------------------------------------------------------------\n";
}

//------------------------------------------------------------------------------
std::ostream& Log::log(Log::Level level) const
//------------------------------------------------------------------------------
{
    static std::string prefix;
    switch (level)
    {
    case DEBUG:
        prefix = "DEBUG @ ";
        break;

    case INFO:
        prefix = "INFO @ ";
        break;

    case ERR:
        prefix = "ERROR @ ";
        break;

    default:
        prefix = "???";
        GE_ASSERT_MSG(false, "Unknown prefix");
        break;
    }

    return mStream<<prefix<<std::setfill('0')<<std::setprecision(4)
                  <<std::setw(5)<<Timer::system().get()/1000.0f<<"s: ";
}


//------------------------------------------------------------------------------
Log& Log::global()
//------------------------------------------------------------------------------
{
#ifdef NDEBUG
    // Not using a static std::ofstream, since the file must live longer
    // than all other objects, so that logging can be used in other objects'
    // destructor calls.
    static std::ofstream* file =
        GE_NEW std::ofstream("GameEngine.log", std::ios::out | std::ios::trunc);
    static Log theLog(*file);
#else
    // When debugging, I want leak detection
    static std::ofstream file("GameEngine.log", std::ios::out | std::ios::trunc);
    static Log theLog(file);
#endif
    
    return theLog;
}

}