#ifndef SYSTEM_MEMORY_H
#define SYSTEM_MEMORY_H
#pragma once

//------------------------------------------------------------------------------
// (c) Mai-2002 Gottfried Chen
//------------------------------------------------------------------------------

#ifdef _WIN32

#include <crtdbg.h>

namespace Ge
{

inline
//------------------------------------------------------------------------------
void turnOnLeakChecking()
//------------------------------------------------------------------------------
{
    static bool first = true;
    if (first)
    {
        _CrtSetDbgFlag(_CRTDBG_LEAK_CHECK_DF | _CrtSetDbgFlag(_CRTDBG_REPORT_FLAG));
        first = false;
    }
}

} // namespace Ge

inline
//------------------------------------------------------------------------------
void* operator new(size_t size, const char* file, unsigned int line)
//------------------------------------------------------------------------------
{
    Ge::turnOnLeakChecking();
    return _malloc_dbg(size, _NORMAL_BLOCK, file, line);
}

// This operator will only be called in case of the above operator new
// throwing an exception.
inline
//------------------------------------------------------------------------------
void operator delete(void* pointer, const char*, unsigned int)
//------------------------------------------------------------------------------
{
    _free_dbg(pointer, _NORMAL_BLOCK);
}

#ifndef NDEBUG
#   define GE_NEW new(__FILE__, __LINE__)
#   define GE_DELETE delete
#   define GE_DELETE_A delete[]
#else // !NDEBUG
#   define GE_NEW new
#   define GE_DELETE delete
#   define GE_DELETE_A delete[]
#endif // !NDEBUG

#else // _WIN32

#define GE_NEW new
#define GE_DELETE delete
#define GE_DELETE_A delete[]

#endif // _WIN32

#endif // !SYSTEM_MEMORY_H