//------------------------------------------------------------------------------
// (c) 03-2001 Gottfried Chen
//------------------------------------------------------------------------------

#ifndef SYSTEM_POINTER_H
#define SYSTEM_POINTER_H

#include <System/Object.h>

namespace Ge
{

class PointerBase
{
protected:
    explicit
    PointerBase::PointerBase(const Object*);

    void reference();
    void dereference();

    const Object* mPointee;
};


// Pointer is a smart pointer that can be used to reference all
// classes derived from Object
template<class T>
class Pointer : public PointerBase
{
public:
    //------------------------------------------------------------------------------
    // Construction
    //------------------------------------------------------------------------------
    explicit
    Pointer(T* pointee = 0);
    // MSVC: the two copy constructors must be in this order (don't know why, but
    // I guess the compiler sees the second one as a specialization of the first
    // one). The first one is needed for upcasting.
    template<class U> inline
    Pointer(const Pointer<U>& pointee) :
    PointerBase(static_cast<T*>(pointee)) { reference(); }
    Pointer(const Pointer& pointer);

    ~Pointer();

    //------------------------------------------------------------------------------
    // Operators
    //------------------------------------------------------------------------------
    T& operator *() const;
    T* operator ->() const;
    operator T*() const;
    
    template<class U> inline
    Pointer& operator =(U* pointee)
    {
        dereference();
        mPointee = pointee;
        reference();
        return *this;
    }

    // MSVC: the two assignement operators must be in this order
    template<class U> inline
    Pointer& operator =(const Pointer<U>& pointee)
    {
        dereference();
        mPointee = static_cast<T*>(pointee);
        reference();
        return *this;
    }

    Pointer& operator =(const Pointer& pointer);

    //------------------------------------------------------------------------------
    // Tools
    //------------------------------------------------------------------------------
    T* get() const;
};

template <class T>
bool operator ==(Pointer<T>&, T*);

template <class T>
bool operator ==(T*, Pointer<T>&);
}

#include <System/Pointer.inl>
#endif // SYSTEM_POINTER_H