//------------------------------------------------------------------------------
// (c) 03-2001 Gottfried Chen
//------------------------------------------------------------------------------

namespace Ge
{

inline
//------------------------------------------------------------------------------
PointerBase::PointerBase(const Object* pointee) :
//------------------------------------------------------------------------------
mPointee(pointee)
{
}

inline
//------------------------------------------------------------------------------
void PointerBase::reference()
//------------------------------------------------------------------------------
{
    if (mPointee != 0)
        mPointee->reference();
}

inline
//------------------------------------------------------------------------------
void PointerBase::dereference()
//------------------------------------------------------------------------------
{
    if (mPointee != 0)
        mPointee->dereference();
}



template<class T> inline
//------------------------------------------------------------------------------
Pointer<T>::Pointer(T* pointee) :
//------------------------------------------------------------------------------
PointerBase(pointee)
{
    reference();
}

template<class T> inline
//------------------------------------------------------------------------------
Pointer<T>::Pointer(const Pointer<T>& pointee) :
//------------------------------------------------------------------------------
PointerBase(static_cast<T*>(pointee))
{
    reference();
}


template<class T> inline
//------------------------------------------------------------------------------
Pointer<T>::~Pointer()
//------------------------------------------------------------------------------
{
    dereference();
    mPointee = 0;
}


template<class T> inline
//------------------------------------------------------------------------------
T& Pointer<T>::operator *() const
//------------------------------------------------------------------------------
{
    return static_cast<T&>(*(const_cast<Object*>(mPointee)));
}


template<class T> inline
//------------------------------------------------------------------------------
T* Pointer<T>::operator ->() const
//------------------------------------------------------------------------------
{
    return static_cast<T*>(const_cast<Object*>(mPointee));
}


template<class T> inline
//------------------------------------------------------------------------------
Pointer<T>::operator T*() const
//------------------------------------------------------------------------------
{
    return static_cast<T*>(const_cast<Object*>(mPointee));
}

template<class T> inline
//------------------------------------------------------------------------------
Pointer<T>& Pointer<T>::operator =(const Pointer<T>& pointer)
//------------------------------------------------------------------------------
{
    dereference();
    mPointee = pointer.mPointee;
    reference();
    return *this;
}

template<class T> inline
//------------------------------------------------------------------------------
T* Pointer<T>::get() const
//------------------------------------------------------------------------------
{
    return static_cast<T*>(const_cast<Object*>(mPointee));
}



template<class T>
//------------------------------------------------------------------------------
bool operator ==(Pointer<T>& lhs, T* rhs)
//------------------------------------------------------------------------------
{
    return static_cast<T*>(lhs) == rhs;
}


template<class T>
//------------------------------------------------------------------------------
bool operator ==(T* lhs, Pointer<T>& rhs)
//------------------------------------------------------------------------------
{
    return rhs == lhs;
}

} // namespace Ge