#include "Precompiled.h"

//------------------------------------------------------------------------------
// (c) 05-2001 Gottfried Chen
//------------------------------------------------------------------------------

#include <System/StringTools.h>
#include <algorithm>
#include <sstream>

namespace Ge
{

//------------------------------------------------------------------------------
Int StringTools::firstWhiteSpace(const std::string& str, UInt start)
//------------------------------------------------------------------------------
{
    GE_ASSERT(start <= str.size());
    UInt i(start < str.size() ? start : 0);
    while (i<str.size() && (str[i] != ' ') && (str[i] != '\t'))
        i++;
        
    return i == str.size() ? -1 : i;
}

//------------------------------------------------------------------------------
Int StringTools::firstNonWhiteSpace(const std::string& str, UInt start)
//------------------------------------------------------------------------------
{
    GE_ASSERT(start <= str.size());
    UInt i(start < str.size() ? start : 0);

    while (i<str.size() && ((str[i] == ' ') || (str[i] == '\t')))
        i++;
        
    return i == str.size() ? -1 : i;
}


//------------------------------------------------------------------------------
void StringTools::split(const std::string& str, std::string& left,
                        std::string& right)
//------------------------------------------------------------------------------
{
    GE_ASSERT(&str != &left);
    Int nonWhite(firstNonWhiteSpace(str));
    if (nonWhite != -1)
    {
        Int white(firstWhiteSpace(str, nonWhite));
        if (white == -1)
        {
            left = str.substr(nonWhite);
            right = "";
        }
        else
        {
            left = str.substr(nonWhite, white-nonWhite);
            right = str.substr(white+1);
        }
    }
    else
    {
        left = "";
        right = str;
    }
}

//------------------------------------------------------------------------------
Float StringTools::toFloat(const std::string& str)
//------------------------------------------------------------------------------
{
    return Float(atof(str.c_str()));
}

//------------------------------------------------------------------------------
Int StringTools::toInt(const std::string& str)
//------------------------------------------------------------------------------
{
    return atoi(str.c_str());
}

//------------------------------------------------------------------------------
bool StringTools::beginsWith(const std::string& prefix, const std::string& str)
//------------------------------------------------------------------------------
{
    return 0 == str.find(prefix, 0);
}

//------------------------------------------------------------------------------
std::string StringTools::toString(UInt number)
//------------------------------------------------------------------------------
{
    char buffer[10];
    itoa(number, buffer, 10);
    return std::string(buffer);
}


//------------------------------------------------------------------------------
void StringTools::toUpper(std::string& string)
//------------------------------------------------------------------------------
{
    std::transform(string.begin(), string.end(), string.begin(), toupper);
}

//------------------------------------------------------------------------------
void StringTools::toLower(std::string& string)
//------------------------------------------------------------------------------
{
    std::transform(string.begin(), string.end(), string.begin(), tolower);
}

//------------------------------------------------------------------------------
void StringTools::trimLeft(std::string& string, char chr)
//------------------------------------------------------------------------------
{
    string.erase(0, string.find_first_not_of(chr));
}

//------------------------------------------------------------------------------
void StringTools::trimRight(std::string& string, char chr)
//------------------------------------------------------------------------------
{
    string.resize(string.find_last_not_of(chr) + 1);
}

//------------------------------------------------------------------------------
void StringTools::trim(std::string& string, char chr)
//------------------------------------------------------------------------------
{
    trimLeft(string, chr);
    trimRight(string, chr);
}

//------------------------------------------------------------------------------
Int StringTools::hexToInt(const std::string& str)
//------------------------------------------------------------------------------
{
    std::string copy(str);
    toLower(copy);
    trim(copy);
    std::istringstream stream(copy);
    Int ret;
    stream>>std::hex>>ret;
    return ret;
}

//------------------------------------------------------------------------------
std::string StringTools::replace(const std::string& str, const char* oldSubstring,
                                                         const char* newSubstring)
//------------------------------------------------------------------------------
{
    GE_ASSERT(oldSubstring != 0);
    GE_ASSERT(newSubstring != 0);

    std::string ret;

    size_t len = strlen(oldSubstring);
    std::string::size_type prevPos = 0;
    std::string::size_type pos = str.find(oldSubstring);
    while (pos != std::string::npos)
    {
        ret += str.substr(prevPos, pos-prevPos);
        ret += newSubstring;
        prevPos = pos + len;
        pos = str.find(oldSubstring, prevPos);
    }

    if (prevPos < str.length())
    {
        ret += str.substr(prevPos, str.length()-prevPos);
    }

    return ret;
}

} // namespace Ge
