//------------------------------------------------------------------------------
// (c) 05-2001 Gottfried Chen
//------------------------------------------------------------------------------

#ifndef SYSTEM_STRING_TOOLS_H
#define SYSTEM_STRING_TOOLS_H

#include <System/Base.h>

namespace Ge
{

struct StringTools
{
    // Find first white space starting at <start>
    static Int firstWhiteSpace(const std::string& str, UInt start = 0);

    // Find first non white space starting at <start>
    static Int firstNonWhiteSpace(const std::string& str, UInt start = 0);

    // Split <str> into <left> and <right> at the first white space
    // separator (leading white spaces are skipped)
    // <str> and <right> can be the same string
    // If <str> contains only white spaces then <left> = "" and <right> = <str>
    static void split(const std::string& str, std::string& left,
                      std::string& right);
    
    static Float toFloat(const std::string& str);
    
    static Int toInt(const std::string& str);
    
    static Int hexToInt(const std::string& str);
    
    static std::string toString(UInt);

    // Returns true, if <str> begins with <prefix>
    static bool beginsWith(const std::string& prefix, const std::string& str);

    // Case conversion
    static void toLower(std::string&);
    
    static void toUpper(std::string&);

    // Trim spaces
    static void trimLeft(std::string&, char = ' ');
    
    static void trimRight(std::string&, char = ' ');
    
    static void trim(std::string&, char = ' ');

    // Replace all occurences of <oldSubstring> by <newSubstring>.
    static std::string replace(const std::string&, const char* oldSubstring,
                               const char* newSubstring);
    
    //------------------------------------------------------------------------------

    // Use this for std::maps with std::string as index
    struct Pred
    {
        inline
        bool operator()(const std::string& lhs, const std::string& rhs) const
        { return lhs.compare(rhs) < 0; }
    };
};

} // namespace Ge

#endif // SYSTEM_STRING_TOOLS_H