#ifndef SYSTEM_TOOLS_H
#define SYSTEM_TOOLS_H
#pragma once

//------------------------------------------------------------------------------
// (c) 03-2001 Gottfried Chen
//------------------------------------------------------------------------------

#include <System/Types.h>

#include <string>

namespace Ge
{

template<class T>
T maximum(const T&, const T&);

template<class T>
T minimum(const T&, const T&);

// This lets you access a dynamic array as a two dimensional
// array. It'll be interpreted as an array of rows (it's like indexing
// into a standard C++ 2D array).
template<class T>
class Array2D
{
public:
    Array2D(T*, unsigned int rows, unsigned int columns);
    
    T& operator ()(unsigned int r, unsigned int c);
    const T& operator ()(unsigned int r, unsigned int c) const;

private:
    T* mData;
    unsigned int mRows;
    unsigned int mColumns;
};

// Suspends the program for <milliseconds> milliseconds.
void sleep(UInt milliseconds);

// Opens a load dialog box and saves the selected file in <path>.
// <filter> is a list of (description, *.xyz) string pairs. List must end with two zeros.
bool openLoadDialog(std::string& path,
                    const char* filter = 0);

// Copy <elementCount> elements of size <elementSize> (in bytes) to <destination>
// where the stride between destination elements is <stride>.
void copyDestinationStride(char* destination, const char* source,
                           UInt elementSize, UInt elementCount, UInt stride);

} // namespace Ge

#include <System/Tools.inl>
#endif // SYSTEM_TOOLS_H