//------------------------------------------------------------------------------
// (c) 03-2001 Gottfried Chen
//------------------------------------------------------------------------------

#include <System/Assert.h>

namespace Ge
{

template<class T> inline
//------------------------------------------------------------------------------
T maximum(const T& lhs, const T& rhs)
//------------------------------------------------------------------------------
{
    return lhs<rhs ? rhs : lhs;
}


template<class T> inline
//------------------------------------------------------------------------------
T minimum(const T& lhs, const T& rhs)
//------------------------------------------------------------------------------
{
    return lhs>rhs ? rhs : lhs;
}

template<class T> inline
//------------------------------------------------------------------------------
Array2D<T>::Array2D(T* data, unsigned int rows, unsigned int columns) :
//------------------------------------------------------------------------------
mData(data),
mRows(rows),
mColumns(columns)
{
}

template<class T> inline
//------------------------------------------------------------------------------
T& Array2D<T>::operator ()(unsigned int r, unsigned int c)
//------------------------------------------------------------------------------
{
    GE_ASSERT(r<mRows && c<mColumns);
    return mData[r*mColumns+c];
}

template<class T> inline
//------------------------------------------------------------------------------
const T& Array2D<T>::operator ()(unsigned int r, unsigned int c) const
//------------------------------------------------------------------------------
{
    GE_ASSERT(r<mRows && c<mColumns);
    return mData[r*mColumns+c];
}

inline
//------------------------------------------------------------------------------
void copyDestinationStride(char* destination, const char* source,
                           UInt elementSize, UInt elementCount, UInt stride)
//------------------------------------------------------------------------------
{
    GE_ASSERT(stride >= elementSize);
    for (UInt i = 0; i<elementCount; ++i)
    {
        memcpy(destination, source, elementSize);
        destination += stride;
        source += elementSize;
    }
}

} // namespace Ge
