#include "Precompiled.h"

//------------------------------------------------------------------------------
// (c) 02-2002 Gottfried Chen
//------------------------------------------------------------------------------

#include "Trace.h"
#include <streambuf>
#include <iostream>

#ifdef _WIN32
#   include <windows.h>
#endif


namespace Ge
{

template<class Character, class Traits = std::char_traits<Character> >
class TraceBuf : public std::basic_streambuf<Character, Traits>
{
public:
    enum
    {
        BUFFER_SIZE = 100
    };

    TraceBuf();

    ~TraceBuf();

protected:
    virtual int_type overflow(int_type c = traits_type::eof());
    
    virtual int sync();

private:
    TraceBuf(const TraceBuf&);
    
    TraceBuf& operator=(const TraceBuf&);

    void flushWriteBuffer(int_type c);
    
    // Leave one character for ends character.
    char_type mWriteBuffer[BUFFER_SIZE+1];
};

TraceBuf<char> theTraceBuf;
::std::ostream theTraceStream(&theTraceBuf);

template<class Character, class Traits>
//------------------------------------------------------------------------------
TraceBuf<Character, Traits>::TraceBuf()
//------------------------------------------------------------------------------
{
    setp(mWriteBuffer, mWriteBuffer + BUFFER_SIZE);
}

template<class Character, class Traits>
//------------------------------------------------------------------------------
TraceBuf<Character, Traits>::~TraceBuf()
//------------------------------------------------------------------------------
{
    flushWriteBuffer(traits_type::eof());
}

template<class C, class T>
//------------------------------------------------------------------------------
void TraceBuf<C, T>::flushWriteBuffer(TraceBuf<C, T>::int_type c)
//------------------------------------------------------------------------------
{
    // Add end of string character
    *pptr() = 0;
#ifdef _WIN32
    OutputDebugString(mWriteBuffer);
#endif
    setp(mWriteBuffer, mWriteBuffer + BUFFER_SIZE);
    
    // Put <c> into write buffer if <c> != eof.
    if (!traits_type::eq_int_type(c, traits_type::eof()))
    {
        sputc(traits_type::to_char_type(c));
    }
}

template<class Character, class Traits>
//------------------------------------------------------------------------------
int TraceBuf<Character, Traits>::sync()
//------------------------------------------------------------------------------
{
    flushWriteBuffer(traits_type::eof());
    return 0;
}

template<class C, class T>
//------------------------------------------------------------------------------
TraceBuf<C, T>::int_type TraceBuf<C, T>::overflow(TraceBuf<C, T>::int_type c)
//------------------------------------------------------------------------------
{
    flushWriteBuffer(c);
    return traits_type::not_eof(c);
}

} // namespace Ge

