#include "Precompiled.h"
//------------------------------------------------------------------------------
// (c) 02-2002 Gottfried Chen
//------------------------------------------------------------------------------

#include "ChoppyWaves.h"

namespace Ge
{

//------------------------------------------------------------------------------
ChoppyWaves::ChoppyWaves(Float lambda,
                         UInt xPower, UInt yPower,
                         Float a, const Vector2& wind,
                         Float xWorld, Float yWorld) :
//------------------------------------------------------------------------------
StatisticalWater(xPower, yPower, a, wind, xWorld, yWorld),
mLambda(lambda)
{
    GE_ASSERT(lambda <= 0);
    mDeltaX = GE_NEW Complex[getXSize()*getYSize()];
    mDeltaZ = GE_NEW Complex[getXSize()*getYSize()];
}

//------------------------------------------------------------------------------
ChoppyWaves::~ChoppyWaves()
//------------------------------------------------------------------------------
{
    GE_DELETE_A mDeltaX;
    GE_DELETE_A mDeltaZ;
}

//------------------------------------------------------------------------------
Float ChoppyWaves::getLambda() const
//------------------------------------------------------------------------------
{
    return mLambda;
}

//------------------------------------------------------------------------------
void ChoppyWaves::setLambda(Float lambda)
//------------------------------------------------------------------------------
{
    GE_ASSERT(lambda <= 0);
    mLambda = lambda;
}

//------------------------------------------------------------------------------
const Complex* ChoppyWaves::getDeltaMapX() const
//------------------------------------------------------------------------------
{
    return mDeltaX;
}

//------------------------------------------------------------------------------
const Complex* ChoppyWaves::getDeltaMapZ() const
//------------------------------------------------------------------------------
{
    return mDeltaZ;
}

//------------------------------------------------------------------------------
void ChoppyWaves::store(UInt index, const Vector2& k, Float kLen,
                        const Complex& result)
//------------------------------------------------------------------------------
{
    StatisticalWater::store(index, k, kLen, result);

    mDeltaX[index] = kLen == 0 ?
                     0 :
                     Complex(0, -k[0]/kLen)*result;

    mDeltaZ[index] = kLen == 0 ?
                     0 :
                     Complex(0, -k[1]/kLen)*result;         
}

//------------------------------------------------------------------------------
void ChoppyWaves::transformToTime()
//------------------------------------------------------------------------------
{
    StatisticalWater::transformToTime();

    UInt yLine(0);

    getFft().calculate(mDeltaX);
    getFft().calculate(mDeltaZ);
    
    for (UInt y = 0; y<getYSize(); ++y)
    {
        yLine = y*getXSize();
        for (UInt x = 0; x<getXSize(); ++x)
        {
            // The neg1pow term results from shifting the sum index from [-N/2, N)
            // to [0, N)
            mDeltaX[yLine+x] *= Float(neg1Pow(x+y))*mLambda;
            mDeltaZ[yLine+x] *= Float(neg1Pow(x+y))*mLambda;
        }
    }

}

} // namespace Ge
