//------------------------------------------------------------------------------
// (c) 02-2002 Gottfried Chen
//------------------------------------------------------------------------------

#ifndef CHOPPY_WAVES_H
#define CHOPPY_WAVES_H

#include "StatisticalWater.h"

namespace Ge
{

class ChoppyWaves : public StatisticalWater
{
public:
    // Parameters:
    // - <lambda> is the choppiness scaling factor. Valuse must be negative.
    //   0 is no chopiness.
    // - The other parameters are the same as in the StatisticalWater constructor.
    //
    // For example values see StatisticalWater.
    // -0.15f is a reasonable value for <lambda>.
    //
    // ToDo: Check why lambda must be negative in my implementation (is this a
    // bug? I've rechecked the equations though).
    ChoppyWaves(Float lambda,
                UInt xPower, UInt yPower,
                Float a, const Vector2& wind,
                Float xWorld, Float yWorld);
    
    ~ChoppyWaves();

    // <lambda> is the choppiness scaling factor.
    Float getLambda() const;

    void setLambda(Float);

    // Delta map in the x/z plane to generate choppy looking waves.
    const Complex* getDeltaMapX() const;
    
    const Complex* getDeltaMapZ() const;

protected:
    virtual
    void store(UInt index, const Vector2& k, Float kLen, const Complex& result);

    virtual
    void transformToTime();

private:
    Complex* mDeltaX;
    Complex* mDeltaZ;
    Float mLambda;
};


} // namespace Ge

#endif // CHOPPY_WAVES_H
