//------------------------------------------------------------------------------
// (c) 02-2002 Gottfried Chen
//------------------------------------------------------------------------------

#ifndef DELTA_XZ_MAP_H
#define DELTA_XZ_MAP_H

#include <System/Base.h>
#include <utility>

namespace Ge
{

class DeltaXzMap
{
public:
    DeltaXzMap(UInt xSize, UInt ySize,
               const Complex* xDelta, const Complex* zDelta);

    UInt getXSize() const;
    UInt getYSize() const;

    std::pair<Float, Float> get(UInt x, UInt y) const;
    std::pair<Float, Float> get(UInt index) const;

private:
    UInt mXSize;
    UInt mYSize;
    const Complex* mXDelta;
    const Complex* mZDelta;
};

inline
//------------------------------------------------------------------------------
DeltaXzMap::DeltaXzMap(UInt xSize, UInt ySize,
                       const Complex* xDelta, const Complex* zDelta) :
//------------------------------------------------------------------------------
mXSize(xSize),
mYSize(ySize),
mXDelta(xDelta),
mZDelta(zDelta)
{
}

inline
//------------------------------------------------------------------------------
std::pair<Float, Float> DeltaXzMap::get(UInt x, UInt y) const
//------------------------------------------------------------------------------
{
    GE_ASSERT(mXDelta != 0 && mZDelta != 0);
    return std::make_pair(mXDelta[y*mXSize+x].real(), mZDelta[y*mXSize+x].real());
}

inline
//------------------------------------------------------------------------------
std::pair<Float, Float> DeltaXzMap::get(UInt index) const
//------------------------------------------------------------------------------
{
    GE_ASSERT(mXDelta != 0 && mZDelta != 0);
    return std::make_pair(mXDelta[index].real(), mZDelta[index].real());
}

inline
//------------------------------------------------------------------------------
UInt DeltaXzMap::getXSize() const
//------------------------------------------------------------------------------
{
    return mXSize;
}

inline
//------------------------------------------------------------------------------
UInt DeltaXzMap::getYSize() const
//------------------------------------------------------------------------------
{
    return mYSize;
}

} // namespace Ge

#endif // DELTA_XZ_MAP_H