//------------------------------------------------------------------------------
// (c) 01-2002 Gottfried Chen
//------------------------------------------------------------------------------

#ifndef FAULT_FORMATION_H
#define FAULT_FORMATION_H

namespace Ge
{

class FaultFormation
{
public:
    // <startHeight> is the added height difference in the first iteration,
    // <endHeight> in the last. (<startHeight> must be bigger than <endHeight>).
    // <itPerFilter> denotes the number of iterations before the map is filtered.
    // <filterConstant> (between 0 and 1) denotes the constant for the FIR filter
    // (0 is unfiltered, 1 is very smooth).
    FaultFormation(unsigned int xSize, unsigned int ySize,
                   float startHeight, float endHeight,
                   unsigned int iterations, unsigned int itPerFilter,
                   float filterConstant);

    ~FaultFormation();

    void calculate();

    unsigned int getXSize() const;
    unsigned int getYSize() const;
    float get(unsigned int x, unsigned int y) const;
    float getMaxHeight() const;
    float getMinHeight() const;

    const float* getMap() const;

private:
    // Calculates the min/max heights that were reached during calculate()
    void calculateMinMax();

    float* mMap;
    unsigned int mXSize;
    unsigned int mYSize;
    float mStartHeight;
    float mHeightDiff;
    float mMaxHeight;
    float mMinHeight;

    unsigned int mIterations;
    unsigned int mItPerFilter;
    float mConstant;
    bool mDone;
};

} // namespace Ge

#endif // FAULT_FORMATION_H
