//------------------------------------------------------------------------------
// (c) 02-2002 Gottfried Chen
//------------------------------------------------------------------------------

#ifndef GERSTNER_WAVES_H
#define GERSTNER_WAVES_H

#include <System/Base.h>
#include <System/Timer.h>
#include <Math/Vector.h>
#include <vector>

namespace Ge
{

// Gerstner waves are not heightmap based and thus differ from the other water
// algorithms. The create a mesh directly and allow overlapping waves.
class GerstnerWaves
{
public:
    // The user takes ownership of the vertex and index arrays and has to delete
    // them afterwards with GE_DELETE_A!
    GerstnerWaves(UInt xSize, UInt ySize, Float xWorld, Float yWorld);

    // Time is absolute (i.e. not relative to last frame)
    void update(const Timer::Unit&);

    // Add a wave. <length> is the wavelength not the length of the vector k.
    // The wave vector only is used to indicate the wave direction.
    void addWave(const Vector2& k, Float length, Float amplitude, Float phase);

    
    //------------------------------------------------------------------------------
    // Mesh accessors
    //------------------------------------------------------------------------------
    
    UInt getVertexCount() const;
    
    Vector3* getVertices();
    
    UInt getIndexCount() const;
    
    UInt16* getIndices();

private:
    struct Wave
    {
        Vector2 mK;
        Float mLength;
        Float mAmplitude;
        Float mFrequency;
        Float mPhase;
    };

    std::vector<Wave> mWaves;
    Vector3* mVertices;
    UInt16* mIndices;
    UInt mXSize;
    UInt mYSize;
    Float mXWldSz;
    Float mYWldSz;
};

} // namespace Ge

#endif // GERSTNER_WAVES_H