#include "Precompiled.h"
//------------------------------------------------------------------------------
// (c) 01-2002 Gottfried Chen
//------------------------------------------------------------------------------

#include "HeightMap.h"
#include "FaultFormation.h"
#include "MidpointDisplacement.h"
#include "ParticleDeposition.h"
#include "Newave.h"
#include "Water2D.h"
#include "StatisticalWater.h"

#include "ImageHeightMap.h"
#include "FloatMap.h"
#include "UCharMap.h"
#include "ComplexMap.h"

#include <Image/Image.h>

namespace Ge
{

//------------------------------------------------------------------------------
HeightMap::HeightMap(Image& im)
//------------------------------------------------------------------------------
{
    mImpl = GE_NEW ImageHeightMap(im);
}

//------------------------------------------------------------------------------
HeightMap::HeightMap(FaultFormation& ff)
//------------------------------------------------------------------------------
{
    mImpl = GE_NEW FloatMap(ff.getMap(), ff.getXSize(), ff.getYSize(),
                         ff.getMinHeight(), ff.getMaxHeight());
}

//------------------------------------------------------------------------------
HeightMap::HeightMap(MidpointDisplacement& md)
//------------------------------------------------------------------------------
{
    mImpl = GE_NEW FloatMap(md.getMap(), md.getSize(), md.getSize(),
                         md.getMinHeight(), md.getMaxHeight());
}

//------------------------------------------------------------------------------
HeightMap::HeightMap(ParticleDeposition& pd)
//------------------------------------------------------------------------------
{
    mImpl = GE_NEW UCharMap(pd.getMap(), pd.getXSize(), pd.getYSize());
}

//------------------------------------------------------------------------------
HeightMap::HeightMap(Newave& nw)
//------------------------------------------------------------------------------
{
    mImpl = GE_NEW FloatMap(nw.getMap(), nw.getSize(), nw.getSize(),
                         -float(nw.getSize()), nw.getSize());
}

//------------------------------------------------------------------------------
HeightMap::HeightMap(Water2D& w2d)
//------------------------------------------------------------------------------
{
    mImpl = GE_NEW FloatMap(w2d.getMap(), w2d.getXSize(), w2d.getYSize(),
                         -float(w2d.getXSize()), w2d.getXSize());
}

//------------------------------------------------------------------------------
HeightMap::HeightMap(StatisticalWater& sw)
//------------------------------------------------------------------------------
{
    mImpl = GE_NEW ComplexMap(sw.getMap(), sw.getXSize(), sw.getYSize(),
                           -60, 60);
}

//------------------------------------------------------------------------------
void HeightMap::save(const char* fileName) const
//------------------------------------------------------------------------------
{
    Image im(getXSize(), getYSize(), Image::LUMINANCE_8);
    UInt8* d = im.getData();
    for (UInt y = 0; y<getYSize(); ++y)
    {
        for (UInt x = 0; x<getXSize(); ++x)
        {
            *d++ = get(x, y);
        }
    }

    im.save(Image::TGA, fileName);
}

} // namespace Ge
