//------------------------------------------------------------------------------
// (c) 01-2002 Gottfried Chen
//------------------------------------------------------------------------------

#ifndef HEIGHT_MAP_H
#define HEIGHT_MAP_H

#include <System/Pointer.h>

namespace Ge
{

class Image;
class FaultFormation;
class MidpointDisplacement;
class HeightMapImpl;
class ParticleDeposition;
class Newave;
class Water2D;
class StatisticalWater;


// A HeightMap does NOT take ownership of the passed heightmap's memory.
// I works merely as an adaptor to provide a common interface for the
// different heightmap algorithms (e.g. FaultFormation or any of the
// water algorithms).
class HeightMap
{
public:
    // Maybe I'll reformulate these constructors to a member template
    // later, so adding a new height map type doesn't require touching
    // this code.
    HeightMap(Image&);
    HeightMap(FaultFormation&);
    HeightMap(MidpointDisplacement&);
    HeightMap(ParticleDeposition&);
    HeightMap(Newave&);
    HeightMap(Water2D&);
    HeightMap(StatisticalWater&);
    
    unsigned char get(unsigned int x, unsigned int y) const;
    
    unsigned int getXSize() const;
    unsigned int getYSize() const;

    // Save heightmap as .tga image.
    void save(const char* fileName) const;

private:
    Pointer<HeightMapImpl> mImpl;
};

class HeightMapImpl : public Object
{
public:
    virtual
    ~HeightMapImpl() {}

    virtual
    unsigned char get(unsigned int x, unsigned int y) const = 0;
    
    virtual
    unsigned int getXSize() const = 0;

    virtual
    unsigned int getYSize() const = 0;
};

} // namespace Ge

#include "HeightMap.inl"
#endif // HEIGHT_MAP_H
