//------------------------------------------------------------------------------
// (c) 01-2002 Gottfried Chen
//------------------------------------------------------------------------------

#ifndef HEIGHT_MAP_CONVERTER_H
#define HEIGHT_MAP_CONVERTER_H

#include <Math/Vector.h>

namespace Ge
{

class Terrain;

// This is class is needed by Terrain to create a mesh from a height map. You
// can use it directly. But it's more considered to be a helper class only.
// Use class Terrain instead.
//
// A height map is considered an array of rows (i.e. subsequent bytes are in
// the same row and not column). The first entry is considered to be the top left.
// -----> x
// |
// |
// |
// v
// y
//
// The mesh is positioned in the x/z plane. I.e. x from above is mapped to the
// x-axis, positive y to the negative z-axis and the height to the positive y-axis.
class HeightMapConverter
{
public:
    enum Triangulation
    {
        MIN_HEIGHT_VARIATION,
        MAX_HEIGHT_VARIATION,
        TOP_TO_BOTTOM,
        BOTTOM_TO_TOP,
        DEFAULT = MIN_HEIGHT_VARIATION
    };

    // If <allocateMemory> is true then createMesh will allocate
    // new memory for the vertex data. Otherwise existing arrays will
    // be written to (the user must assure, that they are big enough!).
    HeightMapConverter(bool allocateMemory);

    // The user takes ownership of the memory allocated for vertex data and <indices>
    // arrays. <indices>, <normal> and <uvs> are calculated if != 0.
    void createMesh(const Terrain&,
                 unsigned int& vertexCount, Vector3*& vertices,
                 Vector3** normals, Vector2** uvs,
                 unsigned int* indexCount, UInt16** indices) const;

private:
    void buildVertexArray(const Terrain&,
                          unsigned int& vertexCount, Vector3*& vertices,
                          Vector3** normals, Vector2** uvs) const;

    void buildVertexArrayWithDeltaXzMap(
        const Terrain&, unsigned int& vertexCount, Vector3*& vertices,
        Vector3** normals, Vector2** uvs) const;

    void triangulate(const Terrain&,
                     unsigned int& indexCount, UInt16*& indices) const;

    void triangulateTopBottom(UInt16* currentSquare, UInt16* corners) const;
    void triangulateBottomTop(UInt16* currentSquare, UInt16* corners) const;

    bool mAllocateMemory;
};

} // namespace Ge

#endif // HEIGHT_MAP_CONVERTER_H
