//------------------------------------------------------------------------------
// (c) 01-2002 Gottfried Chen
//------------------------------------------------------------------------------

#ifndef MIDPOINT_DISPLACEMENT_H
#define MIDPOINT_DISPLACEMENT_H

namespace Ge
{

class MidpointDisplacement
{
public:
    // The size of the map is 2^<sizePower>. Further the initial <displacement>
    // height and a <roughness> factor can be provided.
    // <filterConstant> is the FIR filter constant applied after map
    // generation. If it is 0 then no filtering is done.
    MidpointDisplacement(unsigned int sizePower, float displacement, float roughness,
                         float filterConstant);
                   
    ~MidpointDisplacement();

    void calculate();

    unsigned int getSize() const;
    float get(unsigned int x, unsigned int y) const;
    float getMaxHeight() const;
    float getMinHeight() const;

    const float* getMap() const;

private:
    void set(unsigned int x, unsigned int y, float value);
    void diamond(unsigned int level, float displacement);
    void square(unsigned int level, float displacement);
    // Calculates the min/max heights that were reached during calculate()
    void calculateMinMax();


    float* mMap;
    unsigned int mLevels;
    unsigned int mSize;
    float mDisplacement;
    float mMultiplier;
    float mMaxHeight;
    float mMinHeight;
    float mConstant;

    bool mDone;
};

} // namespace Ge

#endif // MIDPOINT_DISPLACEMENT_H
