//------------------------------------------------------------------------------
// (c) 01-2002 Gottfried Chen
//------------------------------------------------------------------------------

#ifndef NEWAVE_H
#define NEWAVE_H

#include <System/Base.h>

namespace Ge
{

// Water algorithm taken from the SGI newave demo (ported from IrisGL
// to OpenGL by Erik Larsen).
class Newave
{
public:
    enum Mode
    {
        FLAT,
        SPIKE,
        HOLE,
        DIAGONALWALL,
        SIDEWALL,
        DIAGONALBLOCK,
        MIDDLEBLOCK,
        CORNERBLOCK,
        HILL,
        HILLFOUR
    };

    static const Float DAMPENING;

    // - <size> is the size of the water grid (assumed to be a square).
    // - <speed> is the speed of wave propagation. 0.004 is a reasonable value.
    Newave(UInt size, Float speed);

    ~Newave();
    
    // Release a drop of water into the simulation.
    // - <x> must be in [1, <size>-2]
    // - <y> must be in [1, <size>-2]
    void drop(UInt x, UInt y);
 
    // I tried to make update() time dependent. But this didn't work out because
    // this water method relies on fixed time intervals. So introducing time as
    // a parameter made the solution unstable. The only method that might work is
    // assuming a fixed frame time t. If a frame takes longer than t then just
    // call update as many times as needed to catch up (e.g. if a frame took
    // 3*t then call update 3 times). This won't work if the water itself is the
    // bottleneck though because then the update call multiplier will increase
    // to infinity.
    void update();

    // Reset the water simulation.
    void reset(Mode);

    UInt getSize() const;
    const Float* getMap() const;
    
private:
    UInt mSize;
    Float mSpeed;

    Float* mForce;
    Float* mVelocity;
    Float* mPosition;
    
};

} // namespace Ge

#endif // NEWAVE_H