//------------------------------------------------------------------------------
// (c) 01-2002 Gottfried Chen
//------------------------------------------------------------------------------

#ifndef PARTICLE_DEPOSITION_H
#define PARTICLE_DEPOSITION_H

namespace Ge
{

// ToDo: Caldera inversion is recursive and can lead to a stack overflow.
// So it's currently disabled. I'll get rid of the recusrion later.

// Forward declaration needed for private Particle class.
class ParticleDeposition;

// Currently the terrain wraps, which looks quite dumb sometimes. Later i might
// change this to create non wrapping terrain.
class ParticleDeposition
{
public:
    // <dropsPerStep>: How many particles are dropped before the drop point moves
    //                 into an adjacent space.
    // <min/maxParticlesPerMountain>: Range of particle number dropped to form
    //                                a mountain, until the drop point is moved
    //                                to another random spot.
    ParticleDeposition(unsigned int xSize, unsigned int ySize,
        float filterConstant, unsigned int dropsPerStep, unsigned int mountains,
        unsigned int minParticlesPerMountain, unsigned int maxParticlesPerMountain,
        float invertCalderaPercentage);
                   
    ~ParticleDeposition();

    void calculate();

    unsigned int getXSize() const;
    unsigned int getYSize() const;

    unsigned char get(unsigned int x, unsigned int y) const;

    const unsigned char* getMap() const;
  
private:
    class Particle
    {
    public:
        Particle(ParticleDeposition&);
        void drop(int x, int y);
    private:
        ParticleDeposition& mPd;
    };

    friend class ParticleDeposition::Particle;

    // Raise point at <x>, <y> by 1.
    void raise(unsigned int x, unsigned int y);

    // Invert all points recursively starting at a selected point
    // that are higher than <limit>.
    void invert(unsigned int x, unsigned int y, unsigned char limit);

    // Strech result to full [0, 255) range
    void normalize();

    // Finds a highest point. Returns  <x>, <y> and <height>
    void findPeak(unsigned int& x, unsigned int& y,
                  unsigned char& height) const;

    unsigned char* mMap;

    // Parameters
    float mConstant;
    unsigned int mXSize;
    unsigned int mYSize;
    unsigned int mDrops;
    unsigned int mMountains;
    unsigned int mMinPPM;
    unsigned int mMaxPPM;

    bool mDone;
    float mPercentage;
};

} // namespace Ge

#endif // PARTICLE_DEPOSITION_H
