//------------------------------------------------------------------------------
// (c) 01-2002 Gottfried Chen
//------------------------------------------------------------------------------

#ifndef TEXTURE_BLENDER_H
#define TEXTURE_BLENDER_H

#include <System/Base.h>
#include <System/Pointer.h>
#include <Image/Image.h>
#include <Math/Interval.h>

#include <vector>

#include "Terrain.h"

namespace Ge
{

class Terrain;

class TextureBlender
{
public:
    TextureBlender();

    // The texture for the given height map/range is created.
    void createTexture(const Terrain&, Image&) const;          

private:
    void calculateSlopePerTexelMap(const Terrain&, const Image& im) const;
    void calculateHeightPerTexelMap(const Terrain&, const Image& im) const;
    float calculateHeightForTexel(const Terrain&, const Image& im, unsigned int x, unsigned int y) const;

    float getSlopeFactorForTexel(Terrain::Grounds::const_iterator g, unsigned int offset) const;
    float getHeightFactorForTexel(Terrain::Grounds::const_iterator g,unsigned int offset) const;

    // This are only helper arrays used during calculate.
    // So they are mutable to keep calculate() const
    mutable float* mHeightPerTexel;
    mutable float* mSlopePerTexel;
};

} // namespace Ge

#endif // TEXTURE_BLENDER_H