#include "Precompiled.h"
//------------------------------------------------------------------------------
// (c) Aug-2002 Gottfried Chen
//------------------------------------------------------------------------------

#include "BumpMd3Context.h"

#include <Engine/Md3Model.h>
#include <Engine/Md3Importer.h>
#include <Engine/Renderer.h>
#include <Engine/Link.h>
#include <Engine/Cluster.h>
#include <Engine/Mesh.h>
#include <Engine/Light.h>
#include <Engine/BumpMaterial.h>
#include <Engine/TexturePool.h>
#include <Engine/VideoCard.h>

#include <Image/Image.h>

#include <string>

namespace Ge
{

//------------------------------------------------------------------------------
BumpMd3Context::BumpMd3Context() :
//------------------------------------------------------------------------------
mToggleSpecular(Keyboard::S),
mSpecularOn(true)
{
    setName("BumpMd3Context");
}

//------------------------------------------------------------------------------
void BumpMd3Context::init()
//------------------------------------------------------------------------------
{
    Md3Context::init();
    if (mModel != 0)
    {
        mSpecularOn = true;
        Cluster::Links& links = mModel->getCluster().getLinks();
        for (UInt i = 0; i<links.size(); ++i)
        {
            // Convert texture to normal map and set BumpMaterial
            const Texture* texture = links[i]->getMaterial()->getTexture(0);
            if (texture)
            {
                BumpMaterial* material = GE_NEW BumpMaterial;
                Image* image = texture->convertToImage();
                image->convertTo(Image::LUMINANCE_8);
                Image* bump = image->createNormalMap(30);
                Texture* bumpTexture =
                    theTexturePool().get(*bump, (texture->getName()+"Bump").c_str());

                GE_DELETE image;
                GE_DELETE bump;

                material->set(0, *bumpTexture);
                material->set(1, *links[i]->getMaterial()->getTexture(0));
                links[i]->set(*material);
                material->setLink(*links[i]);
            }

            Vector2* uv1 = GE_NEW Vector2[links[i]->getMesh()->getVertexCount()];
            memcpy(uv1,
                   links[i]->getMesh()->getUvs(0),
                   links[i]->getMesh()->getVertexCount()*sizeof(Vector2)
                   );

            links[i]->getMesh()->setUvs(1, Mesh::UV_ARRAY, uv1);
            links[i]->getMesh()->calculateTangents(true);
        }

        mModel->update();
        setLightInModel();
    }
}

//------------------------------------------------------------------------------
void BumpMd3Context::update()
//------------------------------------------------------------------------------
{
    if (mModel != 0)
    {
        Md3Context::update();
        setLightInModel();

        mToggleSpecular.update();
        if (mToggleSpecular.isButtonDown())
        {
            mSpecularOn = !mSpecularOn;
            Cluster::Links& links = mModel->getCluster().getLinks();
            for (UInt i = 0; i<links.size(); ++i)
            {
                BumpMaterial* bm = reinterpret_cast<BumpMaterial*>(links[i]->getMaterial());
                bm->activateSpecular(mSpecularOn);
            }
        }
    }
}

//------------------------------------------------------------------------------
void BumpMd3Context::setLightInModel()
//------------------------------------------------------------------------------
{
    if (mModel != 0)
    {
        Cluster::Links& links = mModel->getCluster().getLinks();
        for (UInt i = 0; i<links.size(); ++i)
        {
            links[i]->setLight(0, mLight->getPosition());
        }
    }
}

//------------------------------------------------------------------------------
void BumpMd3Context::render()
//------------------------------------------------------------------------------
{
    if (!theVideoCard().supportsDot3())
    {
        theRenderer().print(TEXT_X_POS, TEXT_Y_POS, "No dot3 blend mode supported.");
        return;
    }

    Md3Context::render();
}

//------------------------------------------------------------------------------
void BumpMd3Context::exit()
//------------------------------------------------------------------------------
{
    Md3Context::exit();
}

} // namespace Ge
