#include "Precompiled.h"
//------------------------------------------------------------------------------
// (c) Jun-2002 Gottfried Chen
//------------------------------------------------------------------------------

#include "GerstnerWavesContext.h"

#include <Application/Metronome.h>

#include <Engine/Mesh.h>
#include <Engine/Link.h>
#include <Engine/RenderCache.h>
#include <Engine/CoordinateAxes.h>
#include <Engine/TexturePool.h>

#include <Application/Settings.h>

#include <System/FilePath.h>
#include <System/FileSystem.h>


namespace Ge
{

//------------------------------------------------------------------------------
GerstnerWavesContext::GerstnerWavesContext() :
//------------------------------------------------------------------------------
WaterContextBase("GerstnerWavesContext", 0, 0),
mGerstner(60, 60, 60, 60)
{
}


//------------------------------------------------------------------------------
void GerstnerWavesContext::init()
//------------------------------------------------------------------------------
{
    WaterContextBase::init();

    //------------------------------------------------------------------------------
    // Init water
    //------------------------------------------------------------------------------
    
    mGerstner.addWave(Vector2(1, 1), 5, 0.1f, 1);
    mGerstner.addWave(Vector2(2, 1), 20, 0.8f, 3);
    mGerstner.addWave(Vector2(3, 2), 10, 0.3f, 2);
    mGerstner.addWave(Vector2(2, 1), 40, 1.4f, 9);
    mGerstner.update(theMetronome().getAbsolute()/1000.0f);

    //------------------------------------------------------------------------------
    // Add mesh to scene
    //------------------------------------------------------------------------------

    mMesh = GE_NEW Mesh(1, mGerstner.getVertexCount(), mGerstner.getVertices(), 0, 0, 0,
                        mGerstner.getIndexCount(), mGerstner.getIndices());
    mMesh->setDynamic(true);
    mMesh->setUvs(0, Mesh::UV_SPHERE);

    Link* l = GE_NEW Link(*Locator::create(Matrix3x4::IDENTITY()), *mMesh);

//    // Need this so that bounding sphere gets calculated
//    g->updateGeometry();

    // Set texture
    FilePath path(theFileSystem().getApplicationPath());
    path += theSettings.getVariable("data").asString();
    path += "/Textures/ReflectOcean.tga";
    Material* material = GE_NEW Material;
    material->set(0, *(theTexturePool().get(path.get().c_str())));
    l->set(*material);

    // Add geometry to world
    l->calculateBound();
    getWorld().add(*l);

    // Add light
    getWorld().add(getLight());
}

    
//------------------------------------------------------------------------------
void GerstnerWavesContext::update()    
//------------------------------------------------------------------------------
{
    WaterContextBase::update();

    mGerstner.update(theMetronome().getAbsolute()/1000.0f);
    
    // Recalculate the vertex normals. The cache is automatically updated.
    mMesh->calculateNormals();
}


//------------------------------------------------------------------------------
void GerstnerWavesContext::exit()
//------------------------------------------------------------------------------
{
}   

} // namespace Ge
