#include "Precompiled.h"
//------------------------------------------------------------------------------
// (c) Aug-2002 Gottfried Chen
//------------------------------------------------------------------------------

#include "Md3Context.h"

#include <Engine/Md3Model.h>
#include <Engine/Md3Importer.h>
#include <Engine/Renderer.h>
#include <Engine/Link.h>
#include <Engine/Locator.h>
#include <Engine/Mesh.h>
#include <Engine/Light.h>

#include <Image/Image.h>

namespace
{

using namespace Ge;

const char* models[] =
{
    "models/players/anarki",
    "models/players/biker",
    "models/players/bitterman",
    "models/players/bones",
//    "models/players/crash", // Leave this out: crash_f.tga doesn't exist!!
    "models/players/doom",
    "models/players/grunt",
    "models/players/hunter",
    "models/players/keel",
    "models/players/klesk",
    "models/players/lucy",
    "models/players/major",
    "models/players/mynx",
    "models/players/orbb",
    "models/players/ranger",
    "models/players/razor",
    "models/players/sarge",
    "models/players/slash",
    "models/players/sorlag",
    "models/players/tankjr",
    "models/players/uriel",
    "models/players/visor",
    "models/players/xaero"
};

const UInt MODEL_COUNT = 22;

const char* legAnims[] =
{
    "BOTH_DEATH1",
    "BOTH_DEAD1",
    "BOTH_DEATH2",
    "BOTH_DEAD2",
    "BOTH_DEATH3",
    "BOTH_DEAD3",
    "LEGS_WALKCR",
    "LEGS_WALK",
    "LEGS_RUN",
    "LEGS_BACK",
    "LEGS_SWIM",
    "LEGS_JUMP",
    "LEGS_LAND",
    "LEGS_JUMPB",
    "LEGS_LANDB",
    "LEGS_IDLE", // 15
    "LEGS_IDLECR",
    "LEGS_TURN"
};

const UInt LEG_ANIM_COUNT = 18;

const char* torsoAnims[] =
{
    "BOTH_DEATH1",
    "BOTH_DEAD1",
    "BOTH_DEATH2",
    "BOTH_DEAD2",
    "BOTH_DEATH3",
    "BOTH_DEAD3",
    "TORSO_GESTURE",
    "TORSO_ATTACK",
    "TORSO_ATTACK2",	
    "TORSO_DROP",
    "TORSO_RAISE",
    "TORSO_STAND", // 11
    "TORSO_STAND2"
};

const UInt TORSO_ANIM_COUNT = 13;

} // namespace


namespace Ge
{

const UInt Md3Context::TEXT_X_POS = 5;
const UInt Md3Context::TEXT_Y_POS = 80;

//------------------------------------------------------------------------------
Md3Context::Md3Context() :
//------------------------------------------------------------------------------
Context("Md3Context"),
mCamera(*Locator::create(Matrix3x4::IDENTITY())),
mCameraController(mCamera),
mModel(0),
mNextTorsoAnim(Keyboard::T),
mCurrentTorsoAnim(0),
mNextLegsAnim(Keyboard::L),
mCurrentLegsAnim(0),
mStartAnim(Keyboard::SPACE),
mNextModel(Keyboard::ENTER),
mCurrentModel(8), // Start with Klesk
mWorld(0)
{
    Vector3 pos(0,5,-50);
    mCamera.getLocator().get().setColumn(MatrixBase::TRANSLATION, pos);
    mCamera.getLocator().update();
    mCameraController.setDistance(pos.length());
    mCameraController.recalculate();
}

//------------------------------------------------------------------------------
void Md3Context::init()
//------------------------------------------------------------------------------
{
    mWorld = GE_NEW World;
    mWorld->setCamera(&mCamera);

    mLight = GE_NEW Light;
    mLight->setRange(100);
    mWorld->add(*mLight);

    Md3Importer importer(models[mCurrentModel]);
    mModel = importer.import();
    if (mModel != 0)
    {
        mModel->update();
        mWorld->add(mModel->getCluster());
    }
}

//------------------------------------------------------------------------------
void Md3Context::update()
//------------------------------------------------------------------------------
{
    if (mModel != 0)
    {
        mModel->update();

        mCameraController.update();

//        // Point light
//        Vector3 cameraPos;
//        mCamera.getLocator().get().getColumn(MatrixBase::TRANSLATION, cameraPos);
//        Vector4 lightPos;
//        set(lightPos, cameraPos, 1.0f);

        // Directional light
        Vector3 cameraDir;
        mCamera.getLocator().get().getColumn(MatrixBase::AT, cameraDir);
        Vector4 lightPos;
        set(lightPos, -cameraDir, 0.0f);
        
        mLight->setPosition(lightPos);

        mNextTorsoAnim.update();
        mNextLegsAnim.update();
        mStartAnim.update();
        mNextModel.update();
        
        if (mNextTorsoAnim.isButtonDown())
        {
            ++mCurrentTorsoAnim;
            if (mCurrentTorsoAnim >= TORSO_ANIM_COUNT)
            {
                mCurrentTorsoAnim = 0;
            }
        }

        if (mNextLegsAnim.isButtonDown())
        {
            ++mCurrentLegsAnim;
            if (mCurrentLegsAnim >= LEG_ANIM_COUNT)
            {
                mCurrentLegsAnim = 0;
            }
        }

        if (mStartAnim.isButtonDown())
        {
            mModel->setLowerAnimation(legAnims[mCurrentLegsAnim]);
            mModel->setUpperAnimation(torsoAnims[mCurrentTorsoAnim]);
        }

        if (mNextModel.isButtonDown())
        {
            ++mCurrentModel;
            if (mCurrentModel >= MODEL_COUNT)
            {
                mCurrentModel = 0;
            }
                
            GE_DELETE mWorld;
            mWorld = 0;
            GE_DELETE mModel;
            mModel = 0;
            init();
        }
    }
}

//------------------------------------------------------------------------------
void Md3Context::render()
//------------------------------------------------------------------------------
{
    if (mModel == 0)
    {
        theRenderer().print(TEXT_X_POS, TEXT_Y_POS,
            "Error loading .md3 model. Configure q3a variable in 'test.set'.");

        return;
    }

    if (mWorld != 0)
    {
        theRenderer().render(*mWorld);

        if (mModel != 0)
        {
            theRenderer().print(TEXT_X_POS, TEXT_Y_POS,
                ( std::string("Legs anim: ") + legAnims[mCurrentLegsAnim] +
                  "\nTorso anim: " + torsoAnims[mCurrentTorsoAnim]).c_str() );
        }
    }
}

//------------------------------------------------------------------------------
void Md3Context::exit()
//------------------------------------------------------------------------------
{
    GE_DELETE mModel;
    GE_DELETE mWorld;
}

} // namespace Ge
