#include "Precompiled.h"
//------------------------------------------------------------------------------
// (c) Aug-2002 Gottfried Chen
//------------------------------------------------------------------------------

#include "ParticleContext.h"

#include <System/FilePath.h>
#include <System/FileSystem.h>

#include <Math/Tools.h>

#include <Engine/TexturePool.h>
#include <Engine/Renderer.h>

#include <Application/Settings.h>

namespace
{

using namespace Ge;

//------------------------------------------------------------------------------
void initFire(Emitter& e)
//------------------------------------------------------------------------------
{
    FilePath path(theFileSystem().getApplicationPath());
    path += theSettings.getVariable("data").asString();
    path += "/Textures/fire.tga";
    Material* material = GE_NEW Material;
    material->set(0, *(theTexturePool().get(path.get().c_str())));
    e.setPosition(Vector3(-2,0,0));
    e.set(*material);
    e.setSpeed(Vector3(0,1,0));
    e.setSpread(degToRad(20));
    e.set(Color(1,1,0,1), Color(1,0,0,0.5f));
    e.setSize(1.0f, 0.8f);
}


//------------------------------------------------------------------------------
void initSmoke(Emitter& e)
//------------------------------------------------------------------------------
{
    FilePath path(theFileSystem().getApplicationPath());
    path += theSettings.getVariable("data").asString();
    path += "/Textures/smoke.tga";
    Material* material = GE_NEW Material;
    material->set(0, *(theTexturePool().get(path.get().c_str())));
    e.setPosition(Vector3(2,0,0));
    e.set(*material);
    e.setSpeed(Vector3(0,1.5f,0));
    e.setSpread(degToRad(5));
    e.set(Color(0.2f,0.2f,0.2f,0.8f), Color(0.9f,0.9f,0.9f,0.0f));
    e.setSize(0.8f, 2.0f);
    e.setAngle(0.0f, 1.0f);
}


} // namespace






namespace Ge
{

//------------------------------------------------------------------------------
ParticleContext::ParticleContext() :
//------------------------------------------------------------------------------
Context("ParticleContext"),
mCamera(*Locator::create(Matrix3x4::IDENTITY())),
mCameraController(mCamera),
mFire(20,1.0f),
mSmoke(5,3.0f)
{
    mCamera.getLocator().get().setColumn(MatrixBase::TRANSLATION, Vector3(0,0,-5));
    mCamera.getLocator().update();
    mCameraController.setDistance(5);
    mCameraController.recalculate();
}

//------------------------------------------------------------------------------
void ParticleContext::init()
//------------------------------------------------------------------------------
{
    initFire(mFire);
    initSmoke(mSmoke);
}

//------------------------------------------------------------------------------
void ParticleContext::update()
//------------------------------------------------------------------------------
{
    mCameraController.update();
    mFire.update();
    mSmoke.update();
}

//------------------------------------------------------------------------------
void ParticleContext::render()
//------------------------------------------------------------------------------
{
    theRenderer().set(mCamera);
    theRenderer().render(mFire);
    theRenderer().render(mSmoke);
}

//------------------------------------------------------------------------------
void ParticleContext::exit()
//------------------------------------------------------------------------------
{
}


} // namespace Ge

