#include "Precompiled.h"
#include "TestFramework.h"
#include "MfcWindow.h"

#include <Engine/DisplayStatistics.h>

#include <DirectXRenderer/DirectXRenderer.h>

#include <Application/Settings.h>
#include <Application/Context.h>
#include <Application/Metronome.h>
#include <Application/ScaledMetronome.h>


namespace Ge
{

TestFramework theFramework;

//------------------------------------------------------------------------------
TestFramework::TestFramework() :
//------------------------------------------------------------------------------
Framework("Test.set"),
mMfcWindow(0),
mNextContext(Keyboard::ESCAPE),
mModifier(Keyboard::LEFT_SHIFT),
mPreviousContext(mNextContext, mModifier)
{
}

//------------------------------------------------------------------------------
TestFramework::~TestFramework()
//------------------------------------------------------------------------------
{
    if (mMfcWindow != 0)
    {
        GE_DELETE mMfcWindow;
    }
}

//------------------------------------------------------------------------------
void TestFramework::init()
//------------------------------------------------------------------------------
{
    Framework::init();

    // Init all the contexts.
    mBumpContext.init();
    mContexts.push_back(&mBumpContext);

    mMd3.init();
    mContexts.push_back(&mMd3);

    mBumpMd3.init();
    mContexts.push_back(&mBumpMd3);

    mUnitTest.init();
    mContexts.push_back(&mUnitTest);

    mParticles.init();
    mContexts.push_back(&mParticles);
    
    mShadowVolume.init();
    mContexts.push_back(&mShadowVolume);

    mLws.init();
    mContexts.push_back(&mLws);

    mTerrain.init();
    mContexts.push_back(&mTerrain);

    mChoppyWaves.init();
    mContexts.push_back(&mChoppyWaves);

    mWater2D.init();
    mContexts.push_back(&mWater2D);

    mNewave.init();
    mContexts.push_back(&mNewave);

    mGerstner.init();
    mContexts.push_back(&mGerstner);

    mCurrentContext = 0;
    setActiveContext(*mContexts[mCurrentContext]);

    // Need this so that initial deltas are not too big due to the init delay.
    theMetronome().update();
    theScaledMetronome().update();
}

//------------------------------------------------------------------------------
void TestFramework::exit()
//------------------------------------------------------------------------------
{
    Framework::exit();

    mBumpMd3.exit();
    mBumpContext.exit();
    mMd3.exit();
    mUnitTest.exit();
    mParticles.exit();
    mShadowVolume.exit();
    mLws.exit();
    mTerrain.exit();
    mWater2D.exit();
    mNewave.exit();
    mGerstner.exit();
    mChoppyWaves.exit();
}

//------------------------------------------------------------------------------
Handle TestFramework::createWindow()
//------------------------------------------------------------------------------
{
    mMfcWindow = 
        GE_NEW MfcWindow(
            theSettings.getVariable("windowName").asString().c_str(),
            40,
            60,
            theSettings.getVariable("resolutionX").asFloat(),
            theSettings.getVariable("resolutionY").asFloat());

    return mMfcWindow->m_hWnd;
}



//------------------------------------------------------------------------------
MfcWindow* TestFramework::getMfcWindow()
//------------------------------------------------------------------------------
{
    return mMfcWindow;
}
 
//------------------------------------------------------------------------------
void TestFramework::update()
//------------------------------------------------------------------------------
{
    Framework::update();
    
    mNextContext.update();
    mPreviousContext.update();

    if (mNextContext.isButtonDown())
    {
        ++mCurrentContext;
        if (mCurrentContext == mContexts.size())
        {
            mCurrentContext = 0;
        }

        setActiveContext(*mContexts[mCurrentContext]);
    }

    if (mPreviousContext.isButtonDown())
    {
        --mCurrentContext;
        if (mCurrentContext < 0)
        {
            mCurrentContext = mContexts.size()-1;
        }

        setActiveContext(*mContexts[mCurrentContext]);
    }
}



} // namespace Ge
