#include "Precompiled.h"
//------------------------------------------------------------------------------
// (c) Jun-2002 Gottfried Chen
//------------------------------------------------------------------------------

#include "WaterContextBase.h"

#include <Engine/Renderer.h>

namespace Ge
{

//------------------------------------------------------------------------------
WaterContextBase::WaterContextBase(const char* name,
                                   Float waterHeight, Float waterHeightDelta) :
//------------------------------------------------------------------------------
Context(name),
mCamera(*Locator::create(Matrix3x4::IDENTITY())),
mCameraController(mCamera),
mWaterHeight(waterHeight),
mWaterHeightDelta(waterHeightDelta),
mIncHeight(Keyboard::W),
mDecHeight(Keyboard::S),
mRepIncHeight(mIncHeight),
mRepDecHeight(mDecHeight)
{
}

//------------------------------------------------------------------------------
void WaterContextBase::init()
//------------------------------------------------------------------------------
{
    mWorld.setCamera(&mCamera);
    mLight = GE_NEW Light;

    Vector3 cameraPos(0,18,-24);
    mCameraController.setDistance(cameraPos.length());
    makeLookAt(mCamera.getLocator().get(), cameraPos, Vector3::ORIGIN(), Vector3(0,1,0));
    mCamera.getLocator().update();
    mCameraController.recalculate();

    mLight->setPosition(Vector4(0, 18, -24, 0));
}


//------------------------------------------------------------------------------
const World& WaterContextBase::getWorld() const
//------------------------------------------------------------------------------
{
    return mWorld;
}

//------------------------------------------------------------------------------
World& WaterContextBase::getWorld()
//------------------------------------------------------------------------------
{
    return mWorld;
}

//------------------------------------------------------------------------------
Light& WaterContextBase::getLight()
//------------------------------------------------------------------------------
{
    return *mLight;
}

//------------------------------------------------------------------------------
const Camera& WaterContextBase::getCamera() const
//------------------------------------------------------------------------------
{
    return mCamera;
}

//------------------------------------------------------------------------------
void WaterContextBase::update()
//------------------------------------------------------------------------------
{
    updateHeight();
    mCameraController.update();
}

//------------------------------------------------------------------------------
void WaterContextBase::render()
//------------------------------------------------------------------------------
{
    theRenderer().render(getWorld());
}


//------------------------------------------------------------------------------
void WaterContextBase::updateHeight()
//------------------------------------------------------------------------------
{
    mRepIncHeight.update();
    mRepDecHeight.update();
    if (mRepDecHeight.isButtonDown())
    {
        mWaterHeight -= mWaterHeightDelta;
        mWaterHeight = maximum(0.0f, mWaterHeight);
    }
    else if (mRepIncHeight.isButtonDown())
    {
        mWaterHeight += mWaterHeightDelta;
    }
}
    
//------------------------------------------------------------------------------
Float WaterContextBase::getWaterHeight()
//------------------------------------------------------------------------------
{
    return mWaterHeight;
}

} // namespace Ge
